"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _warn = _interopRequireDefault(require("../utils/dev/warn"));

var _isObject = _interopRequireDefault(require("../utils/object/isObject"));

var _wrapInObject = _interopRequireDefault(require("../utils/object/wrapInObject"));

var _resolveOptions = _interopRequireDefault(require("./helpers/resolveOptions"));

var _configuration = require("../configuration");

var _fetchItem = _interopRequireDefault(require("../actions/RESTful/fetchItem"));

var _fetchList = _interopRequireDefault(require("../actions/RESTful/fetchList"));

var _newItem = _interopRequireDefault(require("../actions/RESTful/newItem"));

var _clearNewItem = _interopRequireDefault(require("../actions/RESTful/clearNewItem"));

var _editNewItem = _interopRequireDefault(require("../actions/RESTful/editNewItem"));

var _editNewOrExistingItem = _interopRequireDefault(require("../actions/RESTful/editNewOrExistingItem"));

var _createItem = _interopRequireDefault(require("../actions/RESTful/createItem"));

var _editItem = _interopRequireDefault(require("../actions/RESTful/editItem"));

var _clearItemEdit = _interopRequireDefault(require("../actions/RESTful/clearItemEdit"));

var _updateItem = _interopRequireDefault(require("../actions/RESTful/updateItem"));

var _destroyItem = _interopRequireDefault(require("../actions/RESTful/destroyItem"));

var _selectItem = _interopRequireDefault(require("../actions/selection/selectItem"));

var _selectAnotherItem = _interopRequireDefault(require("../actions/selection/selectAnotherItem"));

var _deselectItem = _interopRequireDefault(require("../actions/selection/deselectItem"));

var _clearSelectedItems = _interopRequireDefault(require("../actions/selection/clearSelectedItems"));

var _clearResource = _interopRequireDefault(require("../actions/clear/clearResource"));

var _clearItem = _interopRequireDefault(require("../actions/clear/clearItem"));

var _clearList = _interopRequireDefault(require("../actions/clear/clearList"));

var _without = _interopRequireDefault(require("../utils/list/without"));

var _ResourcesOnlyActionsDictionary = _interopRequireDefault(require("../constants/ResourcesOnlyActionsDictionary"));

var _pluck = _interopRequireDefault(require("../utils/list/pluck"));

var _isFunction = _interopRequireDefault(require("../utils/object/isFunction"));

var _isString = _interopRequireDefault(require("../utils/string/isString"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var STANDARD_ACTION_CREATORS = {
  fetchList: _fetchList["default"].actionCreator,
  fetchItem: _fetchItem["default"].actionCreator,
  newItem: _newItem["default"].actionCreator,
  clearNewItem: _clearNewItem["default"].actionCreator,
  editNewItem: _editNewItem["default"].actionCreator,
  createItem: _createItem["default"].actionCreator,
  editItem: _editItem["default"].actionCreator,
  editNewOrExistingItem: _editNewOrExistingItem["default"].actionCreator,
  clearItemEdit: _clearItemEdit["default"].actionCreator,
  updateItem: _updateItem["default"].actionCreator,
  destroyItem: _destroyItem["default"].actionCreator,
  clearItem: _clearItem["default"].actionCreator,
  clearList: _clearList["default"].actionCreator,
  clearResource: _clearResource["default"].actionCreator,
  selectItem: _selectItem["default"].actionCreator,
  selectAnotherItem: _selectAnotherItem["default"].actionCreator,
  deselectItem: _deselectItem["default"].actionCreator,
  clearSelectedItems: _clearSelectedItems["default"].actionCreator
};

var LOCAL_ONLY_ACTION_CREATORS = _objectSpread(_objectSpread({}, STANDARD_ACTION_CREATORS), {}, {
  fetchList: _fetchList["default"].localActionCreator,
  fetchItem: _fetchItem["default"].localActionCreator,
  createItem: _createItem["default"].localActionCreator,
  updateItem: _updateItem["default"].localActionCreator,
  destroyItem: _destroyItem["default"].localActionCreator
});

var SINGULAR_ACTION_CREATORS = _objectSpread({}, (0, _without["default"])(STANDARD_ACTION_CREATORS, Object.keys(_ResourcesOnlyActionsDictionary["default"])));

function getActionCreators(_ref) {
  var localOnly = _ref.localOnly,
      singular = _ref.singular;

  if (localOnly) {
    if (singular) {
      return (0, _pluck["default"])(LOCAL_ONLY_ACTION_CREATORS, Object.keys(SINGULAR_ACTION_CREATORS));
    }

    return LOCAL_ONLY_ACTION_CREATORS;
  } else if (singular) {
    return SINGULAR_ACTION_CREATORS;
  }

  return STANDARD_ACTION_CREATORS;
}

function buildActionCreators(resourceOptions, actions, actionsOptions) {
  var name = resourceOptions.name;
  var effectiveActionCreators = getActionCreators(resourceOptions);
  return Object.keys(actionsOptions).reduce(function (memo, key) {
    if (resourceOptions.singular && _ResourcesOnlyActionsDictionary["default"][key]) {
      return memo;
    }

    var actionOptions = (0, _wrapInObject["default"])(actionsOptions[key]);
    var actionCreator = (0, _isObject["default"])(actionOptions) && actionOptions.actionCreator;
    var standardActionCreator = (0, _isString["default"])(actionCreator) ? effectiveActionCreators[actionCreator] : effectiveActionCreators[key];

    if ((0, _isFunction["default"])(actionCreator)) {
      memo[key] = actionCreator;
    } else if (standardActionCreator) {
      var actionOptionsMergedWithResourceOptions = (0, _resolveOptions["default"])(resourceOptions, actionOptions, ['url', 'keyBy', 'urlOnlyParams', 'responseAdaptor', 'requestAdaptor', 'progress', 'requestErrorHandler', 'metadata', 'method', 'localOnly', 'singular']);

      var requestOptions = _objectSpread(_objectSpread({}, resourceOptions.request || {}), actionOptions.request || {});

      var actionConfiguration = _objectSpread(_objectSpread({
        action: actions[key],
        name: name
      }, actionOptionsMergedWithResourceOptions), {}, {
        request: requestOptions
      });

      memo[key] = function (arg1, arg2, arg3) {
        var globalConfiguration = (0, _configuration.getConfiguration)();

        var reloadedOptions = _objectSpread(_objectSpread(_objectSpread({}, globalConfiguration), actionConfiguration), {}, {
          request: _objectSpread(_objectSpread({}, globalConfiguration.request), actionConfiguration.request || {})
        });

        return standardActionCreator(reloadedOptions, arg1, arg2, arg3);
      };
    } else {
      (0, _warn["default"])("'".concat(key, "' must match the list of standard action creators (").concat(Object.keys(STANDARD_ACTION_CREATORS).join(', '), ") or define an 'actionCreator' option. Check the options for ").concat(name));
    }

    return memo;
  }, {});
}

var _default = buildActionCreators;
exports["default"] = _default;