"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _isObject = _interopRequireDefault(require("../../utils/object/isObject"));

var _requestProgress = _interopRequireDefault(require("../requestProgress"));

var _ProgressDirections = require("../../constants/ProgressDirections");

var _without = _interopRequireDefault(require("../../utils/list/without"));

var _NetworkStatuses = require("../../constants/NetworkStatuses");

var _RequestManager = require("../../utils/RequestManager");

var _pluck = _interopRequireDefault(require("../../utils/list/pluck"));

var _normalizeErrors = _interopRequireDefault(require("./normalizeErrors"));

var _isString = _interopRequireDefault(require("../../utils/string/isString"));

var _configuration = require("../../configuration");

var _isEmpty = _interopRequireDefault(require("../../utils/list/isEmpty"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function makeRequest(options) {
  var actionCreatorOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var url = options.url,
      request = options.request,
      credentials = options.credentials,
      progress = options.progress,
      responseAdaptor = options.responseAdaptor,
      dispatch = options.dispatch,
      onSuccess = options.onSuccess,
      onError = options.onError,
      _options = _objectWithoutProperties(options, ["url", "request", "credentials", "progress", "responseAdaptor", "dispatch", "onSuccess", "onError"]);

  var actionCreatorRequestOptions = actionCreatorOptions.request || {};

  var _request = _objectSpread(_objectSpread({
    credentials: credentials
  }, request || {}), actionCreatorRequestOptions);

  var globalConfig = (0, _configuration.getConfiguration)();

  var requestOptions = function () {
    var common = _objectSpread(_objectSpread({}, (0, _without["default"])(_request, ['errorHandler', 'cookie', 'credentials'])), {}, {
      headers: _objectSpread(_objectSpread({
        'Accept': globalConfig.acceptType || globalConfig.contentType,
        'Content-Type': globalConfig.contentType
      }, _request.headers), actionCreatorRequestOptions.headers || {})
    });

    if (_request.cookie) {
      common.headers.Cookie = _request.cookie;
    } else if (_request.credentials) {
      common.credentials = _request.credentials;
    }

    return common;
  }();

  var processResponse = function processResponse(response) {
    var status = response.status;

    if (status < 400) {
      return response.text().then(function (text) {
        var json = text.length ? JSON.parse(text) : {};

        var _json = function () {
          if (responseAdaptor) {
            var _responseAdaptor = responseAdaptor(json, response),
                values = _responseAdaptor.values,
                error = _responseAdaptor.error,
                errors = _responseAdaptor.errors,
                metadata = _responseAdaptor.metadata;

            return _objectSpread(_objectSpread({
              values: values
            }, (0, _normalizeErrors["default"])(error, errors)), {}, {
              metadata: metadata
            });
          } else {
            if ((0, _isObject["default"])(json) && !(0, _isEmpty["default"])(json)) {
              var _error = json.error,
                  _errors = json.errors,
                  _values = _objectWithoutProperties(json, ["error", "errors"]);

              return _objectSpread({
                values: _values
              }, (0, _normalizeErrors["default"])(_error, _errors));
            } else {
              return {
                values: json
              };
            }
          }
        }();

        if (_json.error || _json.errors) {
          return dispatch(onError(_options, actionCreatorOptions, status, (0, _pluck["default"])(_json, ['error', 'errors']), _json.metadata));
        } else {
          return dispatch(onSuccess(_options, actionCreatorOptions, _json.values, _json.metadata, status));
        }
      });
    } else {
      if (_request.errorHandler) {
        _request.errorHandler(response, function (errorOrErrors, metadata) {
          dispatch(onError(_options, actionCreatorOptions, status, (0, _normalizeErrors["default"])(errorOrErrors), metadata));
        });
      } else {
        if (response.headers.get('Content-Type').startsWith(globalConfig.errorContentType || globalConfig.contentType)) {
          return response.text().then(function (text) {
            var json = text.length ? JSON.parse(text) : {};
            var normalizedError = (0, _isString["default"])(json.error) ? {
              message: json.error
            } : json.error;
            return dispatch(onError(_options, actionCreatorOptions, status, (0, _normalizeErrors["default"])(normalizedError, json.errors)));
          });
        } else {
          return response.text().then(function (message) {
            return dispatch(onError(_options, actionCreatorOptions, status, (0, _normalizeErrors["default"])({
              message: message
            })));
          });
        }
      }
    }
  };

  if (progress) {
    return new Promise(function (resolve, reject) {
      var xhRequest = new XMLHttpRequest();

      if (requestOptions.credentials === 'include') {
        xhRequest.withCredentials = true;
      }

      xhRequest.open(requestOptions.method || 'GET', url);
      Object.keys(requestOptions.headers || {}).forEach(function (headerKey) {
        xhRequest.setRequestHeader(headerKey, requestOptions.headers[headerKey]);
      });

      xhRequest.upload.onprogress = function (event) {
        dispatch((0, _requestProgress["default"])(_options, _objectSpread(_objectSpread({}, event), {}, {
          direction: _ProgressDirections.UP
        })));
      };

      xhRequest.upload.onloadend = function (event) {
        dispatch((0, _requestProgress["default"])(_options, _objectSpread(_objectSpread({}, event), {}, {
          direction: _ProgressDirections.UP
        })));
      };

      xhRequest.onprogress = function (event) {
        dispatch((0, _requestProgress["default"])(_options, _objectSpread(_objectSpread({}, event), {}, {
          direction: _ProgressDirections.DOWN
        })));
      };

      xhRequest.onload = function (_ref) {
        var target = _ref.target;
        var responseBlob = new Blob([target.responseText], {
          type: target.responseType || 'application/json'
        });
        var response = new Response(responseBlob, {
          status: target.status,
          statusText: target.statusText
        });
        return processResponse(response).then(resolve);
      };

      xhRequest.onerror = function (error) {
        dispatch(onError(_options, actionCreatorOptions, 0, (0, _normalizeErrors["default"])({
          type: _NetworkStatuses.CLIENT_ERROR,
          name: error.name,
          message: error.message,
          raw: error
        }))).then(reject);
      };

      xhRequest.send(requestOptions.body);
    })["catch"](function (error) {
      throw error;
    })["finally"](function () {
      return (0, _RequestManager.registerRequestEnd)(request.method, url);
    });
  } else {
    return fetch(url, requestOptions).then(processResponse)["catch"](function (error) {
      return dispatch(onError(_options, actionCreatorOptions, 0, (0, _normalizeErrors["default"])({
        type: _NetworkStatuses.CLIENT_ERROR,
        name: error.name,
        message: error.message,
        raw: error
      })));
    })["finally"](function () {
      return (0, _RequestManager.registerRequestEnd)(request.method, url);
    });
  }
}

var _default = makeRequest;
exports["default"] = _default;