"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _constantize = _interopRequireDefault(require("../utils/string/constantize"));

var _toPlural = _interopRequireDefault(require("../utils/string/toPlural"));

var _toSingular = _interopRequireDefault(require("../utils/string/toSingular"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var STANDARD_ACTIONS = {
  fetchList: 'FETCH_ITEMS',
  fetchItem: 'FETCH_ITEM',
  newItem: 'NEW_ITEM',
  clearNewItem: 'CLEAR_NEW_ITEM',
  editNewItem: 'EDIT_NEW_ITEM',
  editNewOrExistingItem: 'EDIT_NEW_OR_EXISTING_ITEM',
  clearItemEdit: 'CLEAR_ITEM_EDIT',
  createItem: 'CREATE_ITEM',
  editItem: 'EDIT_ITEM',
  updateItem: 'UPDATE_ITEM',
  destroyItem: 'DESTROY_ITEM',
  clearItem: 'CLEAR_ITEM',
  clearList: 'CLEAR_ITEMS',
  clearResource: 'CLEAR_ALL_ITEMS',
  selectItem: 'SELECT_ITEM',
  selectAnotherItem: 'SELECT_ANOTHER_ITEM',
  deselectItem: 'DESELECT_ITEM',
  clearSelectedItems: 'CLEAR_SELECTED_ITEMS'
};

function replaceItem(actionName, resourceName) {
  if (actionName.indexOf('ITEMS') !== -1) {
    return actionName.replace('ITEMS', (0, _constantize["default"])((0, _toPlural["default"])(resourceName)));
  } else if (actionName.indexOf('ITEM') !== -1) {
    return actionName.replace('ITEM', (0, _constantize["default"])((0, _toSingular["default"])(resourceName)));
  }
}

function getActionName(resourceName, actionAlias) {
  var standardAction = STANDARD_ACTIONS[actionAlias];

  if (standardAction) {
    return replaceItem(standardAction, resourceName);
  } else {
    var constantizeActionName = (0, _constantize["default"])(actionAlias);
    var actionNameSubstitutingResourceNameForItem = replaceItem(constantizeActionName, resourceName);

    if (actionNameSubstitutingResourceNameForItem) {
      return actionNameSubstitutingResourceNameForItem;
    }

    return "".concat(constantizeActionName, "_").concat((0, _constantize["default"])(resourceName));
  }
}

function buildActions(name, resourceOptions) {
  var actionsConfiguration = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var actionsMap = {};
  Object.keys(actionsConfiguration).forEach(function (action) {
    var actionConfiguration = actionsConfiguration[action];
    actionsMap[action] = actionConfiguration && actionConfiguration.actionName || getActionName(name, action);
  });
  return actionsMap;
}

var _default = buildActions;
exports["default"] = _default;