"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _getItemKey = _interopRequireDefault(require("../../action-creators/helpers/getItemKey"));

var _generateUrl = _interopRequireDefault(require("../../action-creators/helpers/generateUrl"));

var _wrapInObject = _interopRequireDefault(require("../../utils/object/wrapInObject"));

var _extractListOperations = _interopRequireDefault(require("../../action-creators/helpers/extractListOperations"));

var _makeRequest = _interopRequireDefault(require("../../action-creators/helpers/makeRequest"));

var _DataStructures = require("../../constants/DataStructures");

var _Statuses = require("../../constants/Statuses");

var _assertInDevMode = _interopRequireDefault(require("../../utils/assertInDevMode"));

var _warn = _interopRequireDefault(require("../../utils/dev/warn"));

var _applyListOperators = _interopRequireDefault(require("../../reducers/helpers/applyListOperators"));

var _without = _interopRequireDefault(require("../../utils/list/without"));

var _replace = _interopRequireDefault(require("../../utils/list/replace"));

var _applyTransforms = _interopRequireDefault(require("../../reducers/helpers/applyTransforms"));

var _getItem = _interopRequireDefault(require("../../utils/getItem"));

var _isUndefined = _interopRequireDefault(require("../../utils/isUndefined"));

var _mergeStatus = _interopRequireDefault(require("../../reducers/helpers/mergeStatus"));

var _RequestManager = require("../../utils/RequestManager");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

var _isNew = _interopRequireDefault(require("../../public-helpers/isNew"));

var _adaptOptionsForSingularResource = _interopRequireDefault(require("../../action-creators/helpers/adaptOptionsForSingularResource"));

var _arrayFrom = _interopRequireDefault(require("../../utils/array/arrayFrom"));

var _toPlural = _interopRequireDefault(require("../../utils/string/toPlural"));

var _listKeysForItemKeySubstitution = _interopRequireDefault(require("../../reducers/helpers/listKeysForItemKeySubstitution"));

var _getHttpStatusCode = _interopRequireDefault(require("../../public-helpers/getHttpStatusCode"));

var _configuration = require("../../configuration");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var HTTP_REQUEST_TYPE = 'POST';

function actionCreator(options, paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions) {
  var action = options.action,
      transforms = options.transforms,
      urlTemplate = options.url,
      urlOnlyParams = options.urlOnlyParams,
      keyBy = options.keyBy,
      progress = options.progress,
      metadata = options.metadata,
      requestAdaptor = options.requestAdaptor,
      _options$request = options.request,
      request = _options$request === void 0 ? {} : _options$request,
      singular = options.singular,
      _options$method = options.method,
      method = _options$method === void 0 ? HTTP_REQUEST_TYPE : _options$method;

  var _adaptOptionsForSingu = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: true,
    acceptsValues: true
  }, [paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu.params,
      values = _adaptOptionsForSingu.values,
      actionCreatorOptions = _adaptOptionsForSingu.actionCreatorOptions;

  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  var url = (0, _generateUrl["default"])({
    urlTemplate: urlTemplate,
    keyBy: keyBy,
    ignoreOptionalParams: true
  }, normalizedParams);

  if (actionCreatorOptions.force || (0, _RequestManager.isRequestInProgress)(method, url)) {
    return _nop["default"];
  } else {
    (0, _RequestManager.registerRequestStart)(method, url);
  }

  var key = function () {
    var specifiedKey = (0, _getItemKey["default"])([normalizedParams, values], {
      keyBy: keyBy,
      singular: singular
    });

    if (specifiedKey || specifiedKey === _EmptyKey["default"]) {
      return specifiedKey;
    } else {
      return (0, _configuration.getConfiguration)().generateId();
    }
  }();

  return function (dispatch) {
    var listOperations = (0, _extractListOperations["default"])(actionCreatorOptions, urlOnlyParams);
    var requestedAt = Date.now();
    dispatch(submitCreateResource({
      action: action,
      transforms: transforms,
      key: key,
      metadata: metadata,
      requestedAt: requestedAt
    }, actionCreatorOptions, values, listOperations));
    return (0, _makeRequest["default"])(_objectSpread(_objectSpread({}, options), {}, {
      params: normalizedParams,
      listOperations: listOperations,
      key: key,
      url: url,
      request: _objectSpread({
        method: method,
        body: JSON.stringify(requestAdaptor ? requestAdaptor(values) : values)
      }, request),
      dispatch: dispatch,
      onSuccess: receiveCreatedResource,
      onError: handleCreateResourceError,
      progress: progress
    }), actionCreatorOptions);
  };
}

function submitCreateResource(options, actionCreatorOptions, values, listOperations) {
  var transforms = options.transforms,
      action = options.action,
      key = options.key,
      requestedAt = options.requestedAt;
  var metadata = actionCreatorOptions.metadata || options.metadata;
  return {
    type: action,
    status: _Statuses.CREATING,
    temporaryKey: key,
    listOperations: listOperations,
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, _objectSpread(_objectSpread({}, _DataStructures.ITEM), {}, {
      values: values,
      status: {
        type: _Statuses.CREATING,
        requestedAt: requestedAt
      },
      metadata: metadata
    }))
  };
}

function localActionCreator(options, paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions) {
  var urlOnlyParams = options.urlOnlyParams,
      keyBy = options.keyBy;

  var _adaptOptionsForSingu2 = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: true,
    acceptsValues: true
  }, [paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu2.params,
      values = _adaptOptionsForSingu2.values,
      actionCreatorOptions = _adaptOptionsForSingu2.actionCreatorOptions;

  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  var listOperations = (0, _extractListOperations["default"])(actionCreatorOptions, urlOnlyParams);
  var metadata = actionCreatorOptions.metadata || options.metadata;
  return receiveCreatedResource(_objectSpread(_objectSpread({}, options), {}, {
    params: normalizedParams,
    listOperations: listOperations
  }), actionCreatorOptions, values, metadata);
}

function receiveCreatedResource(options, actionCreatorOptions, values, metadata, httpCode) {
  var action = options.action,
      keyBy = options.keyBy,
      transforms = options.transforms,
      params = options.params,
      listOperations = options.listOperations,
      localOnly = options.localOnly,
      singular = options.singular;

  var key = function () {
    var specifiedKey = (0, _getItemKey["default"])([values, params], {
      keyBy: keyBy,
      singular: singular
    });

    if ((0, _isUndefined["default"])(specifiedKey)) {
      (0, _assertInDevMode["default"])(function () {
        (0, _warn["default"])('createItem() did not specify a temporary key. One has been generated and stored in ' + 'newItemKey, but unless you save a reference to it, it will be lost when you create the next item. ' + 'It\'s therefore recommended when using the localOnly option to always specify a key by passing it ' + 'as the first argument to createItem.');
      });
      return (0, _configuration.getConfiguration)().generateId();
    } else {
      return specifiedKey;
    }
  }();

  return {
    type: action,
    status: _Statuses.SUCCESS,
    key: key,
    temporaryKey: options.key,
    listOperations: listOperations,
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, {
      values: values,
      status: {
        type: _Statuses.SUCCESS,
        syncedAt: Date.now(),
        httpCode: httpCode
      },
      metadata: metadata
    }),
    localOnly: localOnly
  };
}

function handleCreateResourceError(options, actionCreatorOptions, httpCode, errorEnvelope, metadata) {
  var action = options.action,
      key = options.key;
  return _objectSpread(_objectSpread({
    type: action,
    status: _Statuses.ERROR,
    temporaryKey: key,
    metadata: metadata,
    httpCode: httpCode
  }, errorEnvelope), {}, {
    errorOccurredAt: Date.now()
  });
}

function reducer(resources, action) {
  var localOnly = action.localOnly,
      type = action.type,
      temporaryKey = action.temporaryKey,
      key = action.key,
      _action$listOperation = action.listOperations,
      listOperations = _action$listOperation === void 0 ? {} : _action$listOperation,
      status = action.status,
      item = action.item,
      httpCode = action.httpCode,
      error = action.error,
      errors = action.errors,
      errorOccurredAt = action.errorOccurredAt,
      metadata = action.metadata;
  var items = resources.items;
  var currentItem = items[temporaryKey] || _DataStructures.ITEM;

  if (status === _Statuses.CREATING) {
    (0, _assertInDevMode["default"])(function () {
      if (!(0, _isNew["default"])(currentItem)) {
        (0, _warn["default"])("".concat(type, " has the same key '").concat(temporaryKey, "' as an existing item. ") + 'Use updateItem() to update an existing item, ' + 'or ensure the new item has a unique temporary key. (The create request was still sent to the server.)');
      }
    });

    var itemsToPersist = function () {
      if ((0, _getItem["default"])(resources, resources.newItemKey).status.type === _Statuses.NEW) {
        return (0, _without["default"])(items, resources.newItemKey);
      } else {
        return items;
      }
    }();

    var newItem = _objectSpread(_objectSpread(_objectSpread({}, currentItem), item), {}, {
      status: (0, _mergeStatus["default"])(currentItem.status, item.status, {
        onlyPersist: ['syncedAt']
      })
    });

    var newItems = _objectSpread(_objectSpread({}, itemsToPersist), {}, _defineProperty({}, temporaryKey, newItem));

    var newLists = (0, _applyListOperators["default"])(_objectSpread(_objectSpread({}, resources), {}, {
      items: newItems
    }), listOperations, temporaryKey);
    return _objectSpread(_objectSpread({}, resources), {}, {
      items: newItems,
      lists: newLists,
      newItemKey: temporaryKey
    });
  } else if (status === _Statuses.SUCCESS) {
    var _itemsToPersist = function () {
      if (localOnly && (0, _getItem["default"])(resources, resources.newItemKey).status.type === _Statuses.NEW) {
        return (0, _without["default"])(items, resources.newItemKey);
      } else {
        return (0, _without["default"])(items, temporaryKey);
      }
    }();

    var _newItem = _objectSpread(_objectSpread({}, item), {}, {
      values: (0, _getHttpStatusCode["default"])(item) === 204 ? currentItem.values : item.values,
      status: (0, _mergeStatus["default"])(currentItem.status, item.status),
      metadata: _objectSpread(_objectSpread({}, currentItem.metadata), item.metadata)
    });

    var _newItems = _objectSpread(_objectSpread({}, _itemsToPersist), {}, _defineProperty({}, key, _newItem));

    var _newLists = function () {
      if (localOnly) {
        return (0, _applyListOperators["default"])(_objectSpread(_objectSpread({}, resources), {}, {
          items: _newItems
        }), listOperations, key);
      } else {
        return _objectSpread(_objectSpread({}, resources.lists), (0, _listKeysForItemKeySubstitution["default"])(listOperations, resources.lists).reduce(function (memo, id) {
          var list = resources.lists[id] || _DataStructures.LIST;
          var positions = list.positions;
          memo[id] = _objectSpread(_objectSpread({}, list), {}, {
            positions: (0, _replace["default"])(positions, temporaryKey, key)
          });
          return memo;
        }, {}));
      }
    }();

    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _newItems,
      lists: _newLists,
      newItemKey: key
    });
  } else if (status === _Statuses.ERROR) {
    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _objectSpread(_objectSpread({}, items), {}, _defineProperty({}, temporaryKey, _objectSpread(_objectSpread({}, currentItem), {}, {
        status: (0, _mergeStatus["default"])(currentItem.status, {
          type: status,
          httpCode: httpCode,
          error: error,
          errors: errors,
          errorOccurredAt: errorOccurredAt
        }),
        metadata: _objectSpread(_objectSpread({}, currentItem.metadata), metadata)
      })))
    });
  } else {
    return resources;
  }
}

function hasManyAssociationsReducer(resources, _ref, _ref2) {
  var temporaryKey = _ref.temporaryKey,
      key = _ref.key,
      status = _ref.status,
      associationItem = _ref.item;
  var relationType = _ref2.relationType,
      foreignKeyName = _ref2.foreignKeyName,
      keyName = _ref2.keyName;
  var associationValues = associationItem.values;
  return _objectSpread(_objectSpread({}, resources), {}, {
    items: _objectSpread(_objectSpread({}, resources.items), (0, _arrayFrom["default"])(associationValues[foreignKeyName] || associationValues[(0, _toPlural["default"])(foreignKeyName)]).reduce(function (memo, itemKey) {
      var item = resources.items[itemKey];

      var newKeyValue = function () {
        var currentKeyValue = item.values[keyName];

        if (relationType === 'hasAndBelongsToMany') {
          if (status === _Statuses.CREATING) {
            return [].concat(_toConsumableArray(currentKeyValue || []), [temporaryKey]);
          } else if (status === _Statuses.SUCCESS) {
            return (0, _replace["default"])(currentKeyValue, temporaryKey, key);
          }
        } else {
          return status === _Statuses.CREATING ? temporaryKey : key;
        }
      }();

      memo[itemKey] = _objectSpread(_objectSpread({}, item), {}, {
        values: _objectSpread(_objectSpread({}, item.values), {}, _defineProperty({}, keyName, newKeyValue))
      });
      return memo;
    }, {}))
  });
}

var _default = {
  reducer: reducer,
  hasManyAssociationsReducer: hasManyAssociationsReducer,
  actionCreator: actionCreator,
  localActionCreator: localActionCreator
};
exports["default"] = _default;