"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _getItemKey = _interopRequireDefault(require("../../action-creators/helpers/getItemKey"));

var _generateUrl = _interopRequireDefault(require("../../action-creators/helpers/generateUrl"));

var _wrapInObject = _interopRequireDefault(require("../../utils/object/wrapInObject"));

var _makeRequest = _interopRequireDefault(require("../../action-creators/helpers/makeRequest"));

var _assertInDevMode = _interopRequireDefault(require("../../utils/assertInDevMode"));

var _warn = _interopRequireDefault(require("../../utils/dev/warn"));

var _Statuses = require("../../constants/Statuses");

var _DataStructures = require("../../constants/DataStructures");

var _removeItemsFromResources = _interopRequireDefault(require("../../reducers/helpers/removeItemsFromResources"));

var _applyTransforms = _interopRequireDefault(require("../../reducers/helpers/applyTransforms"));

var _mergeStatus = _interopRequireDefault(require("../../reducers/helpers/mergeStatus"));

var _RequestManager = require("../../utils/RequestManager");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _adaptOptionsForSingularResource = _interopRequireDefault(require("../../action-creators/helpers/adaptOptionsForSingularResource"));

var _isEmpty = _interopRequireDefault(require("../../utils/list/isEmpty"));

var _contains = _interopRequireDefault(require("../../utils/list/contains"));

var _without = _interopRequireDefault(require("../../utils/list/without"));

var _arrayFrom = _interopRequireDefault(require("../../utils/array/arrayFrom"));

var _toPlural = _interopRequireDefault(require("../../utils/string/toPlural"));

var _serializeKey2 = _interopRequireDefault(require("../../public-helpers/serializeKey"));

var _toSingular = _interopRequireDefault(require("../../utils/string/toSingular"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var HTTP_REQUEST_TYPE = 'DELETE';

function actionCreator(options, optionalParams, optionalActionCreatorOptions) {
  var action = options.action,
      keyBy = options.keyBy,
      urlTemplate = options.url,
      progress = options.progress,
      _options$request = options.request,
      request = _options$request === void 0 ? {} : _options$request,
      singular = options.singular,
      _options$method = options.method,
      method = _options$method === void 0 ? HTTP_REQUEST_TYPE : _options$method;

  var _adaptOptionsForSingu = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: singular,
    acceptsValues: false
  }, [optionalParams, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu.params,
      actionCreatorOptions = _adaptOptionsForSingu.actionCreatorOptions;

  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  var url = (0, _generateUrl["default"])({
    urlTemplate: urlTemplate
  }, normalizedParams);

  if (actionCreatorOptions.force || (0, _RequestManager.isRequestInProgress)(method, url)) {
    return _nop["default"];
  } else {
    (0, _RequestManager.registerRequestStart)(method, url);
  }

  var key = (0, _getItemKey["default"])(normalizedParams, {
    keyBy: keyBy,
    singular: singular
  });
  return function (dispatch) {
    var requestedAt = Date.now();
    dispatch(deleteResourceUpdate({
      action: action,
      key: key,
      requestedAt: requestedAt
    }, actionCreatorOptions));
    return (0, _makeRequest["default"])(_objectSpread(_objectSpread({}, options), {}, {
      key: key,
      requestedAt: requestedAt,
      url: url,
      request: _objectSpread({
        method: method
      }, request),
      dispatch: dispatch,
      onSuccess: removeResource,
      onError: handleDestroyItemError,
      progress: progress
    }), actionCreatorOptions);
  };
}

function deleteResourceUpdate(options, _ref) {
  var previousValues = _ref.previousValues;
  var action = options.action,
      key = options.key,
      requestedAt = options.requestedAt;
  return {
    type: action,
    status: _Statuses.DESTROYING,
    key: key,
    requestedAt: requestedAt,
    previousValues: previousValues
  };
}

function localActionCreator(options, optionalParams, optionalActionCreatorOptions) {
  var keyBy = options.keyBy,
      singular = options.singular;

  var _adaptOptionsForSingu2 = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: singular,
    acceptsValues: false
  }, [optionalParams, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu2.params,
      actionCreatorOptions = _adaptOptionsForSingu2.actionCreatorOptions;

  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  var key = (0, _getItemKey["default"])(normalizedParams, {
    keyBy: keyBy,
    singular: singular
  });
  return removeResource(_objectSpread(_objectSpread({}, options), {}, {
    key: key
  }), actionCreatorOptions);
}

function removeResource(options, _ref2) {
  var previousValues = _ref2.previousValues;
  var action = options.action,
      key = options.key,
      localOnly = options.localOnly;
  return {
    type: action,
    status: _Statuses.SUCCESS,
    key: key,
    previousValues: previousValues,
    localOnly: localOnly
  };
}

function handleDestroyItemError(options, actionCreatorOptions, httpCode, errorEnvelope) {
  var transforms = options.transforms,
      action = options.action,
      key = options.key;
  return {
    type: action,
    status: _Statuses.DESTROY_ERROR,
    key: key,
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, {
      status: _objectSpread(_objectSpread({
        type: _Statuses.DESTROY_ERROR,
        httpCode: httpCode
      }, errorEnvelope), {}, {
        errorOccurredAt: Date.now()
      })
    })
  };
}

function reducer(resources, action) {
  var type = action.type,
      status = action.status,
      requestedAt = action.requestedAt,
      key = action.key,
      item = action.item;
  (0, _assertInDevMode["default"])(function () {
    if (!resources.items[key]) {
      (0, _warn["default"])("".concat(type, "'s key '").concat(key, "' did not match any items in the store. (Destroy request was still sent to the server.)"));
    }
  });
  var currentItem = resources.items[key] || _DataStructures.ITEM;

  if (status === _Statuses.DESTROYING) {
    (0, _assertInDevMode["default"])(function () {
      if (currentItem.status.type === _Statuses.NEW) {
        (0, _warn["default"])("".concat(type, "'s key '").concat(key, "' matched a new item. Use clearNewItem() ") + 'to clear items that haven\'t been saved to the server. (Destroy request was still sent to the server.)');
      }

      if (currentItem.status.type === _Statuses.DESTROYING) {
        (0, _warn["default"])("".concat(type, "'s key '").concat(key, "' matched a new item that has a pending DESTROY action. (Duplicate destroyItem request was still sent to the server.)"));
      }
    });
    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _objectSpread(_objectSpread({}, resources.items), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, currentItem), {}, {
        status: (0, _mergeStatus["default"])(currentItem.status, {
          type: status,
          requestedAt: requestedAt
        }, {
          onlyPersist: ['syncedAt']
        })
      })))
    });
  } else if (status === _Statuses.SUCCESS) {
    return (0, _removeItemsFromResources["default"])(resources, key);
  } else if (status === _Statuses.DESTROY_ERROR) {
    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _objectSpread(_objectSpread({}, resources.items), {}, _defineProperty({}, key, _objectSpread(_objectSpread(_objectSpread({}, currentItem), item), {}, {
        status: (0, _mergeStatus["default"])(currentItem.status, item.status)
      })))
    });
  } else {
    return resources;
  }
}

function hasManyAssociationsReducer(resources, _ref3, _ref4) {
  var key = _ref3.key,
      type = _ref3.type,
      status = _ref3.status,
      previousValues = _ref3.previousValues;
  var dependent = _ref4.dependent,
      relationType = _ref4.relationType,
      foreignKeyName = _ref4.foreignKeyName,
      name = _ref4.name,
      keyName = _ref4.keyName,
      listParameter = _ref4.listParameter;

  if (status === _Statuses.SUCCESS) {
    var _resources = function () {
      if ((0, _isEmpty["default"])(previousValues)) {
        (0, _assertInDevMode["default"])(function () {
          (0, _warn["default"])("".concat(type, " did not specify any previous values. This makes updating '").concat(name, ".").concat(keyName, "' much less ") + 'efficient. Use the previousValues option for destroyItem() to specify these values.');
        });

        if (dependent === 'destroy') {
          var itemKeysWithDeletedAssociation = Object.keys(resources.items).filter(function (associationKey) {
            var item = resources.items[associationKey];
            return (0, _contains["default"])(item.values[keyName], associationKey, {
              stringifyFirst: true
            });
          });
          return (0, _removeItemsFromResources["default"])(resources, itemKeysWithDeletedAssociation);
        } else {
          return _objectSpread(_objectSpread({}, resources), {}, {
            items: Object.keys(resources.items).reduce(function (memo, itemKey) {
              var item = resources.items[itemKey];

              var newValues = function () {
                if (relationType === 'hasAndBelongsToMany') {
                  return _objectSpread(_objectSpread({}, item.values), {}, _defineProperty({}, keyName, (0, _without["default"])(item.values[keyName] || [], key)));
                } else {
                  return (0, _without["default"])(item.values, keyName);
                }
              }();

              memo[itemKey] = _objectSpread(_objectSpread({}, item), {}, {
                values: newValues
              });
              return memo;
            }, {})
          });
        }
      } else {
        var foreignKeys = (0, _arrayFrom["default"])(previousValues[foreignKeyName] || previousValues[(0, _toPlural["default"])(foreignKeyName)]);

        if (dependent === 'destroy') {
          return (0, _removeItemsFromResources["default"])(resources, foreignKeys);
        } else {
          return _objectSpread(_objectSpread({}, resources), {}, {
            items: _objectSpread(_objectSpread({}, resources.items), foreignKeys.reduce(function (memo, addedKey) {
              var item = resources.items[addedKey];

              if (item) {
                var newValues = function () {
                  if (relationType === 'hasAndBelongsToMany') {
                    return _objectSpread(_objectSpread({}, item.values), {}, _defineProperty({}, keyName, (0, _without["default"])(item.values[keyName] || [], key)));
                  } else {
                    return (0, _without["default"])(item.values, keyName);
                  }
                }();

                memo[addedKey] = _objectSpread(_objectSpread({}, item), {}, {
                  values: newValues
                });
              }

              return memo;
            }, {}))
          });
        }
      }
    }();

    if (listParameter === false) {
      return _resources;
    } else {
      var listParameterString = (0, _serializeKey2["default"])(_defineProperty({}, listParameter || (0, _toSingular["default"])(keyName), key));
      var lists = Object.keys(_resources.lists).reduce(function (memo, listKey) {
        if (!(0, _contains["default"])(listKey, listParameterString)) {
          memo[listKey] = _resources.lists[listKey];
        }

        return memo;
      }, {});
      return _objectSpread(_objectSpread({}, _resources), {}, {
        lists: lists
      });
    }
  } else {
    return resources;
  }
}

var _default = {
  reducer: reducer,
  hasManyAssociationsReducer: hasManyAssociationsReducer,
  actionCreator: actionCreator,
  localActionCreator: localActionCreator
};
exports["default"] = _default;