"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _getItemKey = _interopRequireDefault(require("../../action-creators/helpers/getItemKey"));

var _Statuses = require("../../constants/Statuses");

var _DataStructures = require("../../constants/DataStructures");

var _applyTransforms = _interopRequireDefault(require("../../reducers/helpers/applyTransforms"));

var _assertInDevMode = _interopRequireDefault(require("../../utils/assertInDevMode"));

var _warn = _interopRequireDefault(require("../../utils/dev/warn"));

var _wrapInObject = _interopRequireDefault(require("../../utils/object/wrapInObject"));

var _adaptOptionsForSingularResource = _interopRequireDefault(require("../../action-creators/helpers/adaptOptionsForSingularResource"));

var _mergeStatus = _interopRequireDefault(require("../../reducers/helpers/mergeStatus"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function actionCreator(options, paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions) {
  var action = options.action,
      transforms = options.transforms,
      keyBy = options.keyBy,
      singular = options.singular;

  var _adaptOptionsForSingu = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: singular,
    acceptsValues: true
  }, [paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu.params,
      values = _adaptOptionsForSingu.values,
      actionCreatorOptions = _adaptOptionsForSingu.actionCreatorOptions;

  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  var key = (0, _getItemKey["default"])(normalizedParams, {
    keyBy: keyBy,
    singular: singular
  });
  return {
    type: action,
    status: _Statuses.EDITING,
    key: key,
    singular: singular,
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, _objectSpread(_objectSpread({}, _DataStructures.ITEM), {}, {
      values: values,
      status: {
        type: _Statuses.EDITING
      }
    }))
  };
}

function reducer(resources, action) {
  var type = action.type,
      key = action.key,
      item = action.item,
      singular = action.singular;
  var items = resources.items;
  (0, _assertInDevMode["default"])(function () {
    if (!items[key]) {
      if (singular) {
        (0, _warn["default"])('Use newItem() to create a new item or check the arguments passed to editItem(). (A new item was ' + 'created to contain the edit.)');
      } else {
        (0, _warn["default"])("".concat(type, "'s key '").concat(key, "' does not match any items in the store. Use newItem() to create ") + 'a new item or check the arguments passed to editItem(). (A new item was created to contain the edit.)');
      }
    }
  });
  var currentItem = items[key] || _DataStructures.ITEM;

  if (currentItem.status.type === _Statuses.NEW) {
    (0, _assertInDevMode["default"])(function () {
      if (singular) {
        (0, _warn["default"])('Use a editItem() to edit new items that have not yet been saved to an external API. Update ignored.');
      } else {
        (0, _warn["default"])("".concat(type, "'s key '").concat(key, "' matches a NEW item. Use a editItem() to edit ") + 'new items that have not yet been saved to an external API. Update ignored.');
      }
    });
    return resources;
  } else {
    var newValues = _objectSpread(_objectSpread(_objectSpread({}, currentItem.values), item.values), {}, {
      status: (0, _mergeStatus["default"])(currentItem.status, item.status)
    });

    var newStatus = function () {
      if (currentItem.status.dirty) {
        return item.status;
      } else {
        return _objectSpread(_objectSpread({}, item.status), {}, {
          dirty: true,
          originalValues: _objectSpread({}, currentItem.values)
        });
      }
    }();

    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _objectSpread(_objectSpread({}, items), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, item), {}, {
        values: newValues,
        status: newStatus
      })))
    });
  }
}

var _default = {
  reducer: reducer,
  actionCreator: actionCreator
};
exports["default"] = _default;