"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _Statuses = require("../../constants/Statuses");

var _getItemKey = _interopRequireDefault(require("../../action-creators/helpers/getItemKey"));

var _applyTransforms = _interopRequireDefault(require("../../reducers/helpers/applyTransforms"));

var _assertInDevMode = _interopRequireDefault(require("../../utils/assertInDevMode"));

var _warn = _interopRequireDefault(require("../../utils/dev/warn"));

var _isNew = _interopRequireDefault(require("../../public-helpers/isNew"));

var _adaptOptionsForSingularResource = _interopRequireDefault(require("../../action-creators/helpers/adaptOptionsForSingularResource"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function actionCreator(options, paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions) {
  var action = options.action,
      transforms = options.transforms,
      keyBy = options.keyBy,
      singular = options.singular;

  var _adaptOptionsForSingu = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: true,
    acceptsValues: true
  }, [paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu.params,
      values = _adaptOptionsForSingu.values,
      actionCreatorOptions = _adaptOptionsForSingu.actionCreatorOptions;

  var key = (0, _getItemKey["default"])(params, {
    keyBy: keyBy,
    singular: singular
  });
  return {
    type: action,
    status: _Statuses.EDITING,
    key: key,
    singular: singular,
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, {
      values: values
    })
  };
}

function reducer(resources, action) {
  var type = action.type,
      key = action.key,
      item = action.item,
      singular = action.singular;
  var items = resources.items;

  var _key = key || resources.newItemKey;

  var currentItem = items[_key];

  if (!currentItem) {
    (0, _assertInDevMode["default"])(function () {
      if (singular) {
        (0, _warn["default"])('Use newItem() to create a new item first, or check the arguments passed to editItem(). Update ignored.');
      } else {
        (0, _warn["default"])("".concat(type, "'s key '").concat(_key, "' does not match any new items in the store. Use newItem() ") + 'to create a new item first, or check the arguments passed to editItem(). Update ignored.');
      }
    });
    return resources;
  } else if ((0, _isNew["default"])(currentItem)) {
    var newValues = _objectSpread(_objectSpread({}, currentItem.values), item.values);

    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _objectSpread(_objectSpread({}, items), {}, _defineProperty({}, _key, _objectSpread(_objectSpread({}, currentItem), {}, {
        values: newValues
      })))
    });
  } else {
    (0, _assertInDevMode["default"])(function () {
      (0, _warn["default"])("".concat(type, "'s key '").concat(key, "' matches a resource that is NOT new. Use a editItem() to edit ") + 'existing items. Update ignored.');
    });
    return resources;
  }
}

var _default = {
  reducer: reducer,
  actionCreator: actionCreator
};
exports["default"] = _default;