"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _getItemKey = _interopRequireDefault(require("../../action-creators/helpers/getItemKey"));

var _generateUrl = _interopRequireDefault(require("../../action-creators/helpers/generateUrl"));

var _makeRequest = _interopRequireDefault(require("../../action-creators/helpers/makeRequest"));

var _Statuses = require("../../constants/Statuses");

var _DataStructures = require("../../constants/DataStructures");

var _applyTransforms = _interopRequireDefault(require("../../reducers/helpers/applyTransforms"));

var _wrapInObject = _interopRequireDefault(require("../../utils/object/wrapInObject"));

var _mergeStatus = _interopRequireDefault(require("../../reducers/helpers/mergeStatus"));

var _RequestManager = require("../../utils/RequestManager");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _adaptOptionsForSingularResource = _interopRequireDefault(require("../../action-creators/helpers/adaptOptionsForSingularResource"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var HTTP_REQUEST_TYPE = 'GET';

function actionCreator(options, paramsOrActionCreatorOptions, optionalActionCreatorOptions) {
  var action = options.action,
      transforms = options.transforms,
      urlTemplate = options.url,
      keyBy = options.keyBy,
      progress = options.progress,
      _options$request = options.request,
      request = _options$request === void 0 ? {} : _options$request,
      singular = options.singular,
      metadata = options.metadata,
      _options$method = options.method,
      method = _options$method === void 0 ? HTTP_REQUEST_TYPE : _options$method;

  var _adaptOptionsForSingu = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: singular,
    acceptsValues: false
  }, [paramsOrActionCreatorOptions, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu.params,
      actionCreatorOptions = _adaptOptionsForSingu.actionCreatorOptions;

  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  var url = (0, _generateUrl["default"])({
    urlTemplate: urlTemplate
  }, normalizedParams);

  if (!actionCreatorOptions.force && (0, _RequestManager.isRequestInProgress)(method, url)) {
    return _nop["default"];
  } else {
    (0, _RequestManager.registerRequestStart)(method, url);
  }

  var key = (0, _getItemKey["default"])(normalizedParams, {
    keyBy: keyBy,
    singular: singular
  });
  return function (dispatch) {
    var requestedAt = Date.now();
    dispatch(requestResource({
      action: action,
      transforms: transforms,
      key: key,
      metadata: metadata,
      requestedAt: requestedAt
    }, actionCreatorOptions));
    return (0, _makeRequest["default"])(_objectSpread(_objectSpread({}, options), {}, {
      key: key,
      keyBy: keyBy,
      params: params,
      url: url,
      request: _objectSpread({
        method: method
      }, request),
      requestedAt: requestedAt,
      dispatch: dispatch,
      onSuccess: receiveResource,
      onError: handleResourceError,
      progress: progress
    }), actionCreatorOptions);
  };
}

function localActionCreator() {
  throw new Error('fetchItem is not supported with the localOnly option. User getOrFetchItem instead.');
}

function requestResource(options, actionCreatorOptions) {
  var transforms = options.transforms,
      action = options.action,
      key = options.key,
      requestedAt = options.requestedAt;
  var metadata = actionCreatorOptions.metadata || options.metadata || {};
  return {
    type: action,
    status: _Statuses.FETCHING,
    key: key,
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, _objectSpread(_objectSpread({}, _DataStructures.ITEM), {}, {
      values: {},
      status: {
        type: _Statuses.FETCHING,
        requestedAt: requestedAt
      },
      metadata: metadata
    }))
  };
}

function receiveResource(options, actionCreatorOptions, values) {
  var metadata = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var transforms = options.transforms,
      action = options.action,
      params = options.params,
      keyBy = options.keyBy,
      singular = options.singular;
  var item = (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, _objectSpread(_objectSpread({}, _DataStructures.ITEM), {}, {
    values: values,
    status: {
      type: _Statuses.SUCCESS,
      syncedAt: Date.now()
    },
    metadata: metadata
  }));
  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  return {
    type: action,
    status: _Statuses.SUCCESS,
    key: (0, _getItemKey["default"])([item.values, normalizedParams], {
      keyBy: keyBy,
      singular: singular
    }),
    item: item
  };
}

function handleResourceError(options, actionCreatorOptions, httpCode, errorEnvelope, metadata) {
  var action = options.action,
      key = options.key;
  return _objectSpread(_objectSpread({
    type: action,
    status: _Statuses.ERROR,
    metadata: metadata,
    httpCode: httpCode,
    key: key
  }, errorEnvelope), {}, {
    errorOccurredAt: Date.now()
  });
}

function reducer(resources, action) {
  var status = action.status,
      key = action.key,
      httpCode = action.httpCode,
      item = action.item,
      error = action.error,
      errors = action.errors,
      errorOccurredAt = action.errorOccurredAt,
      metadata = action.metadata;
  var currentItem = resources.items[key] || _DataStructures.ITEM;

  if (status === _Statuses.FETCHING) {
    var newStatus = (0, _mergeStatus["default"])(currentItem.status, item.status, {
      onlyPersist: ['syncedAt']
    });

    if (currentItem.status.type === _Statuses.SUCCESS) {
      return _objectSpread(_objectSpread({}, resources), {}, {
        items: _objectSpread(_objectSpread({}, resources.items), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, item), {}, {
          values: _objectSpread(_objectSpread({}, currentItem.values), item.values),
          status: newStatus
        })))
      });
    } else {
      return _objectSpread(_objectSpread({}, resources), {}, {
        items: _objectSpread(_objectSpread({}, resources.items), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, item), {}, {
          status: newStatus
        })))
      });
    }
  } else if (status === _Statuses.SUCCESS) {
    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _objectSpread(_objectSpread({}, resources.items), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, item), {}, {
        status: (0, _mergeStatus["default"])(currentItem.status, item.status),
        metadata: _objectSpread(_objectSpread({}, currentItem.metadata), item.metadata)
      })))
    });
  } else if (status === _Statuses.ERROR) {
    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _objectSpread(_objectSpread({}, resources.items), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, currentItem), {}, {
        status: (0, _mergeStatus["default"])(currentItem.status, {
          type: status,
          httpCode: httpCode,
          error: error,
          errors: errors,
          errorOccurredAt: errorOccurredAt
        }),
        metadata: _objectSpread(_objectSpread({}, currentItem.metadata), metadata)
      })))
    });
  } else {
    return resources;
  }
}

var _default = {
  reducer: reducer,
  actionCreator: actionCreator,
  localActionCreator: localActionCreator
};
exports["default"] = _default;