"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _DataStructures = require("../../constants/DataStructures");

var _Statuses = require("../../constants/Statuses");

var _getListKey = _interopRequireDefault(require("../../action-creators/helpers/getListKey"));

var _generateUrl = _interopRequireDefault(require("../../action-creators/helpers/generateUrl"));

var _makeRequest = _interopRequireDefault(require("../../action-creators/helpers/makeRequest"));

var _getItemKey = _interopRequireDefault(require("../../action-creators/helpers/getItemKey"));

var _applyTransforms = _interopRequireDefault(require("../../reducers/helpers/applyTransforms"));

var _wrapInObject = _interopRequireDefault(require("../../utils/object/wrapInObject"));

var _mergeStatus = _interopRequireDefault(require("../../reducers/helpers/mergeStatus"));

var _RequestManager = require("../../utils/RequestManager");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var HTTP_REQUEST_TYPE = 'GET';

function actionCreator(options, params) {
  var actionCreatorOptions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var action = options.action,
      urlTemplate = options.url,
      keyBy = options.keyBy,
      urlOnlyParams = options.urlOnlyParams,
      progress = options.progress,
      _options$request = options.request,
      request = _options$request === void 0 ? {} : _options$request,
      _options$method = options.method,
      method = _options$method === void 0 ? HTTP_REQUEST_TYPE : _options$method;
  var key = (0, _getListKey["default"])(params, {
    urlOnlyParams: urlOnlyParams
  });
  var url = (0, _generateUrl["default"])({
    urlTemplate: urlTemplate
  }, params);

  if (!actionCreatorOptions.force && (0, _RequestManager.isRequestInProgress)(method, url)) {
    return _nop["default"];
  } else {
    (0, _RequestManager.registerRequestStart)(method, url);
  }

  return function (dispatch) {
    var requestedAt = Date.now();
    var metadata = actionCreatorOptions.metadata || options.metadata || {};
    dispatch(requestList({
      action: action,
      metadata: metadata,
      requestedAt: requestedAt
    }, key));
    return (0, _makeRequest["default"])(_objectSpread(_objectSpread({}, options), {}, {
      key: key,
      keyBy: keyBy,
      params: params,
      url: url,
      requestedAt: requestedAt,
      dispatch: dispatch,
      request: _objectSpread({
        method: method
      }, request),
      onSuccess: receiveList,
      onError: handleListError,
      progress: progress
    }), actionCreatorOptions);
  };
}

function localActionCreator() {
  throw new Error('fetchList is not supported with the localOnly option. User getOrFetchList instead.');
}

function requestList(options, key) {
  var action = options.action,
      metadata = options.metadata,
      requestedAt = options.requestedAt;
  return {
    type: action,
    status: _Statuses.FETCHING,
    list: _objectSpread(_objectSpread({}, _DataStructures.LIST), {}, {
      status: {
        type: _Statuses.FETCHING,
        requestedAt: requestedAt
      },
      metadata: metadata
    }),
    key: key
  };
}

function receiveList(options, actionCreatorOptions, list, metadata) {
  var transforms = options.transforms,
      key = options.key,
      keyBy = options.keyBy,
      action = options.action,
      params = options.params,
      requestedAt = options.requestedAt,
      singular = options.singular;
  var positions = [];
  var syncedAt = Date.now();
  var items = list.reduce(function (memo, values) {
    var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
    var itemKey = (0, _getItemKey["default"])([values, normalizedParams], {
      keyBy: keyBy,
      singular: singular
    });
    positions.push(itemKey);
    memo[itemKey] = (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, _objectSpread(_objectSpread({}, _DataStructures.ITEM), {}, {
      values: values,
      status: {
        type: _Statuses.SUCCESS,
        requestedAt: requestedAt,
        syncedAt: syncedAt
      },
      metadata: actionCreatorOptions.itemsMetadata || options.metadata || {}
    }));
    return memo;
  }, {});
  return {
    type: action,
    status: _Statuses.SUCCESS,
    items: items,
    key: key,
    list: {
      positions: positions,
      status: {
        type: _Statuses.SUCCESS,
        syncedAt: syncedAt,
        itemsInLastResponse: Object.keys(items).length
      },
      metadata: metadata
    }
  };
}

function handleListError(options, actionCreatorOptions, httpCode, errorEnvelope, metadata) {
  var action = options.action,
      key = options.key;
  return _objectSpread(_objectSpread({
    type: action,
    status: _Statuses.ERROR,
    key: key
  }, errorEnvelope), {}, {
    httpCode: httpCode,
    errorOccurredAt: Date.now(),
    metadata: metadata
  });
}

function reducer(resources, action) {
  var status = action.status,
      items = action.items,
      key = action.key,
      httpCode = action.httpCode,
      list = action.list,
      error = action.error,
      errors = action.errors,
      errorOccurredAt = action.errorOccurredAt,
      _action$metadata = action.metadata,
      metadata = _action$metadata === void 0 ? {} : _action$metadata;
  var currentList = resources.lists[key] || _DataStructures.LIST;

  if (status === _Statuses.FETCHING) {
    return _objectSpread(_objectSpread({}, resources), {}, {
      lists: _objectSpread(_objectSpread({}, resources.lists), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, currentList), {}, {
        status: (0, _mergeStatus["default"])(currentList.status, list.status, {
          onlyPersist: ['syncedAt', 'itemsInLastResponse']
        }),
        metadata: list.metadata
      })))
    });
  } else if (status === _Statuses.SUCCESS) {
    var newItems = _objectSpread(_objectSpread({}, resources.items), items);

    var newList = _objectSpread(_objectSpread({}, resources.lists), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, list), {}, {
      status: (0, _mergeStatus["default"])(currentList.status, list.status),
      metadata: _objectSpread(_objectSpread({}, currentList.metadata), list.metadata)
    })));

    return _objectSpread(_objectSpread({}, resources), {}, {
      items: newItems,
      lists: newList
    });
  } else if (status === _Statuses.ERROR) {
    var newLists = _objectSpread(_objectSpread({}, resources.lists), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, currentList), {}, {
      status: (0, _mergeStatus["default"])(currentList.status, {
        type: status,
        httpCode: httpCode,
        error: error,
        errors: errors,
        errorOccurredAt: errorOccurredAt
      }, {
        exclude: ['itemsInLastResponse']
      }),
      metadata: _objectSpread(_objectSpread({}, currentList.metadata), metadata)
    })));

    return _objectSpread(_objectSpread({}, resources), {}, {
      lists: newLists
    });
  } else {
    return resources;
  }
}

var _default = {
  reducer: reducer,
  actionCreator: actionCreator,
  localActionCreator: localActionCreator
};
exports["default"] = _default;