"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _getItemKey = _interopRequireDefault(require("../../action-creators/helpers/getItemKey"));

var _DataStructures = require("../../constants/DataStructures");

var _Statuses = require("../../constants/Statuses");

var _applyTransforms = _interopRequireDefault(require("../../reducers/helpers/applyTransforms"));

var _extractListOperations = _interopRequireDefault(require("../../action-creators/helpers/extractListOperations"));

var _assertInDevMode = _interopRequireDefault(require("../../utils/assertInDevMode"));

var _warn = _interopRequireDefault(require("../../utils/dev/warn"));

var _applyListOperators = _interopRequireDefault(require("../../reducers/helpers/applyListOperators"));

var _wrapInObject = _interopRequireDefault(require("../../utils/object/wrapInObject"));

var _adaptOptionsForSingularResource = _interopRequireDefault(require("../../action-creators/helpers/adaptOptionsForSingularResource"));

var _configuration = require("../../configuration");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function actionCreator(options, paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions) {
  var action = options.action,
      transforms = options.transforms,
      keyBy = options.keyBy,
      urlOnlyParams = options.urlOnlyParams,
      singular = options.singular;

  var _adaptOptionsForSingu = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: true,
    acceptsValues: true
  }, [paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu.params,
      values = _adaptOptionsForSingu.values,
      actionCreatorOptions = _adaptOptionsForSingu.actionCreatorOptions;

  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);

  var temporaryKey = function () {
    var key = (0, _getItemKey["default"])([normalizedParams, values], {
      keyBy: keyBy,
      singular: singular
    });

    if (key || singular) {
      return key;
    }

    return (0, _configuration.getConfiguration)().generateId();
  }();

  return {
    type: action,
    status: _Statuses.NEW,
    temporaryKey: temporaryKey,
    listOperations: (0, _extractListOperations["default"])(actionCreatorOptions, urlOnlyParams),
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, _objectSpread(_objectSpread({}, _DataStructures.ITEM), {}, {
      values: values,
      status: {
        type: _Statuses.NEW
      }
    }))
  };
}

function reducer(resources, action) {
  var type = action.type,
      temporaryKey = action.temporaryKey,
      item = action.item,
      listOperations = action.listOperations;
  (0, _assertInDevMode["default"])(function () {
    var existingItem = resources.items[temporaryKey];

    if (existingItem) {
      if (resources.newItemKey === temporaryKey) {
        (0, _warn["default"])("'".concat(type, "' has same key '").concat(temporaryKey, "' as the previous new item, which has not finished saving ") + 'to the server. If you wish to create new items before the previous ones have finished saving, ' + 'ensure you use unique temporary keys. If you want to discard the previous item, use the ' + 'clearNewItem() action. (The previous item was overridden with new values.)');
      } else {
        (0, _warn["default"])("'".concat(type, "' has same key '").concat(temporaryKey, "' as existing item, use editItem() to ") + 'update it instead, or clearNewItem() if you want to discard the previous values. ' + '(The previous item was overridden with new values.)');
      }
    }
  });

  var newItems = _objectSpread(_objectSpread({}, resources.items), {}, _defineProperty({}, temporaryKey, _objectSpread({}, item)));

  return _objectSpread(_objectSpread({}, resources), {}, {
    items: newItems,
    lists: (0, _applyListOperators["default"])(_objectSpread(_objectSpread({}, resources), {}, {
      items: newItems
    }), listOperations, temporaryKey),
    newItemKey: temporaryKey
  });
}

var _default = {
  reducer: reducer,
  actionCreator: actionCreator
};
exports["default"] = _default;