"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _getItemKey = _interopRequireDefault(require("../../action-creators/helpers/getItemKey"));

var _generateUrl = _interopRequireDefault(require("../../action-creators/helpers/generateUrl"));

var _wrapInObject = _interopRequireDefault(require("../../utils/object/wrapInObject"));

var _makeRequest = _interopRequireDefault(require("../../action-creators/helpers/makeRequest"));

var _assertInDevMode = _interopRequireDefault(require("../../utils/assertInDevMode"));

var _warn = _interopRequireDefault(require("../../utils/dev/warn"));

var _Statuses = require("../../constants/Statuses");

var _DataStructures = require("../../constants/DataStructures");

var _applyTransforms = _interopRequireDefault(require("../../reducers/helpers/applyTransforms"));

var _mergeStatus = _interopRequireDefault(require("../../reducers/helpers/mergeStatus"));

var _without = _interopRequireDefault(require("../../utils/list/without"));

var _RequestManager = require("../../utils/RequestManager");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _adaptOptionsForSingularResource = _interopRequireDefault(require("../../action-creators/helpers/adaptOptionsForSingularResource"));

var _arrayFrom = _interopRequireDefault(require("../../utils/array/arrayFrom"));

var _toPlural = _interopRequireDefault(require("../../utils/string/toPlural"));

var _valuesAdded = _interopRequireDefault(require("../../utils/array/valuesAdded"));

var _valuesRemoved = _interopRequireDefault(require("../../utils/array/valuesRemoved"));

var _contains = _interopRequireDefault(require("../../utils/list/contains"));

var _extractListOperations = _interopRequireDefault(require("../../action-creators/helpers/extractListOperations"));

var _applyListOperators = _interopRequireDefault(require("../../reducers/helpers/applyListOperators"));

var _getHttpStatusCode = _interopRequireDefault(require("../../public-helpers/getHttpStatusCode"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var HTTP_REQUEST_TYPE = 'PUT';

function actionCreator(options, paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions) {
  var action = options.action,
      transforms = options.transforms,
      urlTemplate = options.url,
      progress = options.progress,
      keyBy = options.keyBy,
      metadata = options.metadata,
      requestAdaptor = options.requestAdaptor,
      _options$request = options.request,
      request = _options$request === void 0 ? {} : _options$request,
      singular = options.singular,
      urlOnlyParams = options.urlOnlyParams,
      _options$method = options.method,
      method = _options$method === void 0 ? HTTP_REQUEST_TYPE : _options$method;

  var _adaptOptionsForSingu = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: singular,
    acceptsValues: true
  }, [paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu.params,
      values = _adaptOptionsForSingu.values,
      actionCreatorOptions = _adaptOptionsForSingu.actionCreatorOptions;

  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  var url = (0, _generateUrl["default"])({
    urlTemplate: urlTemplate
  }, (0, _wrapInObject["default"])(normalizedParams, keyBy));

  if (actionCreatorOptions.force || (0, _RequestManager.isRequestInProgress)(method, url)) {
    return _nop["default"];
  } else {
    (0, _RequestManager.registerRequestStart)(method, url);
  }

  var key = (0, _getItemKey["default"])(normalizedParams, {
    keyBy: keyBy,
    singular: singular
  });
  return function (dispatch) {
    var listOperations = (0, _extractListOperations["default"])(actionCreatorOptions, urlOnlyParams);
    var requestedAt = Date.now();
    dispatch(submitUpdateResource({
      transforms: transforms,
      action: action,
      key: key,
      metadata: metadata,
      requestedAt: requestedAt
    }, actionCreatorOptions, values, listOperations));
    return (0, _makeRequest["default"])(_objectSpread(_objectSpread({}, options), {}, {
      params: normalizedParams,
      listOperations: listOperations,
      key: key,
      url: url,
      request: _objectSpread({
        method: method,
        body: JSON.stringify(requestAdaptor ? requestAdaptor(values) : values)
      }, request),
      dispatch: dispatch,
      onSuccess: receiveUpdatedResource,
      onError: handleUpdateResourceError,
      progress: progress
    }), actionCreatorOptions);
  };
}

function submitUpdateResource(options, actionCreatorOptions, values, listOperations) {
  var transforms = options.transforms,
      action = options.action,
      key = options.key,
      requestedAt = options.requestedAt;
  var metadata = actionCreatorOptions.metadata || options.metadata;
  return {
    type: action,
    status: _Statuses.UPDATING,
    key: key,
    listOperations: listOperations,
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, {
      values: values,
      status: {
        type: _Statuses.UPDATING,
        requestedAt: requestedAt
      },
      metadata: metadata
    }),
    previousValues: actionCreatorOptions.previousValues
  };
}

function localActionCreator(options, paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions) {
  var urlOnlyParams = options.urlOnlyParams,
      keyBy = options.keyBy,
      singular = options.singular;

  var _adaptOptionsForSingu2 = (0, _adaptOptionsForSingularResource["default"])({
    paramsOptional: singular,
    acceptsValues: true
  }, [paramsOrValues, valuesOrActionCreatorOptions, optionalActionCreatorOptions]),
      params = _adaptOptionsForSingu2.params,
      values = _adaptOptionsForSingu2.values,
      actionCreatorOptions = _adaptOptionsForSingu2.actionCreatorOptions;

  var metadata = actionCreatorOptions.metadata || options.metadata;
  var normalizedParams = (0, _wrapInObject["default"])(params, keyBy);
  var listOperations = (0, _extractListOperations["default"])(actionCreatorOptions, urlOnlyParams);
  return receiveUpdatedResource(_objectSpread(_objectSpread({}, options), {}, {
    params: normalizedParams,
    listOperations: listOperations
  }), actionCreatorOptions, values, metadata, actionCreatorOptions.previous);
}

function receiveUpdatedResource(options, actionCreatorOptions, values, metadata, httpCode) {
  var transforms = options.transforms,
      action = options.action,
      params = options.params,
      listOperations = options.listOperations,
      keyBy = options.keyBy,
      localOnly = options.localOnly,
      singular = options.singular;
  var previousValues = actionCreatorOptions.previousValues;
  return {
    type: action,
    status: _Statuses.SUCCESS,
    key: (0, _getItemKey["default"])([values, params], {
      keyBy: keyBy,
      singular: singular
    }),
    listOperations: listOperations,
    item: (0, _applyTransforms["default"])(transforms, options, actionCreatorOptions, {
      values: values,
      status: {
        type: _Statuses.SUCCESS,
        httpCode: httpCode,
        syncedAt: Date.now()
      },
      metadata: metadata
    }),
    previousValues: previousValues,
    localOnly: localOnly
  };
}

function handleUpdateResourceError(options, actionCreatorOptions, httpCode, errorEnvelope, metadata) {
  var action = options.action,
      key = options.key,
      localOnly = options.localOnly;
  return _objectSpread(_objectSpread({
    type: action,
    status: _Statuses.ERROR,
    key: key,
    metadata: metadata,
    httpCode: httpCode
  }, errorEnvelope), {}, {
    errorOccurredAt: Date.now(),
    localOnly: localOnly
  });
}

function reducer(resources, action) {
  var localOnly = action.localOnly,
      type = action.type,
      key = action.key,
      _action$listOperation = action.listOperations,
      listOperations = _action$listOperation === void 0 ? {} : _action$listOperation,
      status = action.status,
      item = action.item,
      httpCode = action.httpCode,
      error = action.error,
      errors = action.errors,
      errorOccurredAt = action.errorOccurredAt,
      metadata = action.metadata;
  var items = resources.items;
  (0, _assertInDevMode["default"])(function () {
    if (!items[key]) {
      (0, _warn["default"])("".concat(type, "'s key '").concat(key, "' did not match any items in the store. Check the arguments passed to ") + 'updateItem(). (Update request still sent to the server.)');
    } else if (items[key].status.type === _Statuses.NEW) {
      (0, _warn["default"])("".concat(type, "'s key '").concat(key, "' matched a new resource. Use editItem() to modify an item that has ") + 'not been saved to the server yet. (Update request still sent to the server.)');
    }
  });
  var currentItem = items[key] || _DataStructures.ITEM;

  if (status === _Statuses.UPDATING) {
    var newValues = _objectSpread(_objectSpread({}, currentItem.values), item.values);

    var updatedItem = _objectSpread(_objectSpread({}, item), {}, {
      values: newValues,
      status: (0, _mergeStatus["default"])(currentItem.status, item.status, {
        onlyPersist: ['syncedAt', 'dirty', 'originalValues']
      })
    });

    var newItems = _objectSpread(_objectSpread({}, items), {}, _defineProperty({}, key, updatedItem));

    var newLists = (0, _applyListOperators["default"])(_objectSpread(_objectSpread({}, resources), {}, {
      items: newItems
    }), listOperations, key);
    return _objectSpread(_objectSpread({}, resources), {}, {
      items: newItems,
      lists: newLists
    });
  } else if (status === _Statuses.SUCCESS) {
    var _newValues = _objectSpread(_objectSpread({}, currentItem.values), item.values);

    var _updatedItem = _objectSpread(_objectSpread({}, item), {}, {
      values: (0, _getHttpStatusCode["default"])(item) === 204 ? currentItem.values : _newValues,
      status: (0, _mergeStatus["default"])((0, _without["default"])(currentItem.status, ['dirty', 'originalValues']), item.status),
      metadata: _objectSpread(_objectSpread({}, currentItem.metadata), item.metadata)
    });

    var _newItems = _objectSpread(_objectSpread({}, items), {}, _defineProperty({}, key, _updatedItem));

    var _newLists = function () {
      if (localOnly) {
        return (0, _applyListOperators["default"])(_objectSpread(_objectSpread({}, resources), {}, {
          items: _newItems
        }), listOperations, key);
      } else {
        return resources.lists;
      }
    }();

    return _objectSpread(_objectSpread({}, resources), {}, {
      lists: _newLists,
      items: _newItems
    });
  } else if (status === _Statuses.ERROR) {
    var _newItems2 = _objectSpread(_objectSpread({}, items), {}, _defineProperty({}, key, _objectSpread(_objectSpread({}, items[key]), {}, {
      status: (0, _mergeStatus["default"])(currentItem.status, {
        type: status,
        httpCode: httpCode,
        error: error,
        errors: errors,
        errorOccurredAt: errorOccurredAt
      }),
      metadata: _objectSpread(_objectSpread({}, currentItem.metadata), metadata)
    })));

    return _objectSpread(_objectSpread({}, resources), {}, {
      items: _newItems2
    });
  } else {
    return resources;
  }
}

function hasManyAssociationsReducer(resources, _ref, _ref2) {
  var key = _ref.key,
      type = _ref.type,
      status = _ref.status,
      associatedItem = _ref.item,
      previousValues = _ref.previousValues;
  var relationType = _ref2.relationType,
      foreignKeyName = _ref2.foreignKeyName,
      name = _ref2.name,
      keyName = _ref2.keyName;

  if (status === _Statuses.SUCCESS) {
    var associationValues = associatedItem.values;
    var newForeignKeys = (0, _arrayFrom["default"])(associationValues[foreignKeyName] || associationValues[(0, _toPlural["default"])(foreignKeyName)]);

    if (previousValues) {
      var previousForeignKeys = (0, _arrayFrom["default"])(previousValues[foreignKeyName] || previousValues[(0, _toPlural["default"])(foreignKeyName)]);
      var addedAssociationKeys = (0, _valuesAdded["default"])(newForeignKeys, previousForeignKeys);
      var removedAssociationKeys = (0, _valuesRemoved["default"])(newForeignKeys, previousForeignKeys);
      return _objectSpread(_objectSpread({}, resources), {}, {
        items: _objectSpread(_objectSpread(_objectSpread({}, resources.items), addedAssociationKeys.reduce(function (memo, addedKey) {
          var item = resources.items[addedKey];

          var newKeys = function () {
            if (relationType === 'hasAndBelongsToMany') {
              return [key].concat(_toConsumableArray((0, _without["default"])(item.values[keyName] || [], key)));
            } else {
              return key;
            }
          }();

          if (item) {
            memo[addedKey] = _objectSpread(_objectSpread({}, item), {}, {
              values: _objectSpread(_objectSpread({}, item.values), {}, _defineProperty({}, keyName, newKeys))
            });
          }

          return memo;
        }, {})), removedAssociationKeys.reduce(function (memo, addedKey) {
          var item = resources.items[addedKey];

          var newValues = function () {
            if (relationType === 'hasAndBelongsToMany') {
              return _objectSpread(_objectSpread({}, item.values), {}, _defineProperty({}, keyName, (0, _without["default"])(item.values[keyName] || [], key)));
            } else {
              return (0, _without["default"])(item.values, keyName);
            }
          }();

          if (item) {
            memo[addedKey] = _objectSpread(_objectSpread({}, item), {}, {
              values: newValues
            });
          }

          return memo;
        }, {}))
      });
    } else {
      (0, _assertInDevMode["default"])(function () {
        (0, _warn["default"])("".concat(type, " did not specify any previous values. This makes updating '").concat(name, ".").concat(keyName, "' much ") + 'less efficient. Use the previousValues option for updateItem() to specify these values.');
      });
      return _objectSpread(_objectSpread({}, resources), {}, {
        items: Object.keys(resources.items).reduce(function (memo, itemKey) {
          var item = resources.items[itemKey];

          var newValues = function () {
            if ((0, _contains["default"])(newForeignKeys, itemKey, {
              stringifyFirst: true
            })) {
              if (relationType === 'hasAndBelongsToMany') {
                return _objectSpread(_objectSpread({}, item.values), {}, _defineProperty({}, keyName, [key].concat(_toConsumableArray((0, _without["default"])(item.values[keyName] || [], key)))));
              } else {
                return _objectSpread(_objectSpread({}, item.values), {}, _defineProperty({}, keyName, key));
              }
            } else {
              if (relationType === 'hasAndBelongsToMany') {
                return _objectSpread(_objectSpread({}, item.values), {}, _defineProperty({}, keyName, (0, _without["default"])(item.values[keyName] || [], key)));
              } else {
                return (0, _without["default"])(item.values, keyName);
              }
            }
          }();

          memo[itemKey] = _objectSpread(_objectSpread({}, item), {}, {
            values: newValues
          });
          return memo;
        }, {})
      });
    }
  } else {
    return resources;
  }
}

var _default = {
  reducer: reducer,
  hasManyAssociationsReducer: hasManyAssociationsReducer,
  actionCreator: actionCreator,
  localActionCreator: localActionCreator
};
exports["default"] = _default;