"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _isObject = _interopRequireDefault(require("../utils/object/isObject"));

var _warn = _interopRequireDefault(require("../utils/dev/warn"));

var _arrayFrom = _interopRequireDefault(require("../utils/array/arrayFrom"));

var _resolveOptions = _interopRequireDefault(require("../action-creators/helpers/resolveOptions"));

var _progressReducer = _interopRequireDefault(require("./helpers/progressReducer"));

var _configuration = require("../configuration");

var _fetchList = _interopRequireDefault(require("../actions/RESTful/fetchList"));

var _fetchItem = _interopRequireDefault(require("../actions/RESTful/fetchItem"));

var _newItem = _interopRequireDefault(require("../actions/RESTful/newItem"));

var _clearNewItem = _interopRequireDefault(require("../actions/RESTful/clearNewItem"));

var _editNewItem = _interopRequireDefault(require("../actions/RESTful/editNewItem"));

var _createItem = _interopRequireDefault(require("../actions/RESTful/createItem"));

var _editItem = _interopRequireDefault(require("../actions/RESTful/editItem"));

var _editNewOrExistingItem = _interopRequireDefault(require("../actions/RESTful/editNewOrExistingItem"));

var _clearItemEdit = _interopRequireDefault(require("../actions/RESTful/clearItemEdit"));

var _updateItem = _interopRequireDefault(require("../actions/RESTful/updateItem"));

var _destroyItem = _interopRequireDefault(require("../actions/RESTful/destroyItem"));

var _selectItem = _interopRequireDefault(require("../actions/selection/selectItem"));

var _selectAnotherItem = _interopRequireDefault(require("../actions/selection/selectAnotherItem"));

var _deselectItem = _interopRequireDefault(require("../actions/selection/deselectItem"));

var _clearSelectedItems = _interopRequireDefault(require("../actions/selection/clearSelectedItems"));

var _clearResource = _interopRequireDefault(require("../actions/clear/clearResource"));

var _clearItem2 = _interopRequireDefault(require("../actions/clear/clearItem"));

var _clearList2 = _interopRequireDefault(require("../actions/clear/clearList"));

var _isFunction = _interopRequireDefault(require("../utils/object/isFunction"));

var _mergeItemStatus2 = _interopRequireDefault(require("../public-reducers/mergeItemStatus"));

var _mergeItemValues2 = _interopRequireDefault(require("../public-reducers/mergeItemValues"));

var _mergeListStatus2 = _interopRequireDefault(require("../public-reducers/mergeListStatus"));

var _replaceListPositions2 = _interopRequireDefault(require("../public-reducers/replaceListPositions"));

var _replaceListMetadata2 = _interopRequireDefault(require("../public-reducers/replaceListMetadata"));

var _replaceItemMetadata2 = _interopRequireDefault(require("../public-reducers/replaceItemMetadata"));

var _mergeItemMetadata2 = _interopRequireDefault(require("../public-reducers/mergeItemMetadata"));

var _mergeListMetadata2 = _interopRequireDefault(require("../public-reducers/mergeListMetadata"));

var _clearItem3 = _interopRequireDefault(require("../public-reducers/clearItem"));

var _clearList3 = _interopRequireDefault(require("../public-reducers/clearList"));

var _clearResources = _interopRequireDefault(require("../public-reducers/clearResources"));

var _replaceItemValues2 = _interopRequireDefault(require("../public-reducers/replaceItemValues"));

var _getItemStatus2 = _interopRequireDefault(require("../public-reducers/getItemStatus"));

var _getItemValues2 = _interopRequireDefault(require("../public-reducers/getItemValues"));

var _getItemMetadata2 = _interopRequireDefault(require("../public-reducers/getItemMetadata"));

var _getListStatus2 = _interopRequireDefault(require("../public-reducers/getListStatus"));

var _getListPositions2 = _interopRequireDefault(require("../public-reducers/getListPositions"));

var _getListMetadata2 = _interopRequireDefault(require("../public-reducers/getListMetadata"));

var _isString = _interopRequireDefault(require("../utils/string/isString"));

var _removeItemFromListPositions2 = _interopRequireDefault(require("../public-reducers/removeItemFromListPositions"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var STANDARD_REDUCERS = {
  fetchList: _fetchList["default"].reducer,
  fetchItem: _fetchItem["default"].reducer,
  newItem: _newItem["default"].reducer,
  clearNewItem: _clearNewItem["default"].reducer,
  editNewItem: _editNewItem["default"].reducer,
  createItem: _createItem["default"].reducer,
  editItem: _editItem["default"].reducer,
  editNewOrExistingItem: _editNewOrExistingItem["default"].reducer,
  clearItemEdit: _clearItemEdit["default"].reducer,
  updateItem: _updateItem["default"].reducer,
  destroyItem: _destroyItem["default"].reducer,
  clearResource: _clearResource["default"].reducer,
  clearItem: _clearItem2["default"].reducer,
  clearList: _clearList2["default"].reducer,
  selectItem: _selectItem["default"].reducer,
  selectAnotherItem: _selectAnotherItem["default"].reducer,
  deselectItem: _deselectItem["default"].reducer,
  clearSelectedItems: _clearSelectedItems["default"].reducer
};
var PROGRESS_COMPATIBLE_ACTIONS = {
  fetchList: true,
  fetchItem: true,
  updateItem: true,
  createItem: true
};

function getProgressReducer(key) {
  if (key === 'fetchList') {
    return function (resources, action) {
      return (0, _progressReducer["default"])(resources, action, 'lists');
    };
  } else {
    return _progressReducer["default"];
  }
}

function buildReducersDictionary(resourceOptions, actionsDictionary, actionsOptions) {
  var globalConfiguration = (0, _configuration.getConfiguration)();
  var reducerHelpers = {
    getItemStatus: function getItemStatus(state, params) {
      return (0, _getItemStatus2["default"])(resourceOptions, state, params);
    },
    mergeItemStatus: function mergeItemStatus(state, params, newStatus) {
      return (0, _mergeItemStatus2["default"])(resourceOptions, state, params, newStatus);
    },
    getItemValues: function getItemValues(state, params) {
      return (0, _getItemValues2["default"])(resourceOptions, state, params);
    },
    mergeItemValues: function mergeItemValues(state, params, newValues) {
      return (0, _mergeItemValues2["default"])(resourceOptions, state, params, newValues);
    },
    replaceItemValues: function replaceItemValues(state, params, values) {
      return (0, _replaceItemValues2["default"])(resourceOptions, state, params, values);
    },
    clearItemValues: function clearItemValues(state, params) {
      return (0, _replaceItemValues2["default"])(resourceOptions, state, params, {});
    },
    clearItem: function clearItem(state, params) {
      return (0, _clearItem3["default"])(resourceOptions, state, params);
    },
    getItemMetadata: function getItemMetadata(state, params) {
      return (0, _getItemMetadata2["default"])(resourceOptions, state, params);
    },
    mergeItemMetadata: function mergeItemMetadata(state, params, metadata) {
      return (0, _mergeItemMetadata2["default"])(resourceOptions, state, params, metadata);
    },
    replaceItemMetadata: function replaceItemMetadata(state, params, metadata) {
      return (0, _replaceItemMetadata2["default"])(resourceOptions, state, params, metadata);
    },
    clearItemMetadata: function clearItemMetadata(state, params) {
      return (0, _replaceItemMetadata2["default"])(resourceOptions, state, params, {});
    },
    getListStatus: function getListStatus(state, params) {
      return (0, _getListStatus2["default"])(resourceOptions, state, params);
    },
    mergeListStatus: function mergeListStatus(state, params, newStatus) {
      return (0, _mergeListStatus2["default"])(resourceOptions, state, params, newStatus);
    },
    getListPositions: function getListPositions(state, params) {
      return (0, _getListPositions2["default"])(resourceOptions, state, params);
    },
    removeItemFromListPositions: function removeItemFromListPositions(state, listParams, itemParams) {
      return (0, _removeItemFromListPositions2["default"])(resourceOptions, state, listParams, itemParams);
    },
    replaceListPositions: function replaceListPositions(state, params, positions) {
      return (0, _replaceListPositions2["default"])(resourceOptions, state, params, positions);
    },
    getListMetadata: function getListMetadata(state, params) {
      return (0, _getListMetadata2["default"])(resourceOptions, state, params);
    },
    mergeListMetadata: function mergeListMetadata(state, params, metadata) {
      return (0, _mergeListMetadata2["default"])(resourceOptions, state, params, metadata);
    },
    replaceListMetadata: function replaceListMetadata(state, params, metadata) {
      return (0, _replaceListMetadata2["default"])(resourceOptions, state, params, metadata);
    },
    clearListMetadata: function clearListMetadata(state, params) {
      return (0, _replaceListMetadata2["default"])(resourceOptions, state, params, {});
    },
    clearList: function clearList(state, params) {
      return (0, _clearList3["default"])(resourceOptions, state, params);
    },
    deselectItem: function deselectItem(state, params) {
      return _deselectItem["default"].reducer(state, _deselectItem["default"].actionCreator(resourceOptions, params));
    },
    selectAnotherItem: function selectAnotherItem(state, params) {
      return _selectAnotherItem["default"].reducer(state, _selectAnotherItem["default"].actionCreator(resourceOptions, params));
    },
    selectItem: function selectItem(state, params) {
      return _selectItem["default"].reducer(state, _selectItem["default"].actionCreator(resourceOptions, params));
    },
    clearSelectedItems: _clearSelectedItems["default"].reducer,
    clearResource: _clearResources["default"]
  };
  reducerHelpers.deselectItems = reducerHelpers.deselectItem;
  reducerHelpers.selectItems = reducerHelpers.selectItem;
  reducerHelpers.selectMoreItems = reducerHelpers.selectAnotherItem;
  reducerHelpers.clearResources = reducerHelpers.clearResource;
  var reducersDict = Object.keys(actionsOptions).reduce(function (memo, key) {
    var actionOptions = actionsOptions[key];

    var reducer = function () {
      if ((0, _isObject["default"])(actionOptions)) {
        var customReducer = actionOptions.reducer;

        if ((0, _isFunction["default"])(customReducer)) {
          return function (resources, action) {
            return customReducer(resources, action, reducerHelpers);
          };
        } else if ((0, _isString["default"])(customReducer)) {
          return STANDARD_REDUCERS[customReducer];
        }
      }

      return STANDARD_REDUCERS[key];
    }();

    if (reducer) {
      var reducerOptions = (0, _resolveOptions["default"])(globalConfiguration, resourceOptions, actionOptions, ['progress', 'beforeReducers', 'afterReducers']);
      var beforeReducers = (0, _arrayFrom["default"])(reducerOptions.beforeReducers);

      if (reducerOptions.progress && (!STANDARD_REDUCERS[key] || PROGRESS_COMPATIBLE_ACTIONS[key])) {
        reducerOptions.beforeReducers = [].concat(_toConsumableArray(beforeReducers), [getProgressReducer(key)]);
      }

      var _reducer = function () {
        var afterReducers = (0, _arrayFrom["default"])(reducerOptions.afterReducers);

        if (beforeReducers.length > 0 || afterReducers.length > 0) {
          return function (resources, action) {
            var _resources = resources;
            beforeReducers.forEach(function (beforeReducer) {
              _resources = beforeReducer(_resources, action, reducerHelpers);
            });
            _resources = reducer(_resources, action, reducerHelpers);
            afterReducers.forEach(function (afterReducer) {
              _resources = afterReducer(_resources, action, reducerHelpers);
            });
            return _resources;
          };
        } else {
          return reducer;
        }
      }();

      memo[actionsDictionary[key]] = {
        options: reducerOptions,
        reducer: _reducer
      };
    } else {
      if (resourceOptions.localOnly && STANDARD_REDUCERS[key]) {
        (0, _warn["default"])("Action '".concat(key, "' is not compatible with the localOnly option."));
      } else {
        var standardReducersList = Object.keys(STANDARD_REDUCERS).join(', ');
        (0, _warn["default"])("Action '".concat(key, "' must match the list of standard reducers (").concat(standardReducersList, ") or define a 'reducer' option."));
      }
    }

    return memo;
  }, {});
  (0, _arrayFrom["default"])(resourceOptions.clearOn).forEach(function (action) {
    reducersDict[action] = {
      reducer: STANDARD_REDUCERS.clearResource
    };
  });
  Object.keys(resourceOptions.reducesOn || {}).forEach(function (actionName) {
    var _reducer2 = resourceOptions.reducesOn[actionName];

    if (actionsDictionary[actionName]) {
      var existingReducerKey = actionsDictionary[actionName];
      var existingReducer = reducersDict[existingReducerKey];
      reducersDict[existingReducerKey] = {
        reducer: function reducer(resources, action) {
          return _reducer2(existingReducer.reducer(resources, action), action, reducerHelpers);
        }
      };
    } else {
      reducersDict[actionName] = {
        reducer: function reducer(resources, action) {
          return _reducer2(resources, action, reducerHelpers);
        }
      };
    }
  });
  return reducersDict;
}

var _default = buildReducersDictionary;
exports["default"] = _default;