"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _buildStore = _interopRequireDefault(require("../helpers/buildStore"));

var _index = require("../../index");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('Generating URLs:', function () {
  describe('Specifying URL for all resources actions', function () {
    beforeAll(function () {
      var _resources = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id'
      }, {
        fetchList: true,
        fetchItem: true,
        updateItem: true,
        destroyItem: true,
        createItem: true
      }),
          reducers = _resources.reducers,
          _resources$actionCrea = _resources.actionCreators,
          createUser = _resources$actionCrea.createItem,
          fetchUsers = _resources$actionCrea.fetchList,
          fetchUser = _resources$actionCrea.fetchItem,
          updateUser = _resources$actionCrea.updateItem,
          destroyUser = _resources$actionCrea.destroyItem;

      this.createUser = createUser;
      this.fetchUsers = fetchUsers;
      this.fetchUser = fetchUser;
      this.updateUser = updateUser;
      this.destroyUser = destroyUser;
      this.store = (0, _buildStore["default"])({
        users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
          items: {
            1: {
              values: {
                username: 'Bob'
              },
              status: {
                type: _index.SUCCESS
              }
            }
          }
        })
      }, {
        users: reducers
      });
    });
    it('then makes the correct API request for fetching the index', function () {
      _fetchMock["default"].get('http://test.com/users', {
        body: [{
          id: 1,
          username: 'Bob'
        }],
        status: 200
      });

      this.store.dispatch(this.fetchUsers());

      _fetchMock["default"].restore();
    });
    it('then makes the correct API request for fetching an item', function () {
      _fetchMock["default"].get('http://test.com/users/1', {
        body: {
          id: 1,
          username: 'Bob'
        },
        status: 200
      });

      this.store.dispatch(this.fetchUser({
        id: 1
      }));

      _fetchMock["default"].restore();
    });
    it('then makes the correct API request for creating a new item', function () {
      _fetchMock["default"].post('http://test.com/users', {
        body: {
          id: 1,
          username: 'Bob'
        },
        status: 200
      });

      this.store.dispatch(this.createUser('temp', {
        id: 1,
        username: 'Bob'
      }));

      _fetchMock["default"].restore();
    });
    it('then makes the correct API request for updating an item', function () {
      _fetchMock["default"].put('http://test.com/users/1', {
        body: {
          id: 1,
          username: 'Robert'
        },
        status: 200
      });

      this.store.dispatch(this.updateUser(1, {
        username: 'Robert'
      }));

      _fetchMock["default"].restore();
    });
    it('then makes the correct API request to destroy an item', function () {
      _fetchMock["default"]["delete"]('http://test.com/users/1', {
        body: {},
        status: 200
      });

      this.store.dispatch(this.destroyUser(1));

      _fetchMock["default"].restore();
    });
    describe('when extra params are present', function () {
      it('then uses them as query params when fetching the index', function () {
        _fetchMock["default"].get('http://test.com/users?sandbox=true', {
          body: [{
            id: 1,
            username: 'Bob'
          }],
          status: 200
        });

        this.store.dispatch(this.fetchUsers({
          sandbox: true
        }));

        _fetchMock["default"].restore();
      });
      it('then uses them as query params when fetching an item', function () {
        _fetchMock["default"].get('http://test.com/users/1?sandbox=true', {
          body: {
            id: 1,
            username: 'Bob'
          },
          status: 200
        });

        this.store.dispatch(this.fetchUser({
          id: 1,
          sandbox: true
        }));

        _fetchMock["default"].restore();
      });
      it('then uses them as query params when creating a new item', function () {
        _fetchMock["default"].post('http://test.com/users?sandbox=true', {
          body: {
            id: 1,
            username: 'Bob'
          },
          status: 200
        });

        this.store.dispatch(this.createUser({
          id: 'temp',
          sandbox: true
        }, {
          id: 1,
          username: 'Bob'
        }));

        _fetchMock["default"].restore();
      });
      it('then uses them as query params when updating an item', function () {
        _fetchMock["default"].put('http://test.com/users/1?sandbox=true', {
          body: {
            id: 1,
            username: 'Robert'
          },
          status: 200
        });

        this.store.dispatch(this.updateUser({
          id: 1,
          sandbox: true
        }, {
          username: 'Robert'
        }));

        _fetchMock["default"].restore();
      });
      it('then uses them as query params when destroying an item', function () {
        _fetchMock["default"]["delete"]('http://test.com/users/1?sandbox=true', {
          body: {},
          status: 200
        });

        this.store.dispatch(this.destroyUser({
          id: 1,
          sandbox: true
        }));

        _fetchMock["default"].restore();
      });
    });
  });
  describe('when the default is overridden for a particular action', function () {
    beforeAll(function () {
      var _resources2 = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id'
      }, {
        fetchList: true,
        fetchItem: {
          url: 'http://test.com/guests/:order?'
        }
      }),
          reducers = _resources2.reducers,
          _resources2$actionCre = _resources2.actionCreators,
          fetchUsers = _resources2$actionCre.fetchList,
          fetchUser = _resources2$actionCre.fetchItem;

      this.fetchUsers = fetchUsers;
      this.fetchUser = fetchUser;
      this.store = (0, _buildStore["default"])({
        users: _index.RESOURCES
      }, {
        users: reducers
      });
    });
    it('then uses the url for that action', function () {
      _fetchMock["default"].get('http://test.com/users', {
        body: [{
          id: 1,
          username: 'Bob'
        }],
        status: 200
      });

      this.store.dispatch(this.fetchUsers());

      _fetchMock["default"].restore();
    });
    it('then uses the default url for other actions', function () {
      _fetchMock["default"].get('http://test.com/guests/newest', {
        body: {
          id: 1,
          username: 'Bob'
        },
        status: 200
      });

      this.store.dispatch(this.fetchUser({
        order: 'newest'
      }));

      _fetchMock["default"].restore();
    });
  });
});