"use strict";

var _index = require("../../index");

var _Statuses = require("../../constants/Statuses");

var _MetadataTypes = require("../../constants/MetadataTypes");

var _DataStructures = require("../../constants/DataStructures");

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('buildInitialState:', function () {
  describe('Given a resource has been defined with no special options,', function () {
    beforeAll(function () {
      var _resources = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id'
      }, ['fetchItem']),
          buildInitialState = _resources.buildInitialState;

      this.buildInitialState = buildInitialState;
    });
    describe('when the initial state builder is instantiated with a list of items', function () {
      it('then correctly builds those items', function () {
        var stateBuilder = this.buildInitialState([{
          id: 1,
          username: 'John'
        }]);
        expect(stateBuilder.build()).toEqual(_objectSpread(_objectSpread({}, _DataStructures.RESOURCES), {}, {
          items: {
            '1': {
              values: {
                id: 1,
                username: 'John'
              },
              status: {
                type: _Statuses.SUCCESS
              },
              metadata: {
                type: _MetadataTypes.COMPLETE
              }
            }
          },
          lists: _defineProperty({}, _EmptyKey["default"], {
            positions: [1],
            status: {
              type: _Statuses.SUCCESS
            },
            metadata: {
              type: _MetadataTypes.COMPLETE
            }
          })
        }));
      });
    });
    describe('when the initial state builder is instantiated and then passed a list', function () {
      it('then correctly builds those items', function () {
        var stateBuilder = this.buildInitialState();
        stateBuilder.addList([{
          id: 1,
          username: 'John'
        }]);
        expect(stateBuilder.build()).toEqual(_objectSpread(_objectSpread({}, _DataStructures.RESOURCES), {}, {
          items: {
            '1': {
              values: {
                id: 1,
                username: 'John'
              },
              status: {
                type: _Statuses.SUCCESS
              },
              metadata: {
                type: _MetadataTypes.COMPLETE
              }
            }
          },
          lists: _defineProperty({}, _EmptyKey["default"], {
            positions: [1],
            status: {
              type: _Statuses.SUCCESS
            },
            metadata: {
              type: _MetadataTypes.COMPLETE
            }
          })
        }));
      });
    });
    describe('when the initial state builder is instantiated and then passed an item', function () {
      it('then correctly builds those items as outside any list', function () {
        var stateBuilder = this.buildInitialState();
        stateBuilder.addItem({
          id: 1,
          username: 'John'
        });
        expect(stateBuilder.build()).toEqual(_objectSpread(_objectSpread({}, _DataStructures.RESOURCES), {}, {
          items: {
            '1': {
              values: {
                id: 1,
                username: 'John'
              },
              status: {
                type: _Statuses.SUCCESS
              },
              metadata: {
                type: _MetadataTypes.COMPLETE
              }
            }
          }
        }));
      });
    });
    describe('when the status type is set only on the resource', function () {
      it('then correctly inherits that status type for all items and lists', function () {
        var stateBuilder = this.buildInitialState();
        stateBuilder.addList([{
          id: 1,
          username: 'John'
        }]);
        var customStatusType = 'CUSTOM';
        stateBuilder.setStatusType(customStatusType);
        var results = stateBuilder.build();
        expect(results.lists[_EmptyKey["default"]].status.type).toEqual(customStatusType);
        expect(results.items['1'].status.type).toEqual(customStatusType);
      });
    });
    describe('when the status type is set only on the list', function () {
      it('then correctly inherits that status type for all items and lists', function () {
        var stateBuilder = this.buildInitialState();
        var customStatusType = 'CUSTOM';
        stateBuilder.addList([{
          id: 1,
          username: 'John'
        }]).setStatusType(customStatusType);
        var results = stateBuilder.build();
        expect(results.lists[_EmptyKey["default"]].status.type).toEqual(customStatusType);
        expect(results.items['1'].status.type).toEqual(customStatusType);
      });
    });
    describe('when the status type is set only on the item', function () {
      it('then correctly applies the status type only to the item', function () {
        var stateBuilder = this.buildInitialState();
        var customStatusType = 'CUSTOM';
        stateBuilder.addList([]);
        stateBuilder.addItem({
          id: 1,
          username: 'John'
        }).setStatusType(customStatusType);
        var results = stateBuilder.build();
        expect(results.lists[_EmptyKey["default"]].status.type).toEqual(_Statuses.SUCCESS);
        expect(results.items['1'].status.type).toEqual(customStatusType);
      });
    });
    describe('when the status type is set on the resource and the list', function () {
      it('then correctly applies the list\'s status type to the list and item', function () {
        var listStatus = 'LIST_STATUS';
        var resourceStatusType = 'RESOURCE_STATUS';
        var stateBuilder = this.buildInitialState().setStatusType(resourceStatusType);
        stateBuilder.addList([{
          id: 1,
          username: 'John'
        }]).setStatusType(listStatus);
        var results = stateBuilder.build();
        expect(results.lists[_EmptyKey["default"]].status.type).toEqual(listStatus);
        expect(results.items['1'].status.type).toEqual(listStatus);
      });
    });
    describe('when the status type is set on the list and the item', function () {
      it('then correctly applies status types to the list and item', function () {
        var listStatus = 'LIST_STATUS';
        var itemStatus = 'ITEM_STATUS';
        var stateBuilder = this.buildInitialState();
        var listBuilder = stateBuilder.addList([{
          id: 1,
          username: 'John'
        }]).setStatusType(listStatus);
        listBuilder.addItem({
          id: 2,
          username: 'Bob'
        }).setStatusType(itemStatus);
        var results = stateBuilder.build();
        expect(results.lists[_EmptyKey["default"]].status.type).toEqual(listStatus);
        expect(results.items['1'].status.type).toEqual(listStatus);
        expect(results.items['2'].status.type).toEqual(itemStatus);
      });
    });
  });
});