"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _resourceAssertions = require("../helpers/resourceAssertions");

var _ = require("../..");

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('canFallbackToOldValues:', function () {
  beforeAll(function () {
    this.url = 'http://test.com/users';

    var _resource = (0, _index.resource)({
      name: 'users',
      url: this.url,
      keyBy: 'id'
    }, {
      fetchItem: true
    }),
        reducers = _resource.reducers,
        fetchUser = _resource.actionCreators.fetchItem;

    this.fetchUser = fetchUser;
    this.reducers = reducers;
  });
  describe('Given a resource item does not already exist in the store', function () {
    describe('when the resource is requested and before it has returned from the remote', function () {
      expectToReturnWhileRequestIsInFlight(false);
    });
    describe('and the API request succeeds', function () {
      beforeAll(function () {
        this.newValues = {
          username: 'Bob'
        };
        setUpAfterRequestSuccess(this, this.url, _objectSpread({}, _.RESOURCES), {
          body: this.newValues
        });
      });
      it("then returns ".concat(false), function () {
        expect((0, _index.canFallbackToOldValues)((0, _resourceAssertions.resourceItem)(this, RESOURCE_NAME))).toEqual(false);
      });
      afterAll(function () {
        tearDown(this);
      });
    });
    describe('when the API request errors', function () {
      beforeAll(function () {
        this.options = {
          body: {
            error: 'Not Found'
          },
          status: 404
        };
        setUpAfterRequestFailure(this, this.url, _objectSpread({}, _.RESOURCES), this.options);
      });
      it("then returns ".concat(false), function () {
        expect((0, _index.canFallbackToOldValues)((0, _resourceAssertions.resourceItem)(this, RESOURCE_NAME))).toEqual(false);
      });
      afterAll(function () {
        tearDown(this);
      });
    });
  });
  describe('Given a resource item has already been successfully fetched', function () {
    beforeAll(function () {
      this.newValues = {
        username: 'Bob'
      };
      this.initialState = _objectSpread(_objectSpread({}, _.RESOURCES), {}, {
        items: _defineProperty({}, _EmptyKey["default"], {
          values: this.newValues,
          status: {
            type: _.SUCCESS,
            requestedAt: 1590216487970,
            syncedAt: 1590216487975
          },
          metadata: {
            type: _.COMPLETE
          }
        })
      });
    });
    describe('when the resource is requested and before it has returned from the remote', function () {
      expectToReturnWhileRequestIsInFlight(false);
    });
    describe('and the API request succeeds', function () {
      beforeAll(function () {
        setUpAfterRequestSuccess(this, this.url, this.initialState, {
          body: this.newValues
        });
      });
      it("then returns ".concat(false), function () {
        expect((0, _index.canFallbackToOldValues)((0, _resourceAssertions.resourceItem)(this, RESOURCE_NAME))).toEqual(false);
      });
      afterAll(function () {
        tearDown(this);
      });
    });
    describe('when the API request errors', function () {
      beforeAll(function () {
        this.options = {
          body: {
            error: 'Not Found'
          },
          status: 404
        };
        setUpAfterRequestFailure(this, this.url, this.initialState, this.options);
      });
      it("then returns ".concat(true), function () {
        expect((0, _index.canFallbackToOldValues)((0, _resourceAssertions.resourceItem)(this, RESOURCE_NAME))).toEqual(true);
      });
      afterAll(function () {
        tearDown(this);
      });
    });
  });

  function expectToReturnWhileRequestIsInFlight(expectation) {
    beforeAll(function () {
      setUpBeforeRequest(this, this.url, _objectSpread({}, _.RESOURCES));
    });
    it("then returns ".concat(expectation), function () {
      expect((0, _index.canFallbackToOldValues)((0, _resourceAssertions.resourceItem)(this, RESOURCE_NAME))).toEqual(expectation);
    });
    afterAll(function () {
      tearDown(this);
    });
  }

  function setUpBeforeRequest(context, url, initialState) {
    _fetchMock["default"].get(url, new Promise(_nop["default"]));

    setupState(context, initialState);
  }

  function setUpAfterRequestSuccess(context, url, initialState) {
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
      body: {}
    };

    _fetchMock["default"].get(url, options);

    setupState(context, initialState);
  }

  function setUpAfterRequestFailure(context, url, initialState) {
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"].get(url, options);

    setupState(context, initialState);
  }

  function setupState(context, initialState) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.fetchUser());
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});