"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _resourceAssertions = require("../helpers/resourceAssertions");

var _ = require("../..");

var _getNewItem = _interopRequireDefault(require("../../public-helpers/getNewItem"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('getNewItem:', function () {
  beforeAll(function () {
    this.url = 'http://test.com/users/:id';

    var _resources2 = (0, _index.resources)({
      name: 'users',
      url: this.url,
      keyBy: 'id'
    }, ['newItem', 'createItem']),
        reducers = _resources2.reducers,
        actionCreators = _resources2.actionCreators;

    this.actionCreators = actionCreators;
    this.reducers = reducers;
  });
  describe('Given there is no new item', function () {
    beforeAll(function () {
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
    });
    it('then returns an empty item', function () {
      expect((0, _getNewItem["default"])((0, _resourceAssertions.resourceDefinition)(this, RESOURCE_NAME))).toEqual(_.ITEM);
    });
  });
  describe('Given a new item is being worked on', function () {
    describe('and no id is passed to the action creator', function () {
      beforeAll(function () {
        this.values = {
          username: 'Bob'
        };
        (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
        this.store.dispatch(this.actionCreators.newItem(this.values));
      });
      it('then returns the new item', function () {
        expect((0, _getNewItem["default"])((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME)).values).toEqual(this.values);
      });
    });
    describe('and an id is passed to the action creator', function () {
      beforeAll(function () {
        this.itemId = 'temp';
        this.values = {
          username: 'Bob'
        };
        (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
        this.store.dispatch(this.actionCreators.newItem(this.itemId, this.values));
      });
      it('then returns the new item', function () {
        expect((0, _getNewItem["default"])((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME)).values).toEqual(this.values);
      });
    });
  });
  describe('Given the new item is creating', function () {
    beforeAll(function () {
      this.itemId = 'temp';
      this.values = {
        username: 'Bob'
      };

      _fetchMock["default"].post("http://test.com/users/".concat(this.itemId), new Promise(_nop["default"]));

      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
      this.store.dispatch(this.actionCreators.createItem(this.itemId, this.values));
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then returns the new item', function () {
      var newItem = (0, _getNewItem["default"])((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME));
      expect(newItem.values).toEqual(this.values);
      expect(newItem.status.type).toEqual(_.CREATING);
    });
  });
  describe('Given the new item has just been created', function () {
    beforeAll(function () {
      this.itemId = 'temp';
      this.newId = 1;
      this.values = {
        username: 'Bob',
        id: this.newId
      };

      _fetchMock["default"].post("http://test.com/users/".concat(this.itemId), {
        body: this.values
      });

      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
      this.store.dispatch(this.actionCreators.createItem(this.itemId, this.values));
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then returns the new item', function () {
      var _resources = (0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME);

      expect(_resources.newItemKey).toEqual(this.newId);
      var newItem = (0, _getNewItem["default"])(_resources);
      expect(newItem.values).toEqual(this.values);
      expect(newItem.status.type).toEqual(_.SUCCESS);
    });
  });
  describe('Given the new item has just failed to create', function () {
    beforeAll(function () {
      this.itemId = 'temp';
      this.values = {
        username: 'Bob'
      };

      _fetchMock["default"].post("http://test.com/users/".concat(this.itemId), {
        body: {
          error: 'Unauthorized'
        },
        status: 401
      });

      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
      this.store.dispatch(this.actionCreators.createItem(this.itemId, this.values));
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then returns the new item', function () {
      var newItem = (0, _getNewItem["default"])((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME));
      expect(newItem.values).toEqual(this.values);
      expect(newItem.status.type).toEqual(_.ERROR);
    });
  });

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});