"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _resourceAssertions = require("../helpers/resourceAssertions");

var _ = require("../..");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('getTimeSinceFetchStarted:', function () {
  beforeAll(function () {
    this.urlBase = 'http://test.com/users';
    this.url = "".concat(this.urlBase, "/:id?");

    var _resources = (0, _index.resources)({
      name: 'users',
      url: this.url,
      keyBy: 'id'
    }, ['newItem', 'fetchItem', 'createItem', 'updateItem']),
        reducers = _resources.reducers,
        actionCreators = _resources.actionCreators;

    this.actionCreators = actionCreators;
    this.reducers = reducers;
  });
  describe('Given there are no current outstanding remote API requests', function () {
    beforeAll(function () {
      this.itemId = 1;
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread(_objectSpread({}, _.RESOURCES), {}, {
        items: _defineProperty({}, this.itemId, {
          values: {
            id: this.itemId,
            username: 'Bob'
          },
          status: {
            type: _.SUCCESS
          }
        })
      }));
    });
    it('then returns 0', function () {
      expect((0, _.getTimeSinceFetchStarted)((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, this.itemId))).toEqual(0);
    });
  });
  describe('Given the item is currently being fetched', function () {
    beforeAll(function () {
      this.itemId = 'temp';

      _fetchMock["default"].get("".concat(this.urlBase, "/").concat(this.itemId), new Promise(_nop["default"]));

      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
      this.store.dispatch(this.actionCreators.fetchItem(this.itemId));
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then returns the time since the fetchItem started', function () {
      expect((0, _.getTimeSinceFetchStarted)((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, this.itemId))).toBeGreaterThanOrEqual(0);
    });
  });
  describe('Given the item has finished fetching', function () {
    beforeAll(function () {
      this.itemId = 1;
      this.values = {
        username: 'Bob',
        id: this.itemId
      };

      _fetchMock["default"].get("".concat(this.urlBase, "/").concat(this.itemId), {
        body: this.values
      });

      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
      this.store.dispatch(this.actionCreators.fetchItem(this.itemId));
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then returns 0', function () {
      expect((0, _.getTimeSinceFetchStarted)((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, this.itemId))).toEqual(0);
    });
  });
  describe('Given the item has failed to fetchItem', function () {
    beforeAll(function () {
      this.itemId = 1;

      _fetchMock["default"].get("".concat(this.urlBase, "/").concat(this.itemId), {
        body: {
          error: 'Not Found'
        },
        status: 404
      });

      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
      this.store.dispatch(this.actionCreators.fetchItem(this.itemId));
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then returns 0', function () {
      expect((0, _.getTimeSinceFetchStarted)((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, this.itemId))).toEqual(0);
    });
  });
  describe('Given the item is currently being created', function () {
    beforeAll(function () {
      this.itemId = 'temp';
      this.newItemId = 1;
      this.values = {
        username: 'Bob',
        id: this.newItemId
      };

      _fetchMock["default"].post(this.urlBase, new Promise(_nop["default"]));

      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread({}, _.RESOURCES));
      this.store.dispatch(this.actionCreators.createItem(this.itemId, this.values));
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then returns 0', function () {
      expect((0, _.getTimeSinceFetchStarted)((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, this.itemId))).toEqual(0);
    });
  });
  describe('Given the item is currently being updated', function () {
    beforeAll(function () {
      this.itemId = 1;
      this.values = {
        username: 'Bob'
      };

      _fetchMock["default"].put("".concat(this.urlBase, "/").concat(this.itemId), new Promise(_nop["default"]));

      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, _objectSpread(_objectSpread({}, _.RESOURCES), {}, {
        items: _defineProperty({}, this.itemId, {
          values: {
            id: this.itemId,
            username: 'Bob'
          },
          status: {
            type: _.SUCCESS
          }
        })
      }));
      this.store.dispatch(this.actionCreators.updateItem(this.itemId, this.values));
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then returns 0', function () {
      expect((0, _.getTimeSinceFetchStarted)((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, this.itemId))).toEqual(0);
    });
  });

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});