"use strict";

var _Statuses = require("../../../constants/Statuses");

var _index = require("../../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

var _ = require("../../..");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Clear editItem reducer:', function () {
  beforeAll(function () {
    var _resource = (0, _index.resource)({
      name: RESOURCE_NAME
    }, {
      clearItemEdit: true
    }),
        reducers = _resource.reducers,
        clearUserEdit = _resource.actionCreators.clearItemEdit;

    this.clearUserEdit = clearUserEdit;
    this.reducers = reducers;
  });
  describe('Given the user attempts to clear the resource item when it is NOT being edited', function () {
    beforeAll(function () {
      this.initialState = _objectSpread(_objectSpread({}, _.RESOURCES), {}, {
        items: _defineProperty({}, _EmptyKey["default"], {
          values: {
            id: 2,
            username: 'Bob'
          },
          status: {
            type: _Statuses.SUCCESS,
            syncedAt: 2,
            requestedAt: 1
          }
        })
      });
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, this.initialState);
      this.store.dispatch(this.clearUserEdit());
    });
    it('then does NOT change the status', function () {
      (0, _resourceAssertions.expectToNotChangeResourceItemStatus)(this, RESOURCE_NAME);
    });
    it('then does NOT change the values', function () {
      (0, _resourceAssertions.expectToNotChangeResourceItemValues)(this, RESOURCE_NAME);
    });
  });
  describe('Given the user attempts to clear the resource item when it is being edited', function () {
    beforeAll(function () {
      this.initialState = _objectSpread(_objectSpread({}, _.RESOURCES), {}, {
        items: _defineProperty({}, _EmptyKey["default"], {
          values: {
            id: 1,
            username: 'Jill'
          },
          status: {
            type: _Statuses.EDITING,
            syncedAt: 2,
            requestedAt: 1,
            dirty: 1,
            originalValues: {
              id: 1,
              username: 'Jillian'
            }
          }
        })
      });
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, this.initialState);
      this.store.dispatch(this.clearUserEdit());
    });
    expectToClearResource();
  });
  describe('Given an update has failed to apply for a resource item', function () {
    beforeAll(function () {
      this.initialState = _objectSpread(_objectSpread({}, _.RESOURCES), {}, {
        items: _defineProperty({}, _EmptyKey["default"], {
          values: {
            username: 'Robert',
            id: 1
          },
          status: {
            dirty: true,
            originalValues: {
              username: 'Bobert',
              id: 1
            },
            type: 'ERROR',
            requestedAt: 2,
            syncedAt: 1,
            httpCode: 404,
            error: {
              message: 'Not Found'
            }
          },
          metadata: {
            type: 'COMPLETE'
          }
        })
      });
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, this.initialState);
      this.store.dispatch(this.clearUserEdit());
    });
    describe('when the user tries to clear the edit', function () {
      expectToClearResource();
    });
  });

  function expectToClearResource() {
    it('then changes the status type to SUCCESS', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _Statuses.SUCCESS);
    });
    it('then does NOT change the syncedAt value in the status', function () {
      (0, _resourceAssertions.expectToNotChangeResourceItemStatus)(this, RESOURCE_NAME, 'syncedAt');
    });
    it('then does NOT change the requestedAt value in the status', function () {
      (0, _resourceAssertions.expectToNotChangeResourceItemStatus)(this, RESOURCE_NAME, 'requestedAt');
    });
    it('then clears the original values', function () {
      (0, _resourceAssertions.expectToClearResourceItemStatus)(this, RESOURCE_NAME, 'originalValues');
    });
    it('then clears the dirty bit', function () {
      (0, _resourceAssertions.expectToClearResourceItemStatus)(this, RESOURCE_NAME, 'dirty');
    });
    it('then changes the values back to originalValues', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.initialState.items[_EmptyKey["default"]].status.originalValues);
    });
  }
});