"use strict";

var _index = require("../../../index");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Clear new reducer:', function () {
  beforeAll(function () {
    var _resource = (0, _index.resource)({
      name: RESOURCE_NAME
    }, {
      clearNewItem: true
    }),
        reducers = _resource.reducers,
        clearNewUser = _resource.actionCreators.clearNewItem;

    this.clearNewUser = clearNewUser;
    this.reducers = reducers;
  });
  describe('when the resource is NOT new', function () {
    beforeAll(function () {
      this.initialState = {
        items: _defineProperty({}, _EmptyKey["default"], {
          values: {
            id: 2,
            username: 'Jill'
          },
          status: {
            type: _index.SUCCESS
          }
        }),
        newItemKey: null
      };
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, this.initialState);
      this.store.dispatch(this.clearNewUser());
    });
    it('then does NOT change the resource\'s status', function () {
      (0, _resourceAssertions.expectToNotChangeResourceItemStatus)(this, RESOURCE_NAME);
    });
  });
  describe('when the resource is new', function () {
    beforeAll(function () {
      this.initialState = {
        values: {
          id: 2,
          username: 'Jill'
        },
        status: {
          type: _index.NEW
        }
      };
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, {
        items: _defineProperty({}, _EmptyKey["default"], this.initialState),
        newItemKey: _EmptyKey["default"],
        selectionMap: _defineProperty({}, _EmptyKey["default"], true)
      });
      this.store.dispatch(this.clearNewUser());
    });
    it('then clears the newItemKey', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, null);
    });
    it('then removes the resource\'s key from the selectionMap', function () {
      (0, _resourceAssertions.expectToChangeSelectionMapTo)(this, RESOURCE_NAME, {});
    });
  });
});