"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Create reducer:', function () {
  beforeAll(function () {
    var _resource = (0, _index.resource)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id'
    }, {
      createItem: true
    }),
        reducers = _resource.reducers,
        createUser = _resource.actionCreators.createItem;

    this.createUser = createUser;
    this.reducers = reducers;
    this.newValues = {
      username: 'Bob'
    };
    this.responseValues = _objectSpread(_objectSpread({}, this.newValues), {}, {
      id: 1
    });
  });
  describe('Given no actions have come before it', function () {
    expectToHandleSuccessAndFailedRequests(_objectSpread({}, _index.RESOURCES));
  });
  describe('Given a NEW action has come before it', function () {
    expectToHandleSuccessAndFailedRequests({
      items: _defineProperty({}, _EmptyKey["default"], {
        values: {
          username: 'Robert'
        },
        status: {
          type: _index.NEW
        }
      }),
      lists: {},
      newItemKey: _EmptyKey["default"]
    });
  });
  describe('when there is already an item in the store with the same key', function () {
    var _this2 = this;

    describe('before the request has completed', function () {
      var _this = this;

      beforeAll(function () {
        spyOn(console, 'warn');
        setUpBeforeRequest(this, {
          items: _defineProperty({}, _EmptyKey["default"], {
            values: {
              username: 'Robert'
            },
            status: {
              type: _index.SUCCESS
            }
          }),
          lists: _defineProperty({}, _EmptyKey["default"], {
            positions: [_EmptyKey["default"]],
            status: {
              type: null
            }
          }),
          newItemKey: null
        }, this.newValues);
      });
      it('then replaces the existing item\'s values', function () {
        (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.newValues);
      });
      it('then sets the status of the existing item to CREATING', function () {
        (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.CREATING);
      });
      it('then sets the newItemKey to the temporary key', function () {
        (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, _EmptyKey["default"]);
      });
      afterAll(function () {
        return tearDown(_this);
      });
    });
    describe('when the request has completed', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        this.responseValues = {
          id: 2,
          username: 'Bob'
        };
        setUpAfterRequestSuccess(this, {
          users: {
            items: _defineProperty({}, _EmptyKey["default"], {
              values: {
                username: 'Robert'
              },
              status: {
                type: _index.SUCCESS
              }
            }),
            lists: _defineProperty({}, _EmptyKey["default"], {
              positions: [_EmptyKey["default"]],
              status: {
                type: null
              }
            }),
            newItemKey: null
          }
        }, this.newValues, this.responseValues);
      });
      afterAll(function () {
        return tearDown(_this2);
      });
      it('then moves the item to the new ID and merges in values from the server', function () {
        (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.responseValues);
      });
      it('then sets the items status type to SUCCESS', function () {
        (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.SUCCESS);
      });
      it('then updates the newItemKey ', function () {
        (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, _EmptyKey["default"]);
      });
    });
  });
  describe('Given a createItem action that will succeed with a response that specifies \'errors\' at the top level', function () {
    describe('when the request has completed', function () {
      expectToCorrectlySetErrors(404);
    });
  });
  describe('Given a createItem action that will fail with a response that specifies \'errors\' at the top level', function () {
    describe('when the request has completed', function () {
      expectToCorrectlySetErrors(200);
    });
  });

  function expectToHandleSuccessAndFailedRequests(initialState) {
    describe('and the API request succeeds', function () {
      var _this4 = this;

      describe('before the request has completed', function () {
        var _this3 = this;

        beforeAll(function () {
          setUpBeforeRequest(this, initialState, this.newValues);
        });
        expectToRecordPendingCreate();
        afterAll(function () {
          return tearDown(_this3);
        });
      });
      describe('when the request has completed', function () {
        beforeAll(function () {
          setUpAfterRequestSuccess(this, initialState, this.newValues, this.responseValues);
        });
        expectToRecordCreateSuccess();
        afterAll(function () {
          return tearDown(_this4);
        });
      });
    });
    describe('and the API request errors', function () {
      var _this6 = this;

      describe('before the request has completed', function () {
        var _this5 = this;

        beforeAll(function () {
          setUpBeforeRequest(this, initialState, this.newValues);
        });
        expectToRecordPendingCreate();
        afterAll(function () {
          return tearDown(_this5);
        });
      });
      describe('when the request has completed', function () {
        beforeAll(function () {
          setUpAfterRequestFailure(this, initialState, this.newValues);
        });
        expectToRecordCreateError();
        afterAll(function () {
          return tearDown(_this6);
        });
      });
    });
  }

  function expectToRecordCreateSuccess() {
    it('then moves the item to the new ID and merges in values from the server', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, {
        id: 1,
        username: 'Bob'
      });
    });
    it('then sets the items status type to SUCCESS', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.SUCCESS);
    });
    it('then updates the newItemKey ', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
  }

  function expectToRecordPendingCreate() {
    it('then adds a new item with the correct values', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, {
        username: 'Bob'
      });
    });
    it('then adds a new item with a status type of CREATING', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.CREATING);
    });
    it('then does NOT add the temporary key to the default list', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesList)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
    it('then sets the newItemKey to the temporary key', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
  }

  function expectToRecordCreateError() {
    it('then DOES NOT move the item from its temporary key', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.newValues);
    });
    it('then sets the items status type to ERROR', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.ERROR);
    });
    it('then sets the items status httpCode', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'httpCode', 404);
    });
    it('then sets the syncedAt attribute', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME);
    });
    it('then merges in the server\'s response into the status', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'error', {
        message: 'Not Found'
      });
    });
    it('then DOES NOT update the newItemKey', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
  }

  function expectToCorrectlySetErrors() {
    var status = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 200;
    beforeAll(function () {
      this.newValues = {
        username: 'Bob'
      };
      setUpAfterRequestFailure(this, _objectSpread({}, _index.RESOURCES), this.newValues, {
        body: {
          errors: [{
            message: 'Not Found'
          }]
        },
        status: status
      });
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then DOES NOT move the item from its temporary key', function () {
      expect(this.store.getState().users.items[_EmptyKey["default"]].values).toEqual(this.newValues);
    });
    it('then sets the items status type to ERROR', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.ERROR);
    });
    it('then sets the items status httpCode', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'httpCode', status);
    });
    it('then does NOT set the syncedAt attribute', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME);
    });
    it('then merges in the server\'s response into the status', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'error', {
        message: 'Not Found'
      });
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'errors', [{
        message: 'Not Found'
      }]);
    });
    it('then DOES NOT update the newItemKey', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
  }

  function setUpBeforeRequest(context, initialState, newValues, actionCreatorOptions) {
    _fetchMock["default"].post('http://test.com/users', new Promise(_nop["default"]));

    setupState(context, initialState, newValues, actionCreatorOptions);
  }

  function setUpAfterRequestSuccess(context, initialState, initialValues) {
    var responseValues = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : initialValues;
    var actionCreatorOptions = arguments.length > 4 ? arguments[4] : undefined;

    _fetchMock["default"].post('http://test.com/users', {
      body: responseValues
    });

    setupState(context, initialState, initialValues, actionCreatorOptions);
  }

  function setUpAfterRequestFailure(context, initialState, newValues) {
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"].post('http://test.com/users', options);

    setupState(context, initialState, newValues);
  }

  function setupState(context, initialState, newValues, actionCreatorOptions) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.createUser(newValues, actionCreatorOptions));
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});