"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('EditItem reducer:', function () {
  beforeAll(function () {
    var _resource = (0, _index.resource)({
      name: 'users',
      keyBy: 'id'
    }, {
      editItem: true
    }),
        reducers = _resource.reducers,
        editUser = _resource.actionCreators.editItem;

    this.editUser = editUser;
    this.reducers = reducers;
    this.newValues = {
      username: 'Bob'
    };
  });
  describe('Given there is NO resource in the store', function () {
    expectToWarnButCreateNewItem();
  });
  describe('Given a resource is in the store with a status of SUCCESS', function () {
    expectToStartEditingWithNewValues({
      initialStatus: {
        type: _index.ERROR,
        error: {
          code: 'INVALID_RESOURCE'
        }
      }
    });
  });
  describe('Given a new resource item exists in the store and the editItem action controller is called to update it', function () {
    beforeAll(function () {
      spyOn(console, 'warn');
      setupState(this, getInitialState({
        type: _index.NEW
      }), this.newValues);
    });
    it('then warns that the editItem action controller is not intended to edit new resource items', function () {
      expect(console.warn).toHaveBeenCalledWith('Redux and the REST: Use a editItem() to edit new items that have not yet been saved to an external API. Update ignored.');
    });
    it('then does not update the new resource item\'s values or state', function () {
      (0, _resourceAssertions.expectToNotChangeResourceItemValues)(this, RESOURCE_NAME);
      (0, _resourceAssertions.expectToNotChangeResourceItemStatus)(this, RESOURCE_NAME);
    });
  });

  function expectToWarnButCreateNewItem() {
    beforeAll(function () {
      spyOn(console, 'warn');
      setupState(this, _objectSpread({}, _index.RESOURCES), this.newValues);
    });
    it('then warns about the missing item', function () {
      expect(console.warn).toHaveBeenCalledWith('Redux and the REST: Use newItem() to create a new item or check the arguments passed to editItem(). (A new item was created to contain the edit.)');
    });
    it('then adds an item with the correct values', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.newValues);
    });
    it('then adds an item with a status of EDITING', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.EDITING);
    });
  }

  function expectToStartEditingWithNewValues(options) {
    beforeAll(function () {
      setupState(this, getInitialState(options.initialStatus), this.newValues);
    });
    it('then updates the item\'s values', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.newValues);
    });
    it('then changes the item\'s status to EDITING', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, {
        type: _index.EDITING,
        dirty: true,
        originalValues: {
          username: 'Robert'
        }
      });
    });
  }

  function setupState(context, initialState, newValues) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.editUser(newValues));
  }

  function getInitialState(status) {
    var base = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
      items: _defineProperty({}, _EmptyKey["default"], {
        values: {
          username: 'Robert'
        },
        status: status
      })
    });

    if (status.type === _index.NEW) {
      base.newItemKey = _EmptyKey["default"];
    }

    return base;
  }
});