"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Edit new reducer:', function () {
  beforeAll(function () {
    var _resource = (0, _index.resource)({
      name: 'users',
      keyBy: 'id'
    }, {
      editNewItem: true
    }),
        reducers = _resource.reducers,
        editNewUser = _resource.actionCreators.editNewItem;

    this.editNewUser = editNewUser;
    this.reducers = reducers;
    this.newValues = {
      username: 'Bob'
    };
  });
  describe('Given there is no existing resource item', function () {
    describe('When the action creator is called', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        setupState(this, _objectSpread({}, _index.RESOURCES), this.newValues);
      });
      it('then warns about the missing item', function () {
        expect(console.warn).toHaveBeenCalledWith('Redux and the REST: Use newItem() to create a new item first, or check the arguments passed to editItem(). Update ignored.');
      });
    });
  });
  describe('Given the resource item is NOT new', function () {
    beforeAll(function () {
      spyOn(console, 'warn');
      setupState(this, getInitialState(_index.SUCCESS), this.newValues);
    });
    it('then warns attempting to edit a resource item that is not NEW', function () {
      expect(console.warn).toHaveBeenCalledWith('Redux and the REST: Use newItem() to create a new item first, or check the arguments passed to editItem(). Update ignored.');
    });
    it('then doesn\'t update the item', function () {
      (0, _resourceAssertions.expectToNotChangeResourceItemStatus)(this, RESOURCE_NAME);
      (0, _resourceAssertions.expectToNotChangeResourceItemValues)(this, RESOURCE_NAME);
    });
  });
  describe('Given the resource item is new', function () {
    beforeAll(function () {
      setupState(this, getInitialState(_index.NEW), this.newValues);
    });
    it('then updates the item\'s values', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.newValues);
    });
    it('then leaves the item\'s status as NEW', function () {
      (0, _resourceAssertions.expectToNotChangeResourceItemStatus)(this, RESOURCE_NAME);
    });
  });

  function setupState(context, initialState, newValues) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.editNewUser(newValues));
  }

  function getInitialState(statusType) {
    var base = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
      items: _defineProperty({}, _EmptyKey["default"], {
        values: {
          username: 'Jane'
        },
        status: {
          type: statusType
        }
      })
    });

    if (statusType === _index.NEW) {
      base.newItemKey = _EmptyKey["default"];
    }

    return base;
  }
});