"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Fetch reducers:', function () {
  var _this = this;

  beforeAll(function () {
    var _resource = (0, _index.resource)({
      name: 'users',
      url: 'http://test.com/users',
      keyBy: 'id'
    }, {
      fetchItem: true
    }),
        reducers = _resource.reducers,
        fetchUser = _resource.actionCreators.fetchItem;

    this.fetchUser = fetchUser;
    this.reducers = reducers;
  });
  describe('Given no actions have come before it', function () {
    beforeAll(function () {
      this.url = 'http://test.com/users';
    });
    expectToHandleSuccessAndFailure(_this.url);
  });
  describe('Given another fetchItem action has come before it', function () {
    var _this2 = this;

    beforeAll(function () {
      this.url = 'http://test.com/users';
      this.resourceBefore = {
        items: _defineProperty({}, _EmptyKey["default"], {
          values: {
            id: 1,
            username: 'Bob'
          },
          status: {
            type: _index.SUCCESS
          }
        })
      };
      this.id = 1;
    });
    describe('before the request has completed', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, _objectSpread(_objectSpread({}, _index.RESOURCES), this.resourceBefore), this.url);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then sets the status of the item to FETCHING', function () {
        (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.FETCHING);
      });
      it('then does NOT clear the item\'s values', function () {
        (0, _resourceAssertions.expectToNotChangeResourceItemValues)(this, RESOURCE_NAME);
      });
      it('then does NOT add the item to the default list', function () {
        expect((0, _resourceAssertions.resourceDefinition)(this, RESOURCE_NAME).lists).toEqual({});
      });
    });
    describe('When the API request succeeds', function () {
      expectToMergeInResponseBody(_this2.url, _objectSpread(_objectSpread({}, _index.RESOURCES), _this2.resourceBefore));
    });
    describe('when the API request errors', function () {
      expectToMergeInError(_this2.url, _objectSpread(_objectSpread({}, _index.RESOURCES), _this2.resourceBefore));
    });
  });
  describe('Given a fetchItem action that will succeed with a response that specifies \'errors\' at the top level', function () {
    expectToMergeInMultipleErrors({
      body: {
        errors: ['Not Found']
      },
      status: 200
    });
  });
  describe('Given a fetchItem action that will fail with a response that specifies \'errors\' at the top level', function () {
    expectToMergeInMultipleErrors({
      body: {
        errors: ['Not Found']
      },
      status: 400
    });
  });

  function expectToHandleSuccessAndFailure(url) {
    describe('before the request has completed', function () {
      expectToSetFetchingState(url);
    });
    describe('and the API request succeeds', function () {
      expectToMergeInResponseBody(url, _objectSpread({}, _index.RESOURCES));
    });
    describe('when the API request errors', function () {
      expectToMergeInError(url, _objectSpread({}, _index.RESOURCES));
    });
  }

  function expectToSetFetchingState(url) {
    beforeAll(function () {
      setUpBeforeRequest(this, _objectSpread({}, _index.RESOURCES), url);
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then adds a new item with a status of FETCHING', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.FETCHING);
    });
    it('then adds a new item with empty values', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, {});
    });
    it('then does NOT add the item to the default list', function () {
      expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual({});
    });
  }

  function expectToMergeInResponseBody(url, initialState) {
    beforeAll(function () {
      this.newValues = {
        username: 'Bob'
      };
      setUpAfterRequestSuccess(this, initialState, url, {
        body: this.newValues
      });
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then changes the items\'s status type to SUCCESS', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.SUCCESS);
    });
    it('then sets the item\'s values from the response', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.newValues);
    });
  }

  function expectToMergeInError(url, initialState) {
    beforeAll(function () {
      this.options = {
        body: {
          error: 'Not Found'
        },
        status: 404
      };
      setUpAfterRequestFailure(this, initialState, url, this.options);
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then changes the items\'s status type to ERROR', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.ERROR);
    });
    it('then sets the syncedAt attribute', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME);
    });
    it('then updates the item\'s status httpCode', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'httpCode', 404);
    });
    it('then sets the item\'s status error from the response', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'error', {
        message: this.options.body.error
      });
    });
    it('then does NOT set the item\'s values from the response', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, {});
    });
  }

  function expectToMergeInMultipleErrors(options) {
    describe('when the request has completed', function () {
      beforeAll(function () {
        this.id = 1;
        setUpAfterRequestFailure(this, _objectSpread({}, _index.RESOURCES), 'http://test.com/users', options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the items\'s status type to ERROR', function () {
        (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.ERROR);
      });
      it('then sets the syncedAt attribute', function () {
        (0, _resourceAssertions.expectToChangeResourceItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME);
      });
      it('then updates the item\'s status httpCode', function () {
        (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'httpCode', options.status);
      });
      it('then sets the item\'s status error from the response', function () {
        var errors = options.body.errors;

        var _errors = _slicedToArray(errors, 1),
            errorMessage = _errors[0];

        (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'error', errorMessage);
        (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'errors', [errorMessage]);
      });
    });
  }

  function setUpBeforeRequest(context, initialState, url) {
    _fetchMock["default"].get(url, new Promise(_nop["default"]));

    setupState(context, initialState);
  }

  function setUpAfterRequestSuccess(context, initialState, url) {
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
      body: {}
    };

    _fetchMock["default"].get(url, options);

    setupState(context, initialState);
  }

  function setUpAfterRequestFailure(context, initialState, url) {
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"].get(url, options);

    setupState(context, initialState);
  }

  function setupState(context, initialState) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.fetchUser());
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});