"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('New reducer:', function () {
  beforeAll(function () {
    var _resource = (0, _index.resource)({
      name: 'users',
      keyBy: 'id'
    }, {
      newItem: true
    }),
        reducers = _resource.reducers,
        newUser = _resource.actionCreators.newItem;

    this.newUser = newUser;
    this.reducers = reducers;
  });
  describe('Given the resource does not yet exist', function () {
    expectToAddNewItem();
  });
  describe('Given the resource already has a status of NEW', function () {
    expectToChangeToNewItem(_index.NEW);
  });
  describe('Given the resource already has a status of SUCCESS', function () {
    expectToChangeToNewItem(_index.SUCCESS);
  });
  describe('Given the resource already has a status of CREATING', function () {
    expectToChangeToNewItem(_index.CREATING);
  });
  describe('Given the resource already has a status of EDITING', function () {
    expectToChangeToNewItem(_index.EDITING);
  });
  describe('Given the resource already has a status of UPDATING', function () {
    expectToChangeToNewItem(_index.UPDATING);
  });
  describe('Given the resource already has a status of ERROR', function () {
    expectToChangeToNewItem(_index.ERROR);
  });
  describe('Given the resource already has a status of DESTROYING', function () {
    expectToChangeToNewItem(_index.DESTROYING);
  });
  describe('Given the resource already has a status of DESTROY_ERROR', function () {
    expectToChangeToNewItem(_index.DESTROY_ERROR);
  });

  function expectToAddNewItem() {
    beforeAll(function () {
      this.newValues = {
        username: 'Jill'
      };
      setupState(this, _objectSpread({}, _index.RESOURCES), this.newValues);
    });
    it('then adds the new item', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.newValues);
    });
    it('then sets the status type of the new item to NEW', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.NEW);
    });
    it('then sets the newItemKey', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
    it('then does NOT add the key of the new item to the default list', function () {
      expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual({});
    });
  }

  function expectToChangeToNewItem(status) {
    beforeAll(function () {
      this.previousItemId = 'previous';
      this.newValues = {
        username: 'Jill'
      };

      var initialState = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: _defineProperty({}, _EmptyKey["default"], {
          values: {
            username: 'Bob'
          },
          status: {
            type: status
          }
        }),
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [_EmptyKey["default"]]
        })
      });

      if (status === _index.NEW) {
        initialState.newItemKey = _EmptyKey["default"];
      }

      spyOn(console, 'warn');
      setupState(this, initialState, this.newValues);
    });
    it('then changes the values of then existing item', function () {
      (0, _resourceAssertions.expectToChangeResourceItemValuesTo)(this, RESOURCE_NAME, this.newValues);
    });
    it('then sets the item\'s status type to NEW', function () {
      (0, _resourceAssertions.expectToChangeResourceItemStatusTo)(this, RESOURCE_NAME, 'type', _index.NEW);
    });
    it('then sets the newItemKey to point to the new item', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
  }

  function setupState(context, initialState, newValues, options) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);

    if (options) {
      context.store.dispatch(context.newUser(newValues, options));
    } else {
      context.store.dispatch(context.newUser(newValues));
    }
  }
});