"use strict";

var _buildStore3 = _interopRequireDefault(require("../../helpers/buildStore"));

var _Statuses = require("../../../constants/Statuses");

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Clear editItem reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: RESOURCE_NAME
    }, {
      clearItemEdit: true
    }),
        reducers = _resources.reducers,
        clearUserEdit = _resources.actionCreators.clearItemEdit;

    this.clearUserEdit = clearUserEdit;
    this.reducers = reducers;
  });
  describe('Given the user attempts to clear a resource item that isn\'t in the store', function () {
    beforeAll(function () {
      this.initialState = {
        items: {
          1: {
            values: {
              id: 1,
              username: 'Jill'
            },
            status: {
              type: _Statuses.EDITING,
              syncedAt: 2,
              requestedAt: 1,
              dirty: 1,
              originalValues: {
                id: 1,
                username: 'Jillian'
              }
            }
          }
        },
        selectionMap: {},
        newItemKey: null
      };
      setupInitialState(this, RESOURCE_NAME, this.initialState, 100);
    });
    it('then DOES nothing', function () {
      expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME)).toEqual(this.initialState);
    });
  });
  describe('Given the user attempts to clear a resource item that is NOT being edited', function () {
    beforeAll(function () {
      this.initialState = {
        items: {
          1: {
            values: {
              id: 2,
              username: 'Bob'
            },
            status: {
              type: _Statuses.SUCCESS,
              syncedAt: 2,
              requestedAt: 1
            }
          }
        },
        selectionMap: {},
        newItemKey: null
      };
      setupInitialState(this, RESOURCE_NAME, this.initialState, 1);
    });
    it('then does NOT change the status', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, 1);
    });
    it('then does NOT change the values', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, 1);
    });
  });
  describe('Given the user attempts to clear a resource item that is being edited', function () {
    beforeAll(function () {
      this.initialState = {
        items: {
          1: {
            values: {
              id: 1,
              username: 'Jill'
            },
            status: {
              type: _Statuses.EDITING,
              syncedAt: 2,
              requestedAt: 1,
              dirty: 1,
              originalValues: {
                id: 1,
                username: 'Jillian'
              }
            }
          }
        },
        selectionMap: {},
        newItemKey: null
      };
      setupInitialState(this, RESOURCE_NAME, this.initialState, 1);
    });
    expectToResetResourceItem();
  });
  describe('Given an update has failed to apply for a resource item', function () {
    beforeAll(function () {
      this.initialState = {
        items: {
          1: {
            values: {
              username: 'Robert',
              id: 1
            },
            status: {
              dirty: true,
              originalValues: {
                username: 'Bobert',
                id: 1
              },
              type: 'ERROR',
              requestedAt: 2,
              syncedAt: 1,
              httpCode: 404,
              error: {
                message: 'Not Found'
              }
            },
            metadata: {
              type: 'COMPLETE'
            }
          }
        },
        lists: {},
        selectionMap: {},
        newItemKey: null
      };
      setupInitialState(this, RESOURCE_NAME, this.initialState, 1);
    });
    expectToResetResourceItem();
    it('then clears the error details', function () {
      (0, _resourceAssertions.expectToClearResourcesItemStatus)(this, RESOURCE_NAME, 1, 'error');
      (0, _resourceAssertions.expectToClearResourcesItemStatus)(this, RESOURCE_NAME, 1, 'httpCode');
    });
  });

  function expectToResetResourceItem() {
    it('then changes the status type to SUCCESS', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 1, 'type', _Statuses.SUCCESS);
    });
    it('then does NOT change the syncedAt value in the status', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, 1, 'syncedAt');
    });
    it('then does NOT change the requestedAt value in the status', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, 1, 'requestedAt');
    });
    it('then clears the original values', function () {
      (0, _resourceAssertions.expectToClearResourcesItemStatus)(this, RESOURCE_NAME, 1, 'originalValues');
    });
    it('then clears the dirty bit', function () {
      (0, _resourceAssertions.expectToClearResourcesItemStatus)(this, RESOURCE_NAME, 1, 'dirty');
    });
    it('then changes the values back to originalValues', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, 1, this.initialState.items[1].status.originalValues);
    });
  }

  function setupInitialState(context, resourcesName, initialState, userId) {
    context.store = (0, _buildStore3["default"])(_defineProperty({}, resourcesName, _objectSpread(_objectSpread({}, initialState), {}, {
      newItemKey: null
    })), _defineProperty({}, resourcesName, context.reducers));
    context.store.dispatch(context.clearUserEdit(userId));
  }
});