"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Clear new reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: RESOURCE_NAME
    }, {
      clearNewItem: true
    }),
        reducers = _resources.reducers,
        clearNewUser = _resources.actionCreators.clearNewItem;

    this.clearNewUser = clearNewUser;
    this.reducers = reducers;
  });
  beforeAll(function () {
    this.initialState = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
      items: {
        1: {
          values: {
            id: 2,
            username: 'Jill'
          },
          status: {
            type: _index.NEW
          }
        },
        'temp': {
          values: {
            id: 2,
            username: 'Jill'
          },
          status: {
            type: _index.NEW
          }
        }
      },
      lists: _defineProperty({}, _EmptyKey["default"], {
        positions: ['temp', 1],
        status: {
          type: _index.EDITING
        }
      }),
      selectionMap: {
        temp: true,
        1: true
      }
    });
  });
  describe('when there is NO new resource', function () {
    beforeAll(function () {
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, this.initialState);
      this.store.dispatch(this.clearNewUser());
    });
    it('then DOES NOT clear the newItemKey', function () {
      (0, _resourceAssertions.expectToNotChangeNewItemKey)(this, RESOURCE_NAME);
    });
    it('then DOES NOT remove the resource\'s key from the selectionMap', function () {
      (0, _resourceAssertions.expectToNotChangeSelectionMap)(this, RESOURCE_NAME);
    });
    it('then DOES NOT remove the resource\'s key from any lists', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesList)(this, RESOURCE_NAME, _EmptyKey["default"], 'positions');
    });
  });
  describe('when there is a new resource', function () {
    beforeAll(function () {
      this.initialState = _objectSpread(_objectSpread({}, this.initialState), {}, {
        newItemKey: 'temp'
      });
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, this.initialState);
      this.store.dispatch(this.clearNewUser());
    });
    it('then clears the newItemKey', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, null);
    });
    it('then removes the resource\'s key from the selectionMap', function () {
      (0, _resourceAssertions.expectToChangeSelectionMapTo)(this, RESOURCE_NAME, {
        1: true
      });
    });
    it('then removes the resource\'s key from any lists', function () {
      (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, _EmptyKey["default"], [1]);
    });
  });
});