"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Clear selected reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: RESOURCE_NAME
    }, {
      clearSelectedItems: true
    }),
        reducers = _resources.reducers,
        clearSelectedUsers = _resources.actionCreators.clearSelectedItems;

    this.clearSelectedUsers = clearSelectedUsers;
    this.reducers = reducers;
    this.initialState = {
      items: {
        1: {
          values: {
            id: 1,
            username: 'Bob'
          },
          status: {
            type: _index.SUCCESS
          }
        },
        2: {
          values: {
            id: 2,
            username: 'Jill'
          },
          status: {
            type: _index.SUCCESS
          }
        }
      },
      lists: _defineProperty({}, _EmptyKey["default"], {
        positions: [1, 2],
        status: {
          type: null
        }
      }),
      newItemKey: null
    };
  });
  describe('when there are no resources selected', function () {
    beforeAll(function () {
      this.initialState = _objectSpread(_objectSpread({}, this.initialState), {}, {
        selectionMap: {}
      });
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, this.initialState);
      this.store.dispatch(this.clearSelectedUsers());
    });
    it('then does NOT change the selectionMap', function () {
      (0, _resourceAssertions.expectToNotChangeSelectionMap)(this, RESOURCE_NAME);
    });
  });
  describe('when there are resources selected', function () {
    beforeAll(function () {
      this.initialState = _objectSpread(_objectSpread({}, this.initialState), {}, {
        selectionMap: {
          1: true,
          2: true
        }
      });
      (0, _resourceAssertions.setupInitialState)(this, RESOURCE_NAME, this.initialState);
      this.store.dispatch(this.clearSelectedUsers());
    });
    it('then clears the selectionMap', function () {
      (0, _resourceAssertions.expectToChangeSelectionMapTo)(this, RESOURCE_NAME, {});
    });
    it('then DOES NOT remove the selected resources', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItem)(this, RESOURCE_NAME, 1);
    });
    it('then DOES NOT remove the selected resources from lists', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesList)(this, RESOURCE_NAME, _EmptyKey["default"], 'positions', [1, 2]);
    });
  });
});