"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

var _getListKey = _interopRequireDefault(require("../../../action-creators/helpers/getListKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Create reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id'
    }, {
      createItem: true
    }),
        reducers = _resources.reducers,
        createUser = _resources.actionCreators.createItem;

    this.createUser = createUser;
    this.reducers = reducers;
    this.newValues = {
      username: 'Bob'
    };
    this.responseValues = _objectSpread(_objectSpread({}, this.newValues), {}, {
      id: 1
    });
  });
  describe('Given no actions have come before it', function () {
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToHandleSuccessAndFailedRequests(_objectSpread({}, _index.RESOURCES), 'temp');
    });
  });
  describe('Given a NEW action has come before it', function () {
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToHandleSuccessAndFailedRequests({
        items: {
          temp: {
            values: {
              username: 'Robert'
            },
            status: {
              type: _index.NEW
            }
          }
        },
        lists: {},
        newItemKey: 'temp'
      }, {
        id: 'temp'
      });
    });
  });
  describe('when there is already an item in the store with the same key', function () {
    describe('before the request has completed', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        setUpBeforeRequest(this, {
          items: {
            1: {
              values: {
                username: 'Robert'
              },
              status: {
                type: _index.SUCCESS
              }
            }
          },
          lists: _defineProperty({}, _EmptyKey["default"], {
            positions: [1],
            status: {
              type: null
            }
          }),
          newItemKey: null
        }, 1, this.newValues);
      });
      it('then warns about the collision', function () {
        expect(console.warn).toHaveBeenCalledWith('Redux and the REST: CREATE_USER has the same key \'1\' as an existing item. Use updateItem() to update an existing item, or ensure the new item has a unique temporary key. (The create request was still sent to the server.)');
      });
      it('then replaces the existing item\'s values', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, 1, this.newValues);
      });
      it('then sets the status of the existing item to CREATING', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 1, 'type', _index.CREATING);
      });
      it('then sets the newItemKey to the temporary key', function () {
        (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, 1);
      });
      afterAll(function () {
        tearDown(this);
      });
    });
    describe('when the request has completed', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        this.responseValues = {
          id: 2,
          username: 'Bob'
        };
        setUpAfterRequestSuccess(this, {
          users: {
            items: {
              1: {
                values: {
                  username: 'Robert'
                },
                status: {
                  type: _index.SUCCESS
                }
              }
            },
            lists: _defineProperty({}, _EmptyKey["default"], {
              positions: [1],
              status: {
                type: null
              }
            }),
            newItemKey: null
          }
        }, 1, this.newValues, this.responseValues);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then moves the item to the new ID and merges in values from the server', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.responseValues.id, this.responseValues);
      });
      it('then sets the items status type to SUCCESS', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.responseValues.id, 'type', _index.SUCCESS);
      });
      it('then updates the newItemKey ', function () {
        (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, this.responseValues.id);
      });
    });
  });
  describe('when the push lists operator is used', function () {
    expectToCorrectlyApplyOperator('push', ['temp'], [2], [1, 'temp'], [1, 2]);
  });
  describe('when the unshift lists operator is used', function () {
    expectToCorrectlyApplyOperator('unshift', ['temp'], [2], ['temp', 1], [2, 1]);
  });
  describe('when the invalidate lists operator is used', function () {
    expectToCorrectlyApplyOperator('invalidate', [], [], [], []);
  });
  describe('Given a createItem action that will succeed with a response that specifies \'errors\' at the top level', function () {
    describe('when the request has completed', function () {
      expectToCorrectlySetErrors(404);
    });
  });
  describe('Given a createItem action that will fail with a response that specifies \'errors\' at the top level', function () {
    describe('when the request has completed', function () {
      expectToCorrectlySetErrors(200);
    });
  });

  function expectToHandleSuccessAndFailedRequests(initialState, idArgs) {
    describe('and the API request succeeds', function () {
      describe('before the request has completed', function () {
        beforeAll(function () {
          setUpBeforeRequest(this, initialState, idArgs, this.newValues);
        });
        expectToRecordPendingCreate();
        afterAll(function () {
          tearDown(this);
        });
      });
      describe('when the request has completed', function () {
        beforeAll(function () {
          setUpAfterRequestSuccess(this, initialState, idArgs, this.newValues, this.responseValues);
        });
        expectToRecordCreateSuccess();
        afterAll(function () {
          tearDown(this);
        });
      });
    });
    describe('and the API request errors', function () {
      describe('before the request has completed', function () {
        beforeAll(function () {
          setUpBeforeRequest(this, initialState, idArgs, this.newValues);
        });
        expectToRecordPendingCreate();
        afterAll(function () {
          tearDown(this);
        });
      });
      describe('when the request has completed', function () {
        beforeAll(function () {
          setUpAfterRequestFailure(this, initialState, idArgs, this.newValues);
        });
        expectToRecordCreateError();
        afterAll(function () {
          tearDown(this);
        });
      });
    });
  }

  function expectToRecordCreateSuccess() {
    it('then moves the item to the new ID and merges in values from the server', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, 1, {
        id: 1,
        username: 'Bob'
      });
    });
    it('then sets the items status type to SUCCESS', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 1, 'type', _index.SUCCESS);
    });
    it('then updates the newItemKey ', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, 1);
    });
  }

  function expectToRecordPendingCreate() {
    it('then adds a new item with the correct values', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, 'temp', {
        username: 'Bob'
      });
    });
    it('then adds a new item with a status type of CREATING', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 'temp', 'type', _index.CREATING);
    });
    it('then does NOT add the temporary key to the default list', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesList)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
    it('then sets the newItemKey to the temporary key', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, 'temp');
    });
  }

  function expectToRecordCreateError() {
    it('then DOES NOT move the item from its temporary key', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, 'temp', this.newValues);
    });
    it('then sets the items status type to ERROR', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 'temp', 'type', _index.ERROR);
    });
    it('then sets the items status httpCode', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 'temp', 'httpCode', 404);
    });
    it('then sets the syncedAt attribute', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, 'temp');
    });
    it('then merges in the server\'s response into the status', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 'temp', 'error', {
        message: 'Not Found'
      });
    });
    it('then DOES NOT update the newItemKey', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, 'temp');
    });
  }

  function expectToCorrectlyApplyOperator(operator, expectedIsolatedStateBefore, expectedIsolatedStateAfter, expectedCumulativeStateBefore, expectedCumulativeStateAfter) {
    beforeAll(function () {
      this.newValues = {
        username: 'Bob'
      };
      this.responseValues = {
        id: 2,
        username: 'Bob'
      };
      this.listKey = {
        order: 'newest'
      };
    });
    describe('and there are NO MATCHING lists', function () {
      beforeAll(function () {
        this.initialState = {
          items: {},
          lists: {
            'active=true': {
              positions: [],
              status: {
                type: _index.SUCCESS
              }
            }
          },
          newItemKey: null
        };
      });
      describe('before the request has completed', function () {
        beforeAll(function () {
          setUpBeforeRequest(this, this.initialState, 'temp', this.newValues, _defineProperty({}, operator, this.listKey));
        });
        afterAll(function () {
          tearDown(this);
        });
        it('then does not create any new lists', function () {
          expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual(this.initialState.lists);
        });
        it('then doesn\'t add the new temp key to any other existing lists', function () {
          (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, 'active=true', []);
        });
      });
    });
    describe('and there are lists with keys that exactly match', function () {
      beforeAll(function () {
        this.initialState = {
          items: {
            1: {
              id: 1,
              username: 'Jane'
            }
          },
          lists: {
            'active=true': {
              positions: [],
              status: {
                type: _index.SUCCESS
              }
            },
            'order=newest': {
              positions: [1],
              status: {
                type: _index.SUCCESS
              }
            }
          },
          newItemKey: null
        };
      });
      describe('before the request has completed', function () {
        beforeAll(function () {
          setUpBeforeRequest(this, this.initialState, 'temp', this.newValues, _defineProperty({}, operator, this.listKey));
        });
        afterAll(function () {
          tearDown(this);
        });
        it('then adds the new item\'s temp key to the matching lists', function () {
          (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, (0, _getListKey["default"])(this.listKey), expectedCumulativeStateBefore);
        });
        it('then doesn\'t add the new temp key to any other existing lists', function () {
          (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, 'active=true', []);
        });
      });
      expectToReplaceTempKeyInLists(operator, expectedCumulativeStateAfter);
    });
  }

  function expectToReplaceTempKeyInLists(operator, expectedIsolatedStateAfter) {
    describe('when the request has completed', function () {
      beforeAll(function () {
        setUpAfterRequestSuccess(this, this.initialState, 'temp', this.newValues, this.responseValues, _defineProperty({}, operator, this.listKey));
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then replaces all references to the temporary key with the new item key', function () {
        (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, (0, _getListKey["default"])(this.listKey), expectedIsolatedStateAfter);
      });
    });
  }

  function expectToCorrectlySetErrors() {
    var status = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 200;
    beforeAll(function () {
      this.newValues = {
        username: 'Bob'
      };
      setUpAfterRequestFailure(this, _objectSpread({}, _index.RESOURCES), 'temp', this.newValues, {
        body: {
          errors: [{
            message: 'Not Found'
          }]
        },
        status: status
      });
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then DOES NOT move the item from its temporary key', function () {
      expect(this.store.getState().users.items.temp.values).toEqual(this.newValues);
    });
    it('then sets the items status type to ERROR', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 'temp', 'type', _index.ERROR);
    });
    it('then sets the items status httpCode', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 'temp', 'httpCode', status);
    });
    it('then does NOT set the syncedAt attribute', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, 'temp');
    });
    it('then merges in the server\'s response into the status', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 'temp', 'error', {
        message: 'Not Found'
      });
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 'temp', 'errors', [{
        message: 'Not Found'
      }]);
    });
    it('then DOES NOT update the newItemKey', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, 'temp');
    });
  }

  function setUpBeforeRequest(context, initialState, id, newValues) {
    var actionCreatorOptions = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};

    _fetchMock["default"].post('http://test.com/users', new Promise(_nop["default"]));

    setupState(context, initialState, id, newValues, actionCreatorOptions);
  }

  function setUpAfterRequestSuccess(context, initialState, id, initialValues) {
    var responseValues = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : initialValues;
    var actionCreatorOptions = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};

    _fetchMock["default"].post('http://test.com/users', {
      body: responseValues
    });

    setupState(context, initialState, id, initialValues, actionCreatorOptions);
  }

  function setUpAfterRequestFailure(context, initialState, id, newValues) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"].post('http://test.com/users', options);

    setupState(context, initialState, id, newValues);
  }

  function setupState(context, initialState, id, newValues) {
    var actionCreatorOptions = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.createUser(id, newValues, actionCreatorOptions));
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});