"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

var RESOURCE_NAME = 'users';
describe('Deselect reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users',
      keyBy: 'id'
    }, {
      deselectItem: true
    }),
        reducers = _resources.reducers,
        deselectUser = _resources.actionCreators.deselectItem;

    this.reducers = reducers;
    this.deselectUser = deselectUser;
  });
  describe('when the resource is in the store', function () {
    describe('and there are no resources selected', function () {
      expectToCorrectlyDeselectItem({
        before: {},
        after: {}
      });
    });
    describe('and there are resources selected using the default value of true', function () {
      expectToCorrectlyDeselectItem({
        before: {
          1: true,
          2: true
        },
        after: {
          2: true
        }
      }, 1);
    });
    describe('and there are resources selected using custom values', function () {
      expectToCorrectlyDeselectItem({
        before: {
          1: {
            role: 'admin'
          },
          2: {
            role: 'admin'
          }
        },
        after: {
          2: {
            role: 'admin'
          }
        }
      }, 1);
    });
  });
  describe('and the item\'s id is passed as an object to the action creator', function () {
    describe('and there are resources selected using custom values', function () {
      expectToCorrectlyDeselectItem({
        before: {
          1: {
            role: 'admin'
          },
          2: {
            role: 'admin'
          }
        },
        after: {
          2: {
            role: 'admin'
          }
        }
      }, {
        id: 1
      });
    });
  });

  function expectToCorrectlyDeselectItem(_ref, id) {
    var before = _ref.before,
        after = _ref.after;
    beforeAll(function () {
      setupState(this, {
        items: {
          1: {
            values: {
              id: 1,
              username: 'Bob'
            },
            status: {
              type: _index.SUCCESS
            }
          },
          2: {
            values: {
              id: 2,
              username: 'Jill'
            },
            status: {
              type: _index.SUCCESS
            }
          }
        },
        lists: {},
        newItemKey: null,
        selectionMap: before
      }, id);
    });
    it('then removes the resource\'s key from the selectionMap', function () {
      (0, _resourceAssertions.expectToChangeSelectionMapTo)(this, RESOURCE_NAME, after);
    });
  }

  function setupState(context, initialState, id) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.deselectUser(id));
  }
});