"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var RESOURCE_NAME = 'users';
describe('DestroyItem reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id'
    }, {
      destroyItem: true
    }),
        reducers = _resources.reducers,
        destroyUser = _resources.actionCreators.destroyItem;

    this.reducers = reducers;
    this.destroyUser = destroyUser;
  });
  describe('Given the resources item is NOT in the store', function () {
    expectToHandleSuccessAndFailedRequests({
      items: {},
      lists: {},
      selectionMap: {},
      newItemKey: null
    }, 1, 1, 'Redux and the REST: DESTROY_USER\'s key \'1\' did not match any items in the store. (Destroy request was still sent to the server.)');
  });
  describe('Given a resources item is in the store with a status of NEW', function () {
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToHandleSuccessAndFailedRequests(getInitialState(_index.NEW), 1, 1, 'Redux and the REST: DESTROY_USER\'s key \'1\' matched a new item. Use clearNewItem() to clear items that haven\'t been saved to the server. (Destroy request was still sent to the server.)');
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToHandleSuccessAndFailedRequests(getInitialState(_index.NEW), {
        id: 1
      }, 1, 'Redux and the REST: DESTROY_USER\'s key \'1\' matched a new item. Use clearNewItem() to clear items that haven\'t been saved to the server. (Destroy request was still sent to the server.)');
    });
  });
  describe('Given a resources item is in the store with a status of DESTROYING', function () {
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToHandleSuccessAndFailedRequests(getInitialState(_index.DESTROYING), 1, 1, 'Redux and the REST: DESTROY_USER\'s key \'1\' matched a new item that has a pending DESTROY action. (Duplicate destroyItem request was still sent to the server.)');
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToHandleSuccessAndFailedRequests(getInitialState(_index.DESTROYING), {
        id: 1
      }, 1, 'Redux and the REST: DESTROY_USER\'s key \'1\' matched a new item that has a pending DESTROY action. (Duplicate destroyItem request was still sent to the server.)');
    });
  });
  describe('Given a resources item is in the store with a status of EDITING', function () {
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.EDITING), 1, 1);
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.EDITING), {
        id: 1
      }, 1);
    });
  });
  describe('Given a resources item is in the store with a status of CREATING', function () {
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.CREATING), 1, 1);
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.CREATING), {
        id: 1
      }, 1);
    });
  });
  describe('Given a resources item is in the store with a status of UPDATING', function () {
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.UPDATING), 1, 1);
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.UPDATING), {
        id: 1
      }, 1);
    });
  });
  describe('Given a resources item is in the store with a status of SUCCESS', function () {
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.SUCCESS), 1, 1);
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.SUCCESS), {
        id: 1
      }, 1);
    });
  });
  describe('Given a resources item is in the store with a status of ERROR', function () {
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.ERROR), 1, 1);
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(getInitialState(_index.ERROR), {
        id: 1
      }, 1);
    });
  });
  describe('Given a destroyItem action that will succeed with a response that specifies \'errors\' at the top level', function () {
    describe('when the request has completed', function () {
      expectToHandleErrorsCorrectly(1, 200);
    });
  });
  describe('Given a destroyItem action that will fail with a response that specifies \'errors\' at the top level', function () {
    describe('when the request has completed', function () {
      expectToHandleErrorsCorrectly(1, 404);
    });
  });

  function expectToHandleSuccessAndFailedRequests(initialState, params, id) {
    var warning = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;
    beforeAll(function () {
      spyOn(console, 'warn');
    });
    describe('before the request has completed', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, initialState, params, id);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then warns about the missing resource', function () {
        expect(console.warn).toHaveBeenCalledWith(warning);
      });
      it('then creates a new item and sets its status to DESTROYING', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'type', _index.DESTROYING);
      });
    });
    describe('when the API request succeeds', function () {
      var _this = this;

      beforeAll(function () {
        setUpAfterRequestSuccess(this, initialState, params, id);
      });
      afterAll(function () {
        return tearDown(_this);
      });
      it('then removes the item', function () {
        expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).items).toEqual({});
      });
    });
    describe('when the API request errors', function () {
      beforeAll(function () {
        setUpAfterRequestFailure(this, initialState, params, id);
      });
      afterAll(function () {
        tearDown(this);
      });
      expectToRecordDestroyError(id, 404);
    });
  }

  function expectToRecordDestroyError(id, status) {
    it('then updates the items\'s status to DESTROY_ERROR', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'type', _index.DESTROY_ERROR);
    });
    it('then sets the syncedAt attribute', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, id);
    });
    it('then updates the item\'s status httpCode', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'httpCode', status);
    });
  }

  function expectToHandleErrorsCorrectly(id, status) {
    beforeAll(function () {
      setUpAfterRequestFailure(this, getInitialState(_index.SUCCESS), id, id, {
        body: {
          errors: [{
            message: 'Not Found'
          }]
        },
        status: status
      });
    });
    afterAll(function () {
      tearDown(this);
    });
    expectToRecordDestroyError(id, status);
    it('then sets the item\'s errors', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'error', {
        message: 'Not Found'
      });
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'errors', [{
        message: 'Not Found'
      }]);
    });
  }

  function expectToHandleSuccessAndFailedRequestsForExistingResourcesItem(initialState, params, id) {
    describe('before the request has completed', function () {
      var _this2 = this;

      beforeAll(function () {
        setUpBeforeRequest(this, initialState, params, id);
      });
      afterAll(function () {
        return tearDown(_this2);
      });
      it('then creates a new item and sets its status to DESTROYING', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'type', _index.DESTROYING);
      });
      it('then does NOT remove the item from the selectionMap', function () {
        (0, _resourceAssertions.expectToNotChangeSelectionMap)(this, RESOURCE_NAME);
      });
      it('then does NOT clear the item from thew newItemKey', function () {
        (0, _resourceAssertions.expectToNotChangeNewItemKey)(this, RESOURCE_NAME);
      });
    });
    describe('when the API request succeeds', function () {
      var _this3 = this;

      beforeAll(function () {
        setUpAfterRequestSuccess(this, initialState, params, id);
      });
      afterAll(function () {
        return tearDown(_this3);
      });
      it('then removes the item', function () {
        expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).items).toEqual({});
      });
      it('then removes the deleted item from the selectionMap', function () {
        (0, _resourceAssertions.expectToChangeSelectionMapTo)(this, RESOURCE_NAME, {});
      });
      it('then removes the deleted item from the newItemKey', function () {
        (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, null);
      });
    });
    describe('when the API request errors', function () {
      beforeAll(function () {
        setUpAfterRequestFailure(this, initialState, params, id);
      });
      afterAll(function () {
        tearDown(this);
      });
      expectToRecordDestroyError(id, 404);
      it('then does NOT clear the item from thew newItemKey', function () {
        (0, _resourceAssertions.expectToNotChangeNewItemKey)(this, RESOURCE_NAME);
      });
      it('then updates the item\'s status httpCode', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'httpCode', 404);
      });
    });
  }

  function getInitialState(statusType) {
    return {
      items: {
        1: {
          values: {
            username: 'Bob'
          },
          status: {
            type: statusType
          }
        }
      },
      lists: {},
      selectionMap: {},
      newItemKey: null
    };
  }

  function setUpBeforeRequest(context, initialState, params, id) {
    _fetchMock["default"]["delete"]("http://test.com/users/".concat(id), new Promise(_nop["default"]));

    setupState(context, initialState, params);
  }

  function setUpAfterRequestSuccess(context, initialState, params, id) {
    var actionCreatorOptions = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};

    _fetchMock["default"]["delete"]("http://test.com/users/".concat(id), {
      body: {}
    });

    setupState(context, initialState, params, actionCreatorOptions);
  }

  function setUpAfterRequestFailure(context, initialState, params, id) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"]["delete"]("http://test.com/users/".concat(id), options);

    setupState(context, initialState, params);
  }

  function setupState(context, initialState, params) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.destroyUser(params));
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});