"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Edit reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users',
      keyBy: 'id'
    }, {
      editItem: true
    }),
        reducers = _resources.reducers,
        editUser = _resources.actionCreators.editItem;

    this.editUser = editUser;
    this.reducers = reducers;
    this.newValues = {
      username: 'Bob'
    };
  });
  describe('Given there is NO resource in the store', function () {
    describe('When the action creator is called with the item\'s id', function () {
      expectToWarnButCreateNewItem(1, 1);
    });
    describe('When the action creator is called with the item\'s id as an object', function () {
      expectToWarnButCreateNewItem(1, {
        id: 1
      });
    });
  });
  describe('Given a resource is in the store with a status of SUCCESS', function () {
    describe('When the action creator is called with the item\'s id', function () {
      expectToStartEditingWithNewValues(1, 1, {
        initialStatus: {
          type: _index.ERROR,
          error: {
            code: 'INVALID_RESOURCE'
          }
        }
      });
    });
    describe('When the action creator is called with the item\'s id as an object', function () {
      expectToStartEditingWithNewValues(1, {
        id: 1
      }, {
        initialStatus: {
          type: _index.ERROR,
          error: {
            code: 'INVALID_RESOURCE'
          }
        }
      });
    });
  });
  describe('Given a new resource item exists in the store and the editItem action controller is called to update it', function () {
    beforeAll(function () {
      spyOn(console, 'warn');
      this.id = 'temp';
      setupState(this, getInitialState(this.id, {
        type: _index.NEW
      }), this.id, this.newValues);
    });
    it('then warns that the editItem action controller is not intended to edit new resource items', function () {
      expect(console.warn).toHaveBeenCalledWith('Redux and the REST: EDIT_USER\'s key \'temp\' matches a NEW item. Use a editItem() to edit new items that have not yet been saved to an external API. Update ignored.');
    });
    it('then does not update the new resource item\'s values or state', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.id);
      (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.id);
    });
  });

  function getInitialState(id, status) {
    var base = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
      items: _defineProperty({}, id, {
        values: {
          username: 'Robert'
        },
        status: status
      })
    });

    if (status.type === _index.NEW) {
      base.newItemKey = id;
    }

    return base;
  }

  function setupState(context, initialState, params, newValues) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.editUser(params, newValues));
  }

  function expectToWarnButCreateNewItem(id, params) {
    beforeAll(function () {
      spyOn(console, 'warn');
      setupState(this, _objectSpread({}, _index.RESOURCES), params, this.newValues);
    });
    it('then warns about the missing item', function () {
      expect(console.warn).toHaveBeenCalledWith('Redux and the REST: EDIT_USER\'s key \'1\' does not match any items in the store. Use newItem() to create a new item or check the arguments passed to editItem(). (A new item was created to contain the edit.)');
    });
    it('then adds an item with the correct values', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, id, this.newValues);
    });
    it('then adds an item with a status of EDITING', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'type', _index.EDITING);
    });
  }

  function expectToStartEditingWithNewValues(id, params, options) {
    beforeAll(function () {
      setupState(this, getInitialState(id, options.initialStatus), params, this.newValues);
    });
    it('then updates the item\'s values', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, id, this.newValues);
    });
    it('then changes the item\'s status to EDITING', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, {
        type: _index.EDITING,
        dirty: true,
        originalValues: {
          username: 'Robert'
        }
      });
    });
  }
});