"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Edit new reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users',
      keyBy: 'id'
    }, {
      editNewItem: true
    }),
        reducers = _resources.reducers,
        editNewUser = _resources.actionCreators.editNewItem;

    this.editNewUser = editNewUser;
    this.reducers = reducers;
    this.newValues = {
      username: 'Bob'
    };
  });
  describe('Given there is no existing new resource item', function () {
    describe('and the action creator is called with no params arguments', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        setupState(this, _objectSpread({}, _index.RESOURCES), this.newValues);
      });
      it('then warns about the missing item', function () {
        expect(console.warn).toHaveBeenCalledWith('Redux and the REST: EDIT_NEW_USER\'s key \'null\' does not match any new items in the store. Use newItem() to create a new item first, or check the arguments passed to editItem(). Update ignored.');
      });
    });
    describe('and the action creator is called with a params arguments', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        setupState(this, _objectSpread({}, _index.RESOURCES), 'temp', this.newValues);
      });
      it('then warns about the missing item', function () {
        expect(console.warn).toHaveBeenCalledWith('Redux and the REST: EDIT_NEW_USER\'s key \'temp\' does not match any new items in the store. Use newItem() to create a new item first, or check the arguments passed to editItem(). Update ignored.');
      });
    });
  });
  describe('Given the editNew action creator is called with params', function () {
    describe('that match a resource item that is not new', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        this.id = 1;
        setupState(this, getInitialState(this.id, _index.SUCCESS), this.id, this.newValues);
      });
      it('then warns attempting to edit a resource item that is not NEW', function () {
        expect(console.warn).toHaveBeenCalledWith("Redux and the REST: EDIT_NEW_USER's key '".concat(this.id, "' matches a resource that is NOT new. Use a editItem() to edit existing items. Update ignored."));
      });
      it('then doesn\'t update the item', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.id);
        (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.id);
      });
    });
    describe('that match the new resource item', function () {
      beforeAll(function () {
        this.id = 'temp';
        setupState(this, getInitialState(this.id, _index.NEW), this.id, this.newValues);
      });
      it('then updates the item\'s values', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.id, this.newValues);
      });
      it('then leaves the item\'s status as NEW', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.id);
      });
    });
  });
  describe('Given the editNew action creator is called without a params argument,', function () {
    beforeAll(function () {
      this.id = 'temp';
      setupState(this, getInitialState(this.id, _index.NEW), this.newValues);
    });
    it('then updates the new item\'s values', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.id, this.newValues);
    });
    it('then leaves the item\'s status as NEW', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.id, 'type');
    });
  });

  function setupState(context, initialState, paramsOrNewValues) {
    var newValues = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);

    if (newValues) {
      context.store.dispatch(context.editNewUser(paramsOrNewValues, newValues));
    } else {
      context.store.dispatch(context.editNewUser(paramsOrNewValues));
    }
  }

  function getInitialState(id, statusType) {
    var base = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
      items: _defineProperty({}, id, {
        values: {
          username: 'Jane'
        },
        status: {
          type: statusType
        }
      })
    });

    if (statusType === _index.NEW) {
      base.newItemKey = id;
    }

    return base;
  }
});