"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Fetch reducers:', function () {
  describe('Given the keyBy option passed to resources is a string', function () {
    var _this = this;

    beforeAll(function () {
      var _resources = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id'
      }, {
        fetchItem: true
      }),
          reducers = _resources.reducers,
          fetchUser = _resources.actionCreators.fetchItem;

      this.fetchUser = fetchUser;
      this.reducers = reducers;
    });
    describe('Given no actions have come before it', function () {
      beforeAll(function () {
        this.url = 'http://test.com/users/1';
      });
      describe('When only the item\'s id is passed to the action creator', function () {
        expectToHandleSuccessAndFailure(_this.url, 1, 1);
      });
      describe('When the item\'s id is passed as an object to the action creator', function () {
        expectToHandleSuccessAndFailure(_this.url, 1, {
          id: 1
        });
      });
    });
    describe('and another fetchItem action has come before it', function () {
      var _this2 = this;

      beforeAll(function () {
        this.url = 'http://test.com/users/1';
        this.resourceBefore = {
          items: {
            1: {
              values: {
                id: 1,
                username: 'Bob'
              },
              status: {
                type: _index.SUCCESS
              }
            }
          }
        };
        this.itemId = 1;
      });
      describe('before the request has completed', function () {
        beforeAll(function () {
          setUpBeforeRequest(this, _objectSpread(_objectSpread({}, _index.RESOURCES), this.resourceBefore), this.url, 1);
        });
        afterAll(function () {
          tearDown(this);
        });
        it('then sets the status of the item to FETCHING', function () {
          (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.FETCHING);
        });
        it('then does NOT clear the item\'s values', function () {
          (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.itemId);
        });
        it('then does NOT add the item to the default list', function () {
          expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual({});
        });
      });
      describe('When the API request succeeds', function () {
        expectToMergeInResponseBody(_this2.url, 1, 1, _objectSpread(_objectSpread({}, _index.RESOURCES), _this2.resourceBefore));
      });
      describe('when the API request errors', function () {
        expectToMergeInError(_this2.url, 1, 1, _objectSpread(_objectSpread({}, _index.RESOURCES), _this2.resourceBefore));
      });
    });
    describe('Given a fetchItem action that will succeed with a response that specifies \'errors\' at the top level', function () {
      expectToMergeInMultipleErrors(1, {
        body: {
          errors: ['Not Found']
        },
        status: 200
      });
    });
    describe('Given a fetchItem action that will fail with a response that specifies \'errors\' at the top level', function () {
      expectToMergeInMultipleErrors(1, {
        body: {
          errors: ['Not Found']
        },
        status: 400
      });
    });
  });
  describe('when the keyBy option is an array', function () {
    beforeAll(function () {
      var _resources2 = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/groups/:groupId/users/:id?',
        keyBy: ['id', 'groupId']
      }, {
        fetchItem: true
      }),
          reducers = _resources2.reducers,
          fetchUser = _resources2.actionCreators.fetchItem;

      this.fetchUser = fetchUser;
      this.reducers = reducers;
    });
    expectToHandleSuccessAndFailure('http://test.com/groups/1/users/1', 'groupId=1.id=1', {
      id: 1,
      groupId: 1
    });
  });

  function expectToHandleSuccessAndFailure(url, id, params) {
    describe('before the request has completed', function () {
      expectToSetFetchingState(url, id, params);
    });
    describe('and the API request succeeds', function () {
      expectToMergeInResponseBody(url, id, params, _objectSpread({}, _index.RESOURCES));
    });
    describe('when the API request errors', function () {
      expectToMergeInError(url, id, params, _objectSpread({}, _index.RESOURCES));
    });
  }

  function expectToSetFetchingState(url, id, params) {
    beforeAll(function () {
      setUpBeforeRequest(this, _objectSpread({}, _index.RESOURCES), url, params);
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then adds a new item with a status of FETCHING', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'type', _index.FETCHING);
    });
    it('then adds a new item with empty values', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, id, {});
    });
    it('then does NOT add the item to the default list', function () {
      expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual({});
    });
  }

  function expectToMergeInResponseBody(url, id, params, initialState) {
    beforeAll(function () {
      this.newValues = {
        id: 1,
        username: 'Bob'
      };
      setUpAfterRequestSuccess(this, initialState, url, params, {
        body: this.newValues
      });
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then changes the items\'s status type to SUCCESS', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'type', _index.SUCCESS);
    });
    it('then sets the item\'s values from the response', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, id, this.newValues);
    });
  }

  function expectToMergeInError(url, id, params, initialState) {
    beforeAll(function () {
      this.options = {
        body: {
          error: 'Not Found'
        },
        status: 404
      };
      setUpAfterRequestFailure(this, initialState, url, params, this.options);
    });
    afterAll(function () {
      tearDown(this);
    });
    it('then changes the items\'s status type to ERROR', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'type', _index.ERROR);
    });
    it('then sets the syncedAt attribute', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, id);
    });
    it('then updates the item\'s status httpCode', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'httpCode', 404);
    });
    it('then sets the item\'s status error from the response', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, id, 'error', {
        message: this.options.body.error
      });
    });
    it('then does NOT set the item\'s values from the response', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, id, {});
    });
  }

  function expectToMergeInMultipleErrors(itemId, options) {
    describe('when the request has completed', function () {
      beforeAll(function () {
        setUpAfterRequestFailure(this, _objectSpread({}, _index.RESOURCES), 'http://test.com/users/1', itemId, options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the items\'s status type to ERROR', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, itemId, 'type', _index.ERROR);
      });
      it('then sets the syncedAt attribute', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, itemId);
      });
      it('then updates the item\'s status httpCode', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, itemId, 'httpCode', options.status);
      });
      it('then sets the item\'s status error from the response', function () {
        var errors = options.body.errors;

        var _errors = _slicedToArray(errors, 1),
            errorMessage = _errors[0];

        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, itemId, 'error', errorMessage);
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, itemId, 'errors', [errorMessage]);
      });
    });
  }

  function setUpBeforeRequest(context, initialState, url, params) {
    _fetchMock["default"].get(url, new Promise(_nop["default"]));

    setupState(context, initialState, params);
  }

  function setUpAfterRequestSuccess(context, initialState, url, params) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
      body: {}
    };

    _fetchMock["default"].get(url, options);

    setupState(context, initialState, params);
  }

  function setUpAfterRequestFailure(context, initialState, url, params) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"].get(url, options);

    setupState(context, initialState, params);
  }

  function setupState(context, initialState, params) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.fetchUser(params));
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});