"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('fetchList reducers:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id'
    }, {
      fetchList: true
    }),
        reducers = _resources.reducers,
        fetchUsers = _resources.actionCreators.fetchList;

    this.fetchUsers = fetchUsers;
    this.reducers = reducers;
  });
  describe('Given no actions have come before it', function () {
    describe('When the action creator is not passed any params', function () {
      expectToHandleSuccessAndFailure({
        url: 'http://test.com/users',
        listId: _EmptyKey["default"]
      });
    });
    describe('When the action creator is not passed params', function () {
      expectToHandleSuccessAndFailure({
        url: 'http://test.com/users/newest',
        params: 'newest',
        listId: 'newest'
      });
    });
  });
  describe('Given an INDEX action has come before it', function () {
    beforeAll(function () {
      this.itemId = 1;
      this.listId = _EmptyKey["default"];
      this.initialState = {
        items: _defineProperty({}, this.itemId, {
          values: {
            id: this.itemId,
            username: 'Bob'
          },
          status: {
            type: _index.SUCCESS
          }
        }),
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [this.itemId],
          status: {
            type: _index.SUCCESS,
            itemsInLastResponse: 1
          }
        })
      };
    });
    describe('When the action creator is called and before the request has completed', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, this.initialState, 'http://test.com/users');
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then sets the list\'s status to fetching', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'type', _index.FETCHING);
      });
      it('then does NOT clear the list\'s positions', function () {
        (0, _resourceAssertions.expectToNotChangeResourceListPositions)(this, RESOURCE_NAME, this.listId);
      });
      it('then does NOT clear the list\'s items', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.itemId);
        (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.itemId);
      });
    });
    describe('When the API request succeeds', function () {
      beforeAll(function () {
        this.newValues = [{
          id: 2,
          username: 'Jane'
        }, {
          id: 3,
          username: 'Jack'
        }];
        setUpAfterRequestSuccess(this, this.initialState, 'http://test.com/users', undefined, {
          body: this.newValues
        });
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the list\'s status type to SUCCESS', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'type', _index.SUCCESS);
      });
      it('then replaces the item keys in the list\'s positions with the new ones', function () {
        (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, this.listId, this.newValues.map(function (_ref) {
          var id = _ref.id;
          return id;
        }));
      });
      it('then sets the itemsInLastResponse attribute', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'itemsInLastResponse', this.newValues.length);
      });
      it('then adds items returned in the response not already in the store', function () {
        var _this = this;

        this.newValues.forEach(function (item) {
          (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(_this, RESOURCE_NAME, item.id, item);
        });
      });
      it('then sets the items status to SUCCESS', function () {
        var _this2 = this;

        this.newValues.forEach(function (item) {
          (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(_this2, RESOURCE_NAME, item.id, 'type', _index.SUCCESS);
        });
      });
    });
    describe('When the API request errors', function () {
      beforeAll(function () {
        this.options = {
          body: {
            error: 'Not Found'
          },
          status: 404
        };
        setUpAfterRequestFailure(this, this.initialState, 'http://test.com/users', undefined, this.options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the list\'s status to ERROR', function () {
        expect(this.store.getState().users.lists[_EmptyKey["default"]].status.type).toEqual(_index.ERROR);
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'type', _index.ERROR);
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'httpCode', this.options.status);
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'error', {
          message: this.options.body.error
        });
      });
      it('then sets the syncedAt attribute', function () {
        (0, _resourceAssertions.expectToChangeResourcesListStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, this.listId);
      });
      it('then sets the itemsInLastResponse attribute to undefined', function () {
        (0, _resourceAssertions.expectToClearResourcesListStatus)(this, RESOURCE_NAME, this.listId, 'itemsInLastResponse');
      });
      it('then does NOT change the list\'s positions', function () {
        (0, _resourceAssertions.expectToNotChangeResourceListPositions)(this, RESOURCE_NAME, this.listId);
      });
      it('then does NOT update any of the items', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.itemId);
      });
    });
  });
  describe('Given a SHOW action has come before it', function () {
    beforeAll(function () {
      this.itemId = 1;
      this.listId = _EmptyKey["default"];
      this.initialState = {
        items: _defineProperty({}, this.itemId, {
          values: {
            id: this.itemId,
            username: 'Bob'
          },
          status: {
            type: _index.SUCCESS
          }
        }),
        lists: {}
      };
    });
    describe('When the action creator is called and before the request has completed', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, this.initialState, 'http://test.com/users');
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then does NOT clear the list\'s items', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.itemId);
      });
    });
    describe('When the API request succeeds', function () {
      beforeAll(function () {
        this.newValues = [{
          id: 1,
          username: 'Robert'
        }, {
          id: 2,
          username: 'Jane'
        }];
        setUpAfterRequestSuccess(this, this.initialState, 'http://test.com/users', undefined, {
          body: this.newValues
        });
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then updates any items fetched using SHOW that are in the response', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.itemId, this.newValues[0]);
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.SUCCESS);
      });
      it('then adds any new items in the response that were not already in the store', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.newValues[1].id, this.newValues[1]);
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.newValues[1].id, 'type', _index.SUCCESS);
      });
    });
    describe('When the API request errors', function () {
      beforeAll(function () {
        this.options = {
          body: {
            error: 'Not Found'
          },
          status: 404
        };
        setUpAfterRequestFailure(this, this.initialState, 'http://test.com/users', undefined, this.options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then does NOT update any items', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.itemId);
        (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.itemId);
      });
    });
  });
  describe('Given an fetchList action that will succeed with a response that specifies \'error\' at the top level', function () {
    describe('when the request has completed', function () {
      beforeAll(function () {
        this.options = {
          body: {
            error: 'Not Found'
          },
          status: 200
        };
        this.listId = _EmptyKey["default"];
        setUpAfterRequestFailure(this, this.initialState, 'http://test.com/users', undefined, this.options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then sets the errors of the list', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'error', this.options.body.error);
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'errors', [this.options.body.error]);
      });
    });
  });
  describe('Given an fetchList action that will fail with a response that specifies \'errors\' at the top level', function () {
    describe('when the request has completed', function () {
      beforeAll(function () {
        this.options = {
          body: {
            errors: ['Not Found']
          },
          status: 200
        };
        this.listId = _EmptyKey["default"];
        setUpAfterRequestFailure(this, this.initialState, 'http://test.com/users', undefined, this.options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then sets the errors of the list', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'error', this.options.body.errors[0]);
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, this.listId, 'errors', this.options.body.errors);
      });
    });
  });

  function expectToHandleSuccessAndFailure(_ref2) {
    var url = _ref2.url,
        _ref2$initialState = _ref2.initialState,
        initialState = _ref2$initialState === void 0 ? _objectSpread({}, _index.RESOURCES) : _ref2$initialState,
        params = _ref2.params,
        listId = _ref2.listId;
    describe('and is not yet completed', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, initialState, url, params);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then adds a default list with a status type of fetching', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, listId, 'type', _index.FETCHING);
      });
      it('then adds a default list with an empty list of positions', function () {
        expect(this.store.getState().users.lists[listId].positions).toEqual([]);
        (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, listId, []);
      });
    });
    describe('and the API request succeeds', function () {
      beforeAll(function () {
        this.newValues = [{
          id: 1,
          username: 'Bob'
        }];
        setUpAfterRequestSuccess(this, _objectSpread({}, _index.RESOURCES), url, params, {
          body: this.newValues
        });
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the list\'s status type to SUCCESS', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, listId, 'type', _index.SUCCESS);
      });
      it('then sets the itemsInLastResponse attribute', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, listId, 'itemsInLastResponse', this.newValues.length);
      });
      it('then indexes the returned items according to the keyBy option and places their keys in positions', function () {
        (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, listId, this.newValues.map(function (_ref3) {
          var id = _ref3.id;
          return id;
        }));
      });
      it('then adds the returned items to the resource and keys them according to the keyBy option', function () {
        var _this3 = this;

        this.newValues.forEach(function (newValues) {
          (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(_this3, RESOURCE_NAME, newValues.id, newValues);
        });
      });
      it('then sets the items status to SUCCESS', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, 1, 'type', _index.SUCCESS);
      });
    });
    describe('and the API request errors', function () {
      beforeAll(function () {
        this.options = {
          body: {
            error: 'Not Found'
          },
          status: 404
        };
        setUpAfterRequestFailure(this, _objectSpread({}, _index.RESOURCES), url, params, this.options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the list\'s status', function () {
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, listId, 'type', _index.ERROR);
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, listId, 'error', {
          message: this.options.body.error
        });
        (0, _resourceAssertions.expectToChangeResourceListStatusTo)(this, RESOURCE_NAME, listId, 'httpCode', this.options.status);
      });
      it('then sets the syncedAt attribute', function () {
        (0, _resourceAssertions.expectToChangeResourcesListStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, listId);
      });
      it('then does not set the positions', function () {
        (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, listId, []);
      });
      it('then does not add any items', function () {
        expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).items).toEqual({});
      });
    });
  }

  function setUpBeforeRequest(context, initialState, url) {
    var params = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;

    _fetchMock["default"].get(url, new Promise(_nop["default"]));

    setupState(context, initialState, params);
  }

  function setUpAfterRequestSuccess(context, initialState, url) {
    var params = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
      body: {}
    };

    _fetchMock["default"].get(url, options);

    setupState(context, initialState, params);
  }

  function setUpAfterRequestFailure(context, initialState, url, params) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"].get(url, options);

    setupState(context, initialState, params);
  }

  function setupState(context, initialState) {
    var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);

    if (params) {
      context.store.dispatch(context.fetchUsers(params));
    } else {
      context.store.dispatch(context.fetchUsers());
    }
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});