"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('New reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users',
      keyBy: 'id'
    }, {
      newItem: true
    }),
        reducers = _resources.reducers,
        newUser = _resources.actionCreators.newItem;

    this.newUser = newUser;
    this.reducers = reducers;
  });
  describe('Given there is NOT already a new item', function () {
    describe('When only the item\'s id is passed to the action creator', function () {
      expectToAddNewItem('temp');
    });
    describe('When the item\'s id is passed as an object to the action creator', function () {
      expectToAddNewItem({
        id: 'temp'
      });
    });
  });
  describe('Given newItemKey already points to a resource with a status of NEW', function () {
    expectToChangeNewItem(_index.NEW);
  });
  describe('Given newItemKey already points to a resource with a status of EDITING', function () {
    expectToChangeNewItem(_index.EDITING);
  });
  describe('Given newItemKey already points to a resource with a status of CREATING', function () {
    expectToChangeNewItem(_index.CREATING);
  });
  describe('Given newItemKey already points to a resource with a status of ERROR', function () {
    expectToChangeNewItem(_index.ERROR);
  });
  describe('Given there is already an item with the same key with a status of SUCCESS', function () {
    expectToChangeToNewItem(_index.SUCCESS);
  });
  describe('Given there is already an item with the same key with a status of EDITING', function () {
    expectToChangeToNewItem(_index.EDITING);
  });
  describe('Given there is already an item with the same key with a status of UPDATING', function () {
    expectToChangeToNewItem(_index.UPDATING);
  });
  describe('Given there is already an item with the same key with a status of ERROR', function () {
    expectToChangeToNewItem(_index.ERROR);
  });
  describe('Given there is already an item with the same key with a status of DESTROYING', function () {
    expectToChangeToNewItem(_index.DESTROYING);
  });
  describe('Given there is already an item with the same key with a status of DESTROY_ERROR', function () {
    expectToChangeToNewItem(_index.DESTROY_ERROR);
  });
  describe('when the push lists operator is used', function () {
    expectToAddNewItemToListsPositions('push', 1, [1], [2, 1]);
  });
  describe('when the unshift lists operator is used', function () {
    expectToAddNewItemToListsPositions('unshift', 1, [1], [1, 2]);
  });
  describe('when the invalidate lists operator is used', function () {
    describe('and there are NO lists', function () {
      beforeAll(function () {
        this.listId = 'order=newest';
        setupState(this, _objectSpread({}, _index.RESOURCES), 1, this.newValues, {
          invalidate: {
            order: 'newest'
          }
        });
      });
      it('then does not create any lists', function () {
        expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual({});
      });
    });
    describe('and there are NO matching lists', function () {
      beforeAll(function () {
        this.listId = 'order=newest';
        this.otherListId = 'active=true';
        this.initialState = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
          lists: _defineProperty({}, this.otherListId, {
            positions: [],
            status: {
              type: _index.SUCCESS
            },
            metadata: {
              type: null
            }
          })
        });
        setupState(this, this.initialState, 1, this.newValues, {
          invalidate: {
            order: 'newest'
          }
        });
      });
      it('then does not create any lists', function () {
        expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual(this.initialState.lists);
      });
    });
    describe('and there are lists with keys that exactly match', function () {
      beforeAll(function () {
        var _lists2;

        this.listId = 'order=newest';
        this.otherListId = 'active=true';
        this.initialState = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
          lists: (_lists2 = {}, _defineProperty(_lists2, this.otherListId, {
            positions: [],
            status: {
              type: _index.SUCCESS
            },
            metadata: {
              type: null
            }
          }), _defineProperty(_lists2, this.listId, {
            positions: [2],
            status: {
              type: _index.SUCCESS
            }
          }), _lists2)
        });
        setupState(this, this.initialState, 1, this.newValues, {
          invalidate: {
            order: 'newest'
          }
        });
      });
      it('then clears the positions of the matching lists', function () {
        (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, this.listId, []);
      });
    });
  });

  function expectToAddNewItem(params) {
    beforeAll(function () {
      this.id = 'temp';
      this.newValues = {
        username: 'Jill'
      };
      setupState(this, _objectSpread({}, _index.RESOURCES), params, this.newValues);
    });
    it('then adds the new item', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.id, this.newValues);
    });
    it('then sets the status type of the new item to NEW', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.id, 'type', _index.NEW);
    });
    it('then sets the newItemKey', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, this.id);
    });
    it('then does NOT add the key of the new item to the default list', function () {
      expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual({});
    });
  }

  function expectToChangeNewItem(status) {
    beforeAll(function () {
      this.previousItemId = 'previous';
      this.id = 'temp';
      this.newValues = {
        username: 'Jill'
      };
      spyOn(console, 'warn');
      setupState(this, _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: _defineProperty({}, this.previousItemId, {
          values: {
            username: 'Bob'
          },
          status: {
            type: status
          }
        }),
        newItemKey: this.previousItemId,
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [this.previousItemId]
        })
      }), this.id, this.newValues);
    });
    it('then DOES NOT warn about the existing new record', function () {
      expect(console.warn).not.toHaveBeenCalled();
    });
    it('then DOES NOT change the values of the previous item', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.previousItemId);
      (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.previousItemId);
    });
    it('then adds the new item', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.id, this.newValues);
    });
    it('then sets the new item\'s status type to NEW', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.id, 'type', _index.NEW);
    });
    it('then does NOT add the key of the new item to the default list', function () {
      (0, _resourceAssertions.expectToNotChangeResourceListPositions)(this, RESOURCE_NAME, _EmptyKey["default"]);
    });
  }

  function expectToChangeToNewItem(status) {
    beforeAll(function () {
      this.previousItemId = 'previous';
      this.id = 'temp';
      this.newValues = {
        username: 'Jill'
      };

      var initialState = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: _defineProperty({}, this.id, {
          values: {
            username: 'Bob'
          },
          status: {
            type: status
          }
        }),
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [this.id]
        })
      });

      if (status === _index.NEW) {
        initialState.newItemKey = this.id;
      }

      spyOn(console, 'warn');
      setupState(this, initialState, this.id, this.newValues);
    });
    it('then warns about the existing record with the same key', function () {
      expect(console.warn).toHaveBeenCalledWith("Redux and the REST: 'NEW_USER' has same key '".concat(this.id, "' as existing item, use editItem() to update it instead, or clearNewItem() if you want to discard the previous values. (The previous item was overridden with new values.)"));
    });
    it('then changes the values of then existing item', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.id, this.newValues);
    });
    it('then sets the item\'s status type to NEW', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.id, 'type', _index.NEW);
    });
    it('then sets the newItemKey to point to the new item', function () {
      (0, _resourceAssertions.expectToChangeNewItemKeyTo)(this, RESOURCE_NAME, this.id);
    });
  }

  function expectToAddNewItemToListsPositions(operator, itemId, expectedIsolatedState, expectedCumulativeState) {
    describe('and there are NO lists', function () {
      beforeAll(function () {
        this.listId = 'order=newest';
        setupState(this, _objectSpread({}, _index.RESOURCES), itemId, this.newValues, _defineProperty({}, operator, {
          order: 'newest'
        }));
      });
      it('then creates a new list with the specified key and places the item in it', function () {
        expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual(this.initialState.lists);
      });
    });
    describe('and there are NO matching lists', function () {
      beforeAll(function () {
        this.listId = 'order=newest';
        this.otherListId = 'active=true';
        this.initialState = _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
          lists: _defineProperty({}, this.otherListId, {
            positions: [],
            status: {
              type: _index.SUCCESS
            },
            metadata: {
              type: null
            }
          })
        });
        setupState(this, this.initialState, itemId, this.newValues, _defineProperty({}, operator, {
          order: 'newest'
        }));
      });
      it('then does not create any new lists', function () {
        expect((0, _resourceAssertions.resourcesDefinition)(this, RESOURCE_NAME).lists).toEqual(this.initialState.lists);
      });
    });
    describe('and there are lists with keys that exactly match', function () {
      beforeAll(function () {
        var _lists6;

        this.listId = 'order=newest';
        this.otherListId = 'active=true';
        setupState(this, _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
          lists: (_lists6 = {}, _defineProperty(_lists6, this.otherListId, {
            positions: [],
            status: {
              type: _index.SUCCESS
            },
            metadata: {
              type: null
            }
          }), _defineProperty(_lists6, this.listId, {
            positions: [2],
            status: {
              type: _index.SUCCESS
            }
          }), _lists6)
        }), itemId, this.newValues, _defineProperty({}, operator, {
          order: 'newest'
        }));
      });
      it('then adds the new item to the matching lists', function () {
        (0, _resourceAssertions.expectToChangeResourceListPositionsTo)(this, RESOURCE_NAME, this.listId, expectedCumulativeState);
        (0, _resourceAssertions.expectToNotChangeResourceListPositions)(this, RESOURCE_NAME, this.otherListId);
      });
    });
  }

  function setupState(context, initialState, params, newValues, options) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);

    if (options) {
      context.store.dispatch(context.newUser(params, newValues, options));
    } else {
      context.store.dispatch(context.newUser(params, newValues));
    }
  }
});