"use strict";

var _index = require("../../../index");

var _resourceAssertions = require("../../helpers/resourceAssertions");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Select another reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users'
    }, {
      selectItem: true
    }),
        reducers = _resources.reducers,
        selectUser = _resources.actionCreators.selectItem;

    this.selectUser = selectUser;
    this.reducers = reducers;
    this.resourceBefore = {
      lists: {},
      newItemKey: null
    };
  });
  describe('when the resource is not in the store', function () {
    beforeAll(function () {
      spyOn(console, 'warn');
      setupState(this, {}, 3);
    });
    it('then warns about the resource not being in the store', function () {
      expect(console.warn).toHaveBeenCalledWith('Redux and the REST: selectMap is not intended to hold references to items that are not in the store. SELECT_USER\'s key \'3\' did not match any of the item keys: 1, 2. Check the options passed to selectItem(). (The selection was ignored.)');
    });
    it('then does not change selectionMap', function () {
      (0, _resourceAssertions.expectToNotChangeSelectionMap)(this, RESOURCE_NAME);
    });
  });
  describe('when the resource is in the store', function () {
    describe('and there are no resources already selected', function () {
      beforeAll(function () {
        this.nextSelected = 1;
        setupState(this, {}, this.nextSelected);
      });
      it('then add the item to the selectionMap', function () {
        (0, _resourceAssertions.expectToChangeSelectionMapTo)(this, RESOURCE_NAME, _defineProperty({}, this.nextSelected, true));
      });
    });
    describe('and there is already resources selected', function () {
      beforeAll(function () {
        this.nextSelected = 1;
        this.previouslySelected = 2;
        setupState(this, _defineProperty({}, this.previouslySelected, true), this.nextSelected);
      });
      it('then adds the newly selected item to those already selected', function () {
        (0, _resourceAssertions.expectToChangeSelectionMapTo)(this, RESOURCE_NAME, _defineProperty({}, this.nextSelected, true));
      });
    });
  });
  describe('when the item\'s id is passed as an object to the action creator', function () {
    beforeAll(function () {
      this.nextSelected = 1;
      setupState(this, {}, {
        id: this.nextSelected
      });
    });
    it('then add the item to the selectionMap', function () {
      (0, _resourceAssertions.expectToChangeSelectionMapTo)(this, RESOURCE_NAME, _defineProperty({}, this.nextSelected, true));
    });
  });

  function setupState(context, selectionMap, params) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
      items: {
        1: {
          values: {
            id: 1,
            username: 'Bob'
          },
          status: {
            type: _index.SUCCESS
          }
        },
        2: {
          values: {
            id: 2,
            username: 'Jill'
          },
          status: {
            type: _index.SUCCESS
          }
        }
      },
      selectionMap: selectionMap
    }));
    context.store.dispatch(context.selectUser(params));
  }
});