"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('Update reducer:', function () {
  beforeAll(function () {
    var _resources = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?'
    }, {
      updateItem: true
    }),
        reducers = _resources.reducers,
        updateUser = _resources.actionCreators.updateItem;

    this.updateUser = updateUser;
    this.reducers = reducers;
    this.itemId = 1;
    this.newValues = {
      id: 1,
      username: 'Robert'
    };
    this.serverValues = {
      id: 1,
      username: 'Robert',
      approved: false
    };
  });
  describe('Given the resource is NOT in the store', function () {
    describe('when the action creator is called and before the request has completed', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        setUpBeforeRequest(this, _objectSpread({}, _index.RESOURCES), 'http://test.com/users/1', this.itemId, this.newValues);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then warns about trying to update an item not in the store', function () {
        expect(console.warn).toHaveBeenCalledWith('Redux and the REST: UPDATE_USER\'s key \'1\' did not match any items in the store. Check the arguments passed to updateItem(). (Update request still sent to the server.)');
      });
      it('then adds a new item with a status of UPDATING', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.UPDATING);
      });
      it('then adds a new item with values specified', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.itemId, this.newValues);
      });
    });
    describe('when the API request succeeds', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        setUpAfterRequestSuccess(this, _objectSpread({}, _index.RESOURCES), 'http://test.com/users/1', this.itemId, this.newValues, {
          body: this.serverValues
        });
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the items\'s status type to SUCCESS', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.SUCCESS);
      });
      it('then sets the item\'s values from the response', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.itemId, this.serverValues);
      });
    });
  });
  describe('Given the resource is in the store', function () {
    var resourceBefore = {
      items: {
        1: {
          values: {
            username: 'Bob',
            id: 1
          },
          status: {
            type: _index.SUCCESS
          }
        }
      }
    };
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToCorrectlyUpdateItemInStore(_objectSpread(_objectSpread({}, _index.RESOURCES), resourceBefore), 1);
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToCorrectlyUpdateItemInStore(_objectSpread(_objectSpread({}, _index.RESOURCES), resourceBefore), {
        id: 1
      });
    });
  });
  describe('Given the resource item has been edited previously', function () {
    var resourceBefore = {
      items: {
        1: {
          values: {
            username: 'Bob',
            id: 1
          },
          status: {
            type: _index.EDITING,
            dirty: true,
            originalValues: {
              username: 'Bobert',
              id: 1
            }
          }
        }
      }
    };
    describe('and only the item\'s id is passed to the action creator', function () {
      expectToCorrectlyUpdateAnEditedResourceItem(_objectSpread(_objectSpread({}, _index.RESOURCES), resourceBefore), 1);
    });
    describe('and the item\'s id is passed as an object to the action creator', function () {
      expectToCorrectlyUpdateAnEditedResourceItem(_objectSpread(_objectSpread({}, _index.RESOURCES), resourceBefore), {
        id: 1
      });
    });
  });
  describe('Given a updateItem action that will succeed with a response that specifies \'errors\' at the top level', function () {
    expectToMergeInMultipleErrors(1, {
      body: {
        errors: ['Not Found']
      },
      status: 200
    });
  });
  describe('Given a updateItem action that will fail with a response that specifies \'errors\' at the top level', function () {
    expectToMergeInMultipleErrors(1, {
      body: {
        errors: ['Not Found']
      },
      status: 400
    });
  });

  function expectToCorrectlyUpdateItemInStore(initialState, params) {
    describe('when the action creator is called and before the request has completed', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, initialState, 'http://test.com/users/1', params, this.newValues);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then sets the item\'s status type to UPDATING', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.UPDATING);
      });
      it('then merges in the new values with the item\'s old ones', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.itemId, this.newValues);
      });
    });
    describe('when the API request succeeds', function () {
      beforeAll(function () {
        setUpAfterRequestSuccess(this, initialState, 'http://test.com/users/1', params, this.newValues, {
          body: this.serverValues
        });
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the item\'s status type to SUCCESS', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.SUCCESS);
      });
      it('then sets the item\'s values from the response', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.itemId, this.serverValues);
      });
    });
    describe('when the API request errors', function () {
      beforeAll(function () {
        this.options = {
          body: {
            error: 'Not Found'
          },
          status: 404
        };
        setUpAfterRequestFailure(this, initialState, 'http://test.com/users/1', params, this.options);
      });
      afterAll(function () {
        tearDown(this);
      });
      expectToHandleErrorResponse();
    });
  }

  function expectToCorrectlyUpdateAnEditedResourceItem(initialState, params) {
    describe('before the request has completed', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, initialState, 'http://test.com/users/1', params, this.newValues);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then sets the item\'s status type to UPDATING', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.UPDATING);
      });
      it('then does NOT unset the dirty bit', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.itemId, 'dirty');
      });
      it('then does NOT clear the originalValues', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.itemId, 'originalValues');
      });
      it('then merges in the new values with the item\'s old ones', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.itemId, this.newValues);
      });
    });
    describe('and the API request succeeds', function () {
      beforeAll(function () {
        setUpAfterRequestSuccess(this, initialState, 'http://test.com/users/1', params, this.newValues, {
          body: this.serverValues
        });
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the items\'s status type to SUCCESS', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.SUCCESS);
      });
      it('then removes the dirty bit', function () {
        (0, _resourceAssertions.expectToClearResourcesItemStatus)(this, RESOURCE_NAME, this.itemId, 'dirty');
      });
      it('then clears the original values', function () {
        (0, _resourceAssertions.expectToClearResourcesItemStatus)(this, RESOURCE_NAME, this.itemId, 'originalValues');
      });
      it('then sets the item\'s values from the response', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, this.itemId, this.serverValues);
      });
    });
    describe('and the API request errors', function () {
      beforeAll(function () {
        this.options = {
          body: {
            error: 'Not Found'
          },
          status: 404
        };
        setUpAfterRequestFailure(this, initialState, 'http://test.com/users/1', params, this.options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then does NOT unset the dirty bit', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.itemId, 'dirty');
      });
      it('then does NOT clear the originalValues', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemStatus)(this, RESOURCE_NAME, this.itemId, 'originalValues');
      });
      expectToHandleErrorResponse();
    });
  }

  function expectToHandleErrorResponse() {
    it('then changes the items\'s status type to ERROR', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.ERROR);
    });
    it('then sets the syncedAt attribute', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, this.itemId);
    });
    it('then updates the item\'s status httpCode', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'httpCode', this.options.status);
    });
    it('then does not update the values from the response', function () {
      (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, this.itemId);
    });
    it('then sets the status error from the response', function () {
      (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'error', {
        message: this.options.body.error
      });
    });
  }

  function expectToMergeInMultipleErrors(itemId, options) {
    describe('when the request has completed', function () {
      beforeAll(function () {
        setUpAfterRequestFailure(this, _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
          items: _defineProperty({}, itemId, {
            values: {
              username: 'Bob',
              id: itemId
            },
            status: {
              type: _index.SUCCESS
            }
          })
        }), 'http://test.com/users/1', itemId, options);
      });
      afterAll(function () {
        tearDown(this);
      });
      it('then changes the items\'s status type to ERROR', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'type', _index.ERROR);
      });
      it('then sets the syncedAt attribute', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusErrorOccurredAtToBeSet)(this, RESOURCE_NAME, this.itemId);
      });
      it('then updates the item\'s status httpCode', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, this.itemId, 'httpCode', options.status);
      });
      it('then sets the item\'s status error from the response', function () {
        var errors = options.body.errors;

        var _errors = _slicedToArray(errors, 1),
            errorMessage = _errors[0];

        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, itemId, 'error', errorMessage);
        (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, itemId, 'errors', [errorMessage]);
      });
    });
  }

  function setUpBeforeRequest(context, initialState, url, params, values) {
    _fetchMock["default"].put(url, new Promise(_nop["default"]));

    setupState(context, initialState, params, values);
  }

  function setUpAfterRequestSuccess(context, initialState, url, params, values) {
    var options = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {
      body: {}
    };

    _fetchMock["default"].put(url, options);

    setupState(context, initialState, params, values);
  }

  function setUpAfterRequestFailure(context, initialState, url, params) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"].put(url, options);

    setupState(context, initialState, params);
  }

  function setupState(context, initialState, params, values) {
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.updateUser(params, values));
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});