"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../index");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _resourceAssertions = require("../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('beforeReducers option:', function () {
  beforeAll(function () {
    var _resources2 = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id'
    }, {
      createItem: {
        beforeReducers: [function (_resources, _ref) {
          var status = _ref.status;
          var _items = _resources.items;
          var currentItem = _items.temp;

          if (status === _index.CREATING) {
            return _objectSpread(_objectSpread({}, _resources), {}, {
              items: _objectSpread(_objectSpread({}, _items), {}, {
                temp: _objectSpread(_objectSpread({}, currentItem), {}, {
                  beforeRequest: true
                })
              })
            });
          } else if (status === _index.SUCCESS) {
            return _objectSpread(_objectSpread({}, _resources), {}, {
              items: _objectSpread(_objectSpread({}, _items), {}, {
                temp: _objectSpread(_objectSpread({}, currentItem), {}, {
                  status: _objectSpread(_objectSpread({}, currentItem.status), {}, {
                    afterRequest: true
                  })
                })
              })
            });
          } else if (status === _index.ERROR) {
            return _objectSpread(_objectSpread({}, _resources), {}, {
              items: _objectSpread(_objectSpread({}, _items), {}, {
                temp: _objectSpread(_objectSpread({}, currentItem), {}, {
                  afterRequestError: true
                })
              })
            });
          }

          return _resources;
        }]
      }
    }),
        reducers = _resources2.reducers,
        createUser = _resources2.actionCreators.createItem;

    this.createUser = createUser;
    this.reducers = reducers;
    this.initialState = {
      items: {
        temp: {
          values: {
            username: 'Robert'
          },
          status: {
            type: _index.NEW
          }
        }
      },
      lists: {},
      newItemKey: 'temp'
    };
    this.newValues = {
      username: 'Bob'
    };
    this.responseValues = _objectSpread(_objectSpread({}, this.newValues), {}, {
      id: 1
    });
  });
  describe('Given a beforeReducers option is provided when defining a resource', function () {
    describe('and the API request succeeds', function () {
      describe('before the request has completed', function () {
        beforeAll(function () {
          setUpBeforeRequest(this, _objectSpread(_objectSpread({}, _index.RESOURCES), this.initialState), 'temp', this.newValues);
        });
        it('then calls the beforeReducers', function () {
          expect((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, 'temp').beforeRequest).toEqual(true);
        });
        afterAll(function () {
          tearDown(this);
        });
      });
      describe('when the request has completed', function () {
        beforeAll(function () {
          setUpAfterRequestSuccess(this, _objectSpread(_objectSpread({}, _index.RESOURCES), this.initialState), 'temp', this.newValues, this.responseValues);
        });
        it('then calls the beforeReducers', function () {
          expect((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, '1').status.afterRequest).toEqual(true);
        });
        afterAll(function () {
          tearDown(this);
        });
      });
    });
    describe('and the API request errors', function () {
      describe('before the request has completed', function () {
        beforeAll(function () {
          setUpBeforeRequest(this, _objectSpread(_objectSpread({}, _index.RESOURCES), this.initialState), 'temp', this.newValues);
        });
        it('then calls the beforeReducers', function () {
          expect((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, 'temp').beforeRequest).toEqual(true);
        });
        afterAll(function () {
          tearDown(this);
        });
      });
      describe('when the request has completed', function () {
        beforeAll(function () {
          setUpAfterRequestFailure(this, _objectSpread(_objectSpread({}, _index.RESOURCES), this.initialState), 'temp', this.newValues);
        });
        it('then calls the beforeReducers', function () {
          expect((0, _resourceAssertions.resourcesItem)(this, RESOURCE_NAME, 'temp').afterRequestError).toEqual(true);
        });
        afterAll(function () {
          tearDown(this);
        });
      });
    });
  });

  function setUpBeforeRequest(context, initialState, id, newValues) {
    var actionCreatorOptions = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};

    _fetchMock["default"].post('http://test.com/users', new Promise(_nop["default"]));

    setupState(context, initialState, id, newValues, actionCreatorOptions);
  }

  function setUpAfterRequestSuccess(context, initialState, id, initialValues) {
    var responseValues = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : initialValues;
    var actionCreatorOptions = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};

    _fetchMock["default"].post('http://test.com/users', {
      body: responseValues
    });

    setupState(context, initialState, id, initialValues, actionCreatorOptions);
  }

  function setUpAfterRequestFailure(context, initialState, id, newValues) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
      body: {
        error: 'Not Found'
      },
      status: 404
    };

    _fetchMock["default"].post('http://test.com/users', options);

    setupState(context, initialState, id, newValues);
  }

  function setupState(context, initialState, id, newValues) {
    var actionCreatorOptions = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
    (0, _resourceAssertions.setupInitialState)(context, RESOURCE_NAME, initialState);
    context.store.dispatch(context.createUser(id, newValues, actionCreatorOptions));
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }
});