"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _buildStore = _interopRequireDefault(require("../../helpers/buildStore"));

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('belongsTo:', function () {
  describe('Given a one-to-one association between items of two resources', function () {
    var _items;

    var userId = 1;
    var addressId = 1;
    var newUserId = 2;
    var baseUrl = 'http://test.com/addresses';
    var initialState = {
      users: {
        items: (_items = {}, _defineProperty(_items, userId, {
          values: {
            id: userId,
            username: 'Bob',
            addressId: addressId
          },
          status: {
            type: _index.SUCCESS
          }
        }), _defineProperty(_items, newUserId, {
          values: {
            id: 2,
            username: 'Rupert'
          },
          status: {
            type: _index.SUCCESS
          }
        }), _defineProperty(_items, "temp", {
          values: {
            username: 'Jane'
          },
          status: {
            type: _index.NEW
          }
        }), _items),
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }),
        selectionMap: {
          1: true
        },
        newItemKey: 'temp'
      },
      addresses: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: _defineProperty({}, addressId, {
          values: {
            city: 'City 1',
            userId: userId
          },
          status: {
            type: _index.SUCCESS
          }
        })
      })
    };
    var newAddressValues = {
      userId: userId,
      city: 'New City 3'
    };
    var updatedAddressValues = {
      id: addressId,
      userId: newUserId,
      city: 'New City 3'
    };
    expectToCorrectlyDefineAssociationIrrespectiveOfOrderResourcesAreDefined({
      setup: {
        baseUrl: baseUrl,
        initialState: initialState,
        userId: userId,
        addressId: addressId,
        newUserId: newUserId,
        newAddressValues: newAddressValues,
        updatedAddressValues: updatedAddressValues
      }
    });
  });

  function expectToCorrectlyDefineAssociationIrrespectiveOfOrderResourcesAreDefined(_ref) {
    var setup = _ref.setup;
    var baseUrl = setup.baseUrl,
        initialState = setup.initialState,
        userId = setup.userId,
        addressId = setup.addressId,
        newUserId = setup.newUserId,
        newAddressValues = setup.newAddressValues,
        updatedAddressValues = setup.updatedAddressValues;
    describe('and the associated resource is defined first', function () {
      beforeAll(function () {
        Reflect.apply(defineAddressesThenUsers, this, [baseUrl]);
      });
      expectToCorrectlySynchroniseItemWhenAssociatedOneChanges({
        setup: {
          initialState: initialState,
          userId: userId,
          addressId: addressId,
          newUserId: newUserId,
          newAddressValues: newAddressValues,
          updatedAddressValues: updatedAddressValues,
          baseUrl: baseUrl
        }
      });
    });
    describe('and the resource defining the association is defined first', function () {
      beforeAll(function () {
        Reflect.apply(defineUsersThenAddresses, this, [baseUrl]);
      });
      expectToCorrectlySynchroniseItemWhenAssociatedOneChanges({
        setup: {
          initialState: initialState,
          userId: userId,
          addressId: addressId,
          newUserId: newUserId,
          newAddressValues: newAddressValues,
          updatedAddressValues: updatedAddressValues,
          baseUrl: baseUrl
        }
      });
    });
  }

  describe('Given a one-to-many association between items of two resources', function () {
    var _items3;

    var userId = 1;
    var addressId = 1;
    var newUserId = 2;
    var baseUrl = 'http://test.com/addresses';
    var initialState = {
      users: {
        items: (_items3 = {}, _defineProperty(_items3, userId, {
          values: {
            id: 1,
            username: 'Bob',
            addressId: addressId
          },
          status: {
            type: _index.SUCCESS
          }
        }), _defineProperty(_items3, newUserId, {
          values: {
            id: 2,
            username: 'Rupert'
          },
          status: {
            type: _index.SUCCESS
          }
        }), _defineProperty(_items3, "temp", {
          values: {
            username: 'Jane'
          },
          status: {
            type: _index.NEW
          }
        }), _items3),
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }),
        selectionMap: {
          1: true
        },
        newItemKey: 'temp'
      },
      addresses: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: _defineProperty({}, addressId, {
          values: {
            city: 'City 1',
            userIds: [userId]
          },
          status: {
            type: _index.SUCCESS
          }
        })
      })
    };
    var newAddressValues = {
      userIds: [userId],
      city: 'New City 3'
    };
    var updatedAddressValues = {
      addressId: addressId,
      userIds: [newUserId],
      city: 'New City 3'
    };
    expectToCorrectlyDefineAssociationIrrespectiveOfOrderResourcesAreDefined({
      setup: {
        baseUrl: baseUrl,
        initialState: initialState,
        userId: userId,
        addressId: addressId,
        newUserId: newUserId,
        newAddressValues: newAddressValues,
        updatedAddressValues: updatedAddressValues
      }
    });
  });

  function expectToCorrectlySynchroniseItemWhenAssociatedOneChanges(_ref2) {
    var setup = _ref2.setup;
    var initialState = setup.initialState,
        userId = setup.userId,
        addressId = setup.addressId,
        newUserId = setup.newUserId,
        newAddressValues = setup.newAddressValues,
        updatedAddressValues = setup.updatedAddressValues,
        baseUrl = setup.baseUrl;
    describe('and the createItem action creator is called for the associated resource item', function () {
      var _this2 = this;

      var tempAddressId = 'temp';
      describe('when the request has yet to complete', function () {
        var _this = this;

        beforeAll(function () {
          setUpBeforeRequest(this, _objectSpread({}, initialState), {
            users: this.reducers,
            addresses: this.addresses.reducers
          }, this.addresses.actionCreators.createItem, tempAddressId, newAddressValues, {}, 'post', baseUrl);
        });
        it('then sets the new association to the default attribute', function () {
          (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, userId, 'addressId', tempAddressId);
        });
        afterAll(function () {
          return tearDown(_this);
        });
      });
      describe('when the request has completed', function () {
        var remoteAddressValues = _objectSpread(_objectSpread({}, newAddressValues), {}, {
          id: addressId
        });

        beforeAll(function () {
          setUpAfterRequestSuccess(this, _objectSpread({}, initialState), {
            users: this.reducers,
            addresses: this.addresses.reducers
          }, this.addresses.actionCreators.createItem, 'temp', newAddressValues, {}, 'post', baseUrl, {
            body: remoteAddressValues
          });
        });
        it('then updates the key of the association', function () {
          (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, userId, 'addressId', remoteAddressValues.id);
        });
        afterAll(function () {
          return tearDown(_this2);
        });
      });
    });
    describe('and the updateItem action creator for the associated resource item is called', function () {
      describe('with previousValues', function () {
        expectToSynchroniseItemsForeignKeyWhenAssociatedItemIsUpdated({
          setup: {
            initialState: initialState,
            addressId: addressId,
            updatedAddressValues: updatedAddressValues,
            baseUrl: baseUrl,
            userId: userId,
            newUserId: newUserId,
            actionCreatorOptions: {
              previousValues: initialState.addresses.items[addressId].values
            }
          },
          expectations: {
            showWarning: false
          }
        });
      });
      describe('WITHOUT previousValues', function () {
        expectToSynchroniseItemsForeignKeyWhenAssociatedItemIsUpdated({
          setup: {
            initialState: initialState,
            addressId: addressId,
            updatedAddressValues: updatedAddressValues,
            baseUrl: baseUrl,
            userId: userId,
            newUserId: newUserId,
            actionCreatorOptions: {}
          },
          expectations: {
            showWarning: true
          }
        });
      });
    });
    describe('and the destroyItem action creator for the associated resource item is called', function () {
      describe('with previousValues', function () {
        expectToSynchroniseItemsForeignKeyWhenAssociatedItemIsDeleted({
          setup: {
            initialState: initialState,
            addressId: addressId,
            baseUrl: baseUrl,
            userId: userId,
            actionCreatorOptions: {
              previousValues: initialState.addresses.items[addressId].values
            }
          },
          expectations: {
            showWarning: false
          }
        });
      });
      describe('WITHOUT previousValues', function () {
        expectToSynchroniseItemsForeignKeyWhenAssociatedItemIsDeleted({
          setup: {
            initialState: initialState,
            addressId: addressId,
            baseUrl: baseUrl,
            userId: userId,
            actionCreatorOptions: {}
          },
          expectations: {
            showWarning: true
          }
        });
      });
    });
  }

  function expectToSynchroniseItemsForeignKeyWhenAssociatedItemIsUpdated(_ref3) {
    var setup = _ref3.setup,
        expectations = _ref3.expectations;
    var initialState = setup.initialState,
        addressId = setup.addressId,
        updatedAddressValues = setup.updatedAddressValues,
        baseUrl = setup.baseUrl,
        userId = setup.userId,
        newUserId = setup.newUserId,
        actionCreatorOptions = setup.actionCreatorOptions;
    var showWarning = expectations.showWarning;
    describe('when the request has yet to complete', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, _objectSpread({}, initialState), {
          users: this.reducers,
          addresses: this.addresses.reducers
        }, this.addresses.actionCreators.updateItem, addressId, updatedAddressValues, actionCreatorOptions, 'put', "".concat(baseUrl, "/").concat(addressId));
      });
      it('then does NOT remove the associated item', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, userId, 'addressId');
        (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, newUserId, 'addressId');
      });
      afterAll(function () {
        tearDown(this);
      });
    });
    describe('when the request has completed', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        setUpAfterRequestSuccess(this, _objectSpread({}, initialState), {
          users: this.reducers,
          addresses: this.addresses.reducers
        }, this.addresses.actionCreators.updateItem, addressId, updatedAddressValues, actionCreatorOptions, 'put', "".concat(baseUrl, "/").concat(addressId), {
          body: updatedAddressValues
        });
      });

      if (showWarning) {
        it('then displays a warning', function () {
          expect(console.warn).toHaveBeenCalledWith('Redux and the REST: UPDATE_ADDRESS did not specify any previous values. This makes updating \'users.addressId\' much less efficient. Use the previousValues option for updateItem() to specify these values.');
        });
      } else {
        it('then does NOT display a warning', function () {
          expect(console.warn).not.toHaveBeenCalled();
        });
      }

      it('then removes the associated item from the previously associated items', function () {
        (0, _resourceAssertions.expectToClearResourcesItemValues)(this, RESOURCE_NAME, userId, 'addressId');
      });
      it('then adds the associated item to newly associated items', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, newUserId, 'addressId', addressId);
      });
      afterAll(function () {
        tearDown(this);
      });
    });
  }

  function expectToSynchroniseItemsForeignKeyWhenAssociatedItemIsDeleted(_ref4) {
    var setup = _ref4.setup,
        expectations = _ref4.expectations;
    var initialState = setup.initialState,
        addressId = setup.addressId,
        baseUrl = setup.baseUrl,
        userId = setup.userId,
        actionCreatorOptions = setup.actionCreatorOptions;
    var showWarning = expectations.showWarning;
    describe('when the request has yet to complete', function () {
      beforeAll(function () {
        setUpBeforeRequest(this, _objectSpread({}, initialState), {
          users: this.reducers,
          addresses: this.addresses.reducers
        }, this.addresses.actionCreators.destroyItem, addressId, actionCreatorOptions, null, 'delete', "".concat(baseUrl, "/").concat(addressId));
      });
      it('then does NOT remove the associated item', function () {
        (0, _resourceAssertions.expectToNotChangeResourcesItemValues)(this, RESOURCE_NAME, userId, 'addressId');
      });
      afterAll(function () {
        tearDown(this);
      });
    });
    describe('when the request has completed', function () {
      beforeAll(function () {
        spyOn(console, 'warn');
        setUpAfterRequestSuccess(this, _objectSpread({}, initialState), {
          users: this.reducers,
          addresses: this.addresses.reducers
        }, this.addresses.actionCreators.destroyItem, addressId, actionCreatorOptions, null, 'delete', "".concat(baseUrl, "/").concat(addressId), {
          body: {}
        });
      });

      if (showWarning) {
        it('then displays a warning', function () {
          expect(console.warn).toHaveBeenCalledWith('Redux and the REST: DESTROY_ADDRESS did not specify any previous values. This makes updating \'users.addressId\' much less efficient. Use the previousValues option for destroyItem() to specify these values.');
        });
      } else {
        it('then does NOT display a warning', function () {
          expect(console.warn).not.toHaveBeenCalled();
        });
      }

      it('then removes the associated item', function () {
        (0, _resourceAssertions.expectToClearResourcesItemValues)(this, RESOURCE_NAME, userId, 'addressId');
      });
      afterAll(function () {
        tearDown(this);
      });
    });
  }

  function defineUsersThenAddresses(baseUrl) {
    var _resources = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id',
      belongsTo: ['address']
    }, {
      fetchList: true,
      newItem: true
    }),
        reducers = _resources.reducers;

    this.addresses = (0, _index.resources)({
      name: 'addresses',
      url: "".concat(baseUrl, "/:id?"),
      keyBy: 'id'
    }, ['createItem', 'updateItem', 'destroyItem']);
    this.reducers = reducers;
  }

  function defineAddressesThenUsers(baseUrl) {
    this.addresses = (0, _index.resources)({
      name: 'addresses',
      url: "".concat(baseUrl, "/:id?"),
      keyBy: 'id'
    }, ['createItem', 'updateItem', 'destroyItem']);

    var _resources2 = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id',
      belongsTo: ['address']
    }, {
      fetchList: true,
      newItem: true
    }),
        reducers = _resources2.reducers;

    this.reducers = reducers;
  }

  function tearDown(context) {
    _fetchMock["default"].restore();

    context.store = null;
  }

  function setUpBeforeRequest(context, initialState, reducers, actionCreator, params, values, actionCreatorOptions, requestType, url) {
    _fetchMock["default"][requestType](url, new Promise(_nop["default"]));

    setupState(context, initialState, reducers, actionCreator, params, values, actionCreatorOptions);
  }

  function setUpAfterRequestSuccess(context, initialState, reducers, actionCreator, params, values, actionCreatorOptions, requestType, url, requestOptions) {
    _fetchMock["default"][requestType](url, requestOptions);

    setupState(context, initialState, reducers, actionCreator, params, values, actionCreatorOptions);
  }

  function setupState(context, initialState, reducers, actionCreator, params, values) {
    var actionCreatorOptions = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {};

    _setupInitialState(context, RESOURCE_NAME, initialState, reducers);

    context.store.dispatch(actionCreator(params, values, actionCreatorOptions));
  }

  function _setupInitialState(context, resourceName, initialState, reducers) {
    context.initialState = initialState;
    context.store = (0, _buildStore["default"])(initialState, reducers);
  }
});