"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _buildStore = _interopRequireDefault(require("../../helpers/buildStore"));

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('belongsTo:', function () {
  beforeAll(function () {
    var _lists;

    this.initialState = {
      users: {
        items: {
          1: {
            values: {
              id: 1,
              username: 'Bob',
              addressId: 1
            },
            status: {
              type: _index.SUCCESS
            }
          }
        },
        lists: (_lists = {}, _defineProperty(_lists, _EmptyKey["default"], {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }), _defineProperty(_lists, 'addressId=1', {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }), _defineProperty(_lists, 'address=1', {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }), _lists),
        selectionMap: {
          1: true
        },
        newItemKey: null
      },
      addresses: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: {
          1: {
            values: {
              userId: 1
            },
            status: {
              type: _index.SUCCESS
            }
          }
        }
      })
    };
  });
  [{
    description: 'when the \'listParameter\' option is NOT defined',
    listParameter: undefined,
    matchingListKey: 'addressId=1',
    nonMatchingListKey: 'address=1'
  }, {
    description: 'when the \'listParameter\' option is defined',
    listParameter: 'address',
    matchingListKey: 'address=1',
    nonMatchingListKey: 'addressId=1'
  }].forEach(function (_ref) {
    var listParameterDescription = _ref.description,
        listParameter = _ref.listParameter,
        matchingListKey = _ref.matchingListKey,
        nonMatchingListKey = _ref.nonMatchingListKey;
    describe(listParameterDescription, function () {
      [{
        description: 'and the \'dependent\' option is not defined',
        dependent: undefined,
        expectedNonMatchingList: {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }
      }, {
        description: 'and the \'dependent\' option is \'destroy\'',
        dependent: 'destroy',
        expectedNonMatchingList: {
          positions: [],
          status: {
            type: _index.SUCCESS
          }
        }
      }].forEach(function (_ref2) {
        var description = _ref2.description,
            dependent = _ref2.dependent,
            expectedNonMatchingList = _ref2.expectedNonMatchingList;
        describe(description, function () {
          beforeAll(function () {
            this.addresses = (0, _index.resources)({
              name: 'addresses',
              url: 'http://test.com/addresses/:id?',
              keyBy: 'id'
            }, {
              destroyItem: true
            });

            var _resources = (0, _index.resources)({
              name: 'users',
              url: 'http://test.com/users/:id?',
              keyBy: 'id',
              belongsTo: {
                addresses: {
                  dependent: dependent,
                  listParameter: listParameter
                }
              }
            }, {
              newItem: true
            }),
                reducers = _resources.reducers;

            this.reducers = reducers;
          });
          describe('and the previous values are included in the destroyItem action', function () {
            describe('before a request to destroy an associated resource item has completed', function () {
              beforeAll(function () {
                _fetchMock["default"]["delete"]('http://test.com/addresses/1', new Promise(_nop["default"]));

                this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
                  users: this.reducers,
                  addresses: this.addresses.reducers
                });
                this.store.dispatch(this.addresses.actionCreators.destroyItem(1, {
                  previousValues: {
                    userId: 1,
                    city: 'City 3'
                  }
                }));
              });
              afterAll(function () {
                _fetchMock["default"].restore();

                this.store = null;
              });
              it('then does NOT remove lists that contain the matching id parameter', function () {
                expect(this.store.getState().users.lists[matchingListKey]).toEqual(this.initialState.users.lists[matchingListKey]);
              });
            });
            describe('after a request to destroy an associated resource item has completed', function () {
              beforeAll(function () {
                _fetchMock["default"]["delete"]('http://test.com/addresses/1', {
                  body: {}
                });

                this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
                  users: this.reducers,
                  addresses: this.addresses.reducers
                });
                this.store.dispatch(this.addresses.actionCreators.destroyItem(1, {
                  previousValues: {
                    userId: 1,
                    city: 'City 3'
                  }
                }));
                spyOn(console, 'warn');
              });
              afterAll(function () {
                _fetchMock["default"].restore();

                this.store = null;
              });
              it('then removes any lists that contain the matching id parameter', function () {
                expect(this.store.getState().users.lists[matchingListKey]).toEqual(undefined);
              });
              it('then does NOT remove lists that don\'t match the id parameter', function () {
                expect(this.store.getState().users.lists[nonMatchingListKey]).toEqual(expectedNonMatchingList);
              });
              it('then does NOT display a warning', function () {
                expect(console.warn).not.toHaveBeenCalledWith('Redux and the REST: DESTROY_ADDRESS did not specify any previous values. This makes updating \'users.addressId\' much less efficient. Use the previousValues option for destroyItem() to specify these values.');
              });
            });
          });
          describe('and the previous values are NOT included in the destroyItem action', function () {
            describe('before a request to destroy an associated resource item has completed', function () {
              beforeAll(function () {
                _fetchMock["default"]["delete"]('http://test.com/addresses/1', new Promise(_nop["default"]));

                this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
                  users: this.reducers,
                  addresses: this.addresses.reducers
                });
                this.store.dispatch(this.addresses.actionCreators.destroyItem(1));
              });
              afterAll(function () {
                _fetchMock["default"].restore();

                this.store = null;
              });
              it('then does NOT remove lists that contain the matching id parameter', function () {
                expect(this.store.getState().users.lists[matchingListKey]).toEqual(this.initialState.users.lists[matchingListKey]);
              });
            });
            describe('after a request to destroy an associated resource item has completed', function () {
              beforeAll(function () {
                _fetchMock["default"]["delete"]('http://test.com/addresses/1', {
                  body: {}
                });

                this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
                  users: this.reducers,
                  addresses: this.addresses.reducers
                });
                spyOn(console, 'warn');
                this.store.dispatch(this.addresses.actionCreators.destroyItem(1));
              });
              afterAll(function () {
                _fetchMock["default"].restore();

                this.store = null;
              });
              it('then removes any lists that contain the matching id parameter', function () {
                expect(this.store.getState().users.lists[matchingListKey]).toEqual(undefined);
              });
              it('then does NOT remove lists that don\'t match the id parameter', function () {
                expect(this.store.getState().users.lists[nonMatchingListKey]).toEqual(expectedNonMatchingList);
              });
              it('then displays a warning', function () {
                expect(console.warn).toHaveBeenCalledWith('Redux and the REST: DESTROY_ADDRESS did not specify any previous values. This makes updating \'users.addressId\' much less efficient. Use the previousValues option for destroyItem() to specify these values.');
              });
            });
          });
        });
      });
    });
  });
});