"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../index");

var _buildStore = _interopRequireDefault(require("../helpers/buildStore"));

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('clearOn:', function () {
  beforeAll(function () {
    this.initialState = {
      users: {
        items: {
          1: {
            values: {
              id: 1,
              username: 'Bob'
            },
            status: {
              type: _index.SUCCESS
            }
          },
          temp: {
            values: {
              username: 'Jane'
            },
            status: {
              type: _index.NEW
            }
          }
        },
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }),
        selectionMap: {
          1: true
        },
        newItemKey: 'temp'
      },
      session: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: {
          1: {
            values: {
              active: true
            },
            status: {
              type: _index.SUCCESS
            }
          }
        }
      })
    };

    var _resources = (0, _index.resources)({
      name: 'session',
      url: 'http://test.com/session/:id'
    }, {
      destroyItem: true
    }),
        sessionReducers = _resources.reducers,
        destroySession = _resources.actionCreators.destroyItem,
        actions = _resources.actions;

    this.sessionReducers = sessionReducers;
    this.destroySession = destroySession;
    this.sessionActions = actions;

    _fetchMock["default"]["delete"]('http://test.com/session/1', {
      body: {
        status: 200,
        response: {}
      },
      status: 200
    });
  });
  afterAll(function () {
    _fetchMock["default"].restore();
  });
  describe('when it is not used', function () {
    beforeAll(function () {
      var _resources2 = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id'
      }, {
        fetchList: true,
        newItem: true
      }),
          usersReducers = _resources2.reducers;

      this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
        users: usersReducers,
        session: this.sessionReducers
      });
    });
    it('then does NOT clear this resource when another\'s actions occur', function () {
      var _this = this;

      return this.store.dispatch(this.destroySession(1)).then(function () {
        expect(_this.store.getState().users).toEqual(_this.initialState.users);
      });
    });
  });
  describe('when set to another resource\'s actions', function () {
    beforeAll(function () {
      var _resources3 = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id',
        clearOn: this.sessionActions.destroyItem
      }, {
        fetchList: true,
        newItem: true
      }),
          usersReducers = _resources3.reducers;

      this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
        users: usersReducers,
        session: this.sessionReducers
      });
    });
    it('then clears this resource when another\'s actions occur', function () {
      var _this2 = this;

      return this.store.dispatch(this.destroySession(1)).then(function () {
        expect(_this2.store.getState().users).toEqual(_index.RESOURCES);
      });
    });
  });
});