"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _buildStore = _interopRequireDefault(require("../../helpers/buildStore"));

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('hasAndBelongsToMany:', function () {
  describe('when the \'as\' option is used', function () {
    beforeAll(function () {
      this.initialState = {
        users: {
          items: {
            1: {
              values: {
                id: 1,
                username: 'Bob',
                postIds: [1]
              },
              status: {
                type: _index.SUCCESS
              }
            }
          },
          lists: _defineProperty({}, _EmptyKey["default"], {
            positions: [1],
            status: {
              type: _index.SUCCESS
            }
          }),
          selectionMap: {
            1: true
          },
          newItemKey: null
        },
        posts: _objectSpread({}, _index.RESOURCES)
      };
    });
    afterAll(function () {
      _fetchMock["default"].restore();
    });
    beforeAll(function () {
      this.posts = (0, _index.resources)({
        name: 'posts',
        url: 'http://test.com/posts/:id?',
        keyBy: 'id'
      }, {
        createItem: true
      });

      var _resources = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id',
        hasAndBelongsToMany: {
          posts: {
            as: 'author'
          }
        }
      }, {
        newItem: true
      }),
          reducers = _resources.reducers;

      this.reducers = reducers;
    });
    describe('before the request has completed', function () {
      beforeAll(function () {
        _fetchMock["default"].post('http://test.com/posts', new Promise(_nop["default"]));

        this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
          users: this.reducers,
          posts: this.posts.reducers
        });
        this.store.dispatch(this.posts.actionCreators.createItem('temp', {
          authorId: 1,
          title: 'New Post 3'
        }));
      });
      afterAll(function () {
        _fetchMock["default"].restore();

        this.store = null;
      });
      it('then uses the value of the \'as\' option to find the foreign key on the associated resource', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1, 'temp']);
      });
    });
    describe('and the request has completed', function () {
      beforeAll(function () {
        _fetchMock["default"].post('http://test.com/posts', {
          body: {
            id: 3,
            authorId: 1,
            title: 'New Post 3'
          }
        });

        this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
          users: this.reducers,
          posts: this.posts.reducers
        });
        this.store.dispatch(this.posts.actionCreators.createItem('temp', {
          authorId: 1,
          title: 'New Post 3'
        }));
      });
      afterAll(function () {
        _fetchMock["default"].restore();

        this.store = null;
      });
      it('then uses the value of the \'as\' option to find the foreign key on the associated resource', function () {
        (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1, 3]);
      });
    });
  });
});