"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _buildStore = _interopRequireDefault(require("../../helpers/buildStore"));

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('hasAndBelongsToMany:', function () {
  beforeAll(function () {
    this.initialState = {
      users: {
        items: {
          1: {
            values: {
              id: 1,
              username: 'Bob',
              addressIds: [1]
            },
            status: {
              type: _index.SUCCESS
            }
          }
        },
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }),
        selectionMap: {
          1: true
        },
        newItemKey: null
      },
      addresses: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: {
          1: {
            values: {
              userIds: [1]
            },
            status: {
              type: _index.SUCCESS
            }
          }
        }
      })
    };
  });
  describe('when the \'dependent\' option is \'destroy\'', function () {
    beforeAll(function () {
      this.addresses = (0, _index.resources)({
        name: 'addresses',
        url: 'http://test.com/addresses/:id?',
        keyBy: 'id'
      }, {
        destroyItem: true
      });

      var _resources = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id',
        hasAndBelongsToMany: {
          addresses: {
            dependent: 'destroy'
          }
        }
      }, {
        newItem: true
      }),
          reducers = _resources.reducers;

      this.reducers = reducers;
    });
    describe('and the previous values are included in the destroyItem action', function () {
      describe('before a request to destroy an associated resource item has completed', function () {
        beforeAll(function () {
          _fetchMock["default"]["delete"]('http://test.com/addresses/1', new Promise(_nop["default"]));

          this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
            users: this.reducers,
            addresses: this.addresses.reducers
          });
          this.store.dispatch(this.addresses.actionCreators.destroyItem(1, {
            previousValues: {
              userIds: [1],
              city: 'City 3'
            }
          }));
        });
        afterAll(function () {
          _fetchMock["default"].restore();

          this.store = null;
        });
        it('then does NOT remove the item', function () {
          expect(this.store.getState().users.items['1']).toEqual(this.initialState.users.items['1']);
        });
      });
      describe('after a request to destroy an associated resource item has completed', function () {
        beforeAll(function () {
          _fetchMock["default"]["delete"]('http://test.com/addresses/1', {
            body: {}
          });

          this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
            users: this.reducers,
            addresses: this.addresses.reducers
          });
          this.store.dispatch(this.addresses.actionCreators.destroyItem(1, {
            previousValues: {
              userIds: [1],
              city: 'City 3'
            }
          }));
        });
        afterAll(function () {
          _fetchMock["default"].restore();

          this.store = null;
        });
        it('then removes the item', function () {
          expect(this.store.getState().users.items['1']).toEqual(undefined);
        });
      });
    });
    describe('and the previous values are NOT included in the destroyItem action', function () {
      describe('before a request to destroy an associated resource item has completed', function () {
        beforeAll(function () {
          _fetchMock["default"]["delete"]('http://test.com/addresses/1', new Promise(_nop["default"]));

          this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
            users: this.reducers,
            addresses: this.addresses.reducers
          });
          this.store.dispatch(this.addresses.actionCreators.destroyItem(1));
        });
        afterAll(function () {
          _fetchMock["default"].restore();

          this.store = null;
        });
        it('then does NOT remove the item', function () {
          expect(this.store.getState().users.items['1']).toEqual(this.initialState.users.items['1']);
        });
      });
      describe('after a request to destroy an associated resource item has completed', function () {
        beforeAll(function () {
          _fetchMock["default"]["delete"]('http://test.com/addresses/1', {
            body: {}
          });

          this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
            users: this.reducers,
            addresses: this.addresses.reducers
          });
          spyOn(console, 'warn');
          this.store.dispatch(this.addresses.actionCreators.destroyItem(1));
        });
        afterAll(function () {
          _fetchMock["default"].restore();

          this.store = null;
        });
        it('then removes the item', function () {
          expect(this.store.getState().users.items['1']).toEqual(undefined);
        });
        it('then displays a warning', function () {
          expect(console.warn).toHaveBeenCalledWith('Redux and the REST: DESTROY_ADDRESS did not specify any previous values. This makes updating \'users.addressIds\' much less efficient. Use the previousValues option for destroyItem() to specify these values.');
        });
      });
    });
  });
});