"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../../index");

var _buildStore = _interopRequireDefault(require("../../helpers/buildStore"));

var _nop = _interopRequireDefault(require("../../../utils/function/nop"));

var _EmptyKey = _interopRequireDefault(require("../../../constants/EmptyKey"));

var _resourceAssertions = require("../../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('hasAndBelongsToMany:', function () {
  describe('when the association is many-to-one', function () {
    beforeAll(function () {
      this.initialState = {
        users: {
          items: {
            1: {
              values: {
                id: 1,
                username: 'Bob',
                postIds: [1]
              },
              status: {
                type: _index.SUCCESS
              }
            },
            2: {
              values: {
                id: 2,
                username: 'Rupert'
              },
              status: {
                type: _index.SUCCESS
              }
            },
            temp: {
              values: {
                username: 'Jane'
              },
              status: {
                type: _index.NEW
              }
            }
          },
          lists: _defineProperty({}, _EmptyKey["default"], {
            positions: [1],
            status: {
              type: _index.SUCCESS
            }
          }),
          selectionMap: {
            1: true
          },
          newItemKey: 'temp'
        },
        posts: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
          items: {
            1: {
              values: {
                title: 'Post 1',
                userId: 1
              },
              status: {
                type: _index.SUCCESS
              }
            }
          }
        })
      };
    });
    afterAll(function () {
      _fetchMock["default"].restore();
    });
    beforeAll(function () {
      this.posts = (0, _index.resources)({
        name: 'posts',
        url: 'http://test.com/posts/:id?',
        keyBy: 'id'
      }, {
        createItem: true,
        updateItem: true,
        destroyItem: true
      });

      var _resources = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id',
        hasAndBelongsToMany: ['posts']
      }, {
        fetchList: true,
        newItem: true
      }),
          reducers = _resources.reducers;

      this.reducers = reducers;
    });
    describe('and the association\'s CREATE action occurs', function () {
      describe('before the request has completed', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
            users: this.reducers,
            posts: this.posts.reducers
          });

          _fetchMock["default"].post('http://test.com/posts', new Promise(_nop["default"]));

          this.store.dispatch(this.posts.actionCreators.createItem('temp', {
            userId: 1,
            title: 'New Post 3'
          }));
        });
        afterAll(function () {
          _fetchMock["default"].restore();

          this.store = null;
        });
        it('then adds the new association to the default attribute', function () {
          (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1, 'temp']);
        });
      });
      describe('and the request has completed', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
            users: this.reducers,
            posts: this.posts.reducers
          });

          _fetchMock["default"].post('http://test.com/posts', {
            body: {
              id: 3,
              userId: 1,
              title: 'New Post 3'
            }
          });

          this.store.dispatch(this.posts.actionCreators.createItem('temp', {
            userId: 1,
            title: 'New Post 3'
          }));
        });
        afterAll(function () {
          _fetchMock["default"].restore();

          this.store = null;
        });
        it('then updates the key of the association', function () {
          (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1, 3]);
        });
      });
    });
    describe('and the association\'s UPDATE action occurs', function () {
      describe('and the previous values have been specified', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"].put('http://test.com/posts/1', new Promise(_nop["default"]));

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.updateItem(1, {
              title: 'Post 1',
              userId: 2
            }, {
              previousValues: {
                title: 'Post 1',
                userId: 1
              }
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT remove the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1]);
            (0, _resourceAssertions.expectToClearResourcesItemValues)(this, RESOURCE_NAME, '2', 'postIds');
          });
        });
        describe('and the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"].put('http://test.com/posts/1', {
              body: {
                title: 'Post 1',
                userId: 2
              }
            });

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.updateItem(1, {
              title: 'Post 1',
              userId: 2
            }, {
              previousValues: {
                title: 'Post 1',
                userId: 1
              }
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT display a warning', function () {
            expect(console.warn).not.toHaveBeenCalled();
          });
          it('then removes the associated item from old associated items', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', []);
          });
          it('then adds the associated item to new associated items', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '2', 'postIds', [1]);
          });
        });
      });
      describe('and the previous values have NOT been specified', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"].put('http://test.com/posts/1', new Promise(_nop["default"]));

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.updateItem(1, {
              title: 'Post 1',
              userId: 2
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT remove the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1]);
          });
        });
        describe('and the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"].put('http://test.com/posts/1', {
              body: {
                title: 'Post 1',
                userId: 2
              }
            });

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.updateItem(1, {
              title: 'Post 1',
              userId: 2
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then displays a warning', function () {
            expect(console.warn).toHaveBeenCalledWith('Redux and the REST: UPDATE_POST did not specify any previous values. This makes updating \'users.postIds\' much less efficient. Use the previousValues option for updateItem() to specify these values.');
          });
          it('then removes the associated item from old associated items', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', []);
          });
          it('then adds the associated item to new associated items', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '2', 'postIds', [1]);
          });
        });
      });
    });
    describe('and the association\'s DESTROY action occurs', function () {
      describe('and the previous values have been specified', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/posts/1', new Promise(_nop["default"]));

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.destroyItem(1, {
              previousValues: {
                title: 'Post 1',
                userId: 1
              }
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT remove the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1]);
          });
        });
        describe('and the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/posts/1', {
              body: {}
            });

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.destroyItem(1, {
              previousValues: {
                title: 'Post 1',
                userId: 1
              }
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT display a warning', function () {
            expect(console.warn).not.toHaveBeenCalled();
          });
          it('then removes the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', []);
          });
        });
      });
      describe('and the previous values have NOT been specified', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/posts/1', new Promise(_nop["default"]));

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.destroyItem(1));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT remove the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1]);
          });
        });
        describe('and the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/posts/1', {
              body: {}
            });

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.destroyItem(1));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then displays a warning', function () {
            expect(console.warn).toHaveBeenCalledWith('Redux and the REST: DESTROY_POST did not specify any previous values. This makes updating \'users.postIds\' much less efficient. Use the previousValues option for destroyItem() to specify these values.');
          });
          it('then removes the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', []);
          });
        });
      });
    });
  });
  describe('when the association is many-to-many', function () {
    beforeAll(function () {
      this.initialState = {
        users: {
          items: {
            1: {
              values: {
                id: 1,
                username: 'Bob',
                postIds: [1]
              },
              status: {
                type: _index.SUCCESS
              }
            },
            2: {
              values: {
                id: 2,
                username: 'Rupert'
              },
              status: {
                type: _index.SUCCESS
              }
            },
            temp: {
              values: {
                username: 'Jane'
              },
              status: {
                type: _index.NEW
              }
            }
          },
          lists: _defineProperty({}, _EmptyKey["default"], {
            positions: [1],
            status: {
              type: _index.SUCCESS
            }
          }),
          selectionMap: {
            1: true
          },
          newItemKey: 'temp'
        },
        posts: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
          items: {
            1: {
              values: {
                title: 'Post 1',
                userIds: [1]
              },
              status: {
                type: _index.SUCCESS
              }
            }
          }
        })
      };
    });
    afterAll(function () {
      _fetchMock["default"].restore();
    });
    beforeAll(function () {
      this.posts = (0, _index.resources)({
        name: 'posts',
        url: 'http://test.com/posts/:id?',
        keyBy: 'id'
      }, {
        createItem: true,
        updateItem: true,
        destroyItem: true
      });

      var _resources2 = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id',
        hasAndBelongsToMany: ['posts']
      }, {
        fetchList: true,
        newItem: true
      }),
          reducers = _resources2.reducers;

      this.reducers = reducers;
    });
    describe('and the association\'s CREATE action occurs', function () {
      describe('before the request has completed', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
            users: this.reducers,
            posts: this.posts.reducers
          });

          _fetchMock["default"].post('http://test.com/posts', new Promise(_nop["default"]));

          this.store.dispatch(this.posts.actionCreators.createItem('temp', {
            userIds: [1],
            title: 'New Post 3'
          }));
        });
        afterAll(function () {
          _fetchMock["default"].restore();

          this.store = null;
        });
        it('then adds the new association to the default attribute', function () {
          (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1, 'temp']);
        });
      });
      describe('and the request has completed', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
            users: this.reducers,
            posts: this.posts.reducers
          });

          _fetchMock["default"].post('http://test.com/posts', {
            body: {
              id: 3,
              userIds: [1],
              title: 'New Post 3'
            }
          });

          this.store.dispatch(this.posts.actionCreators.createItem('temp', {
            userIds: [1],
            title: 'New Post 3'
          }));
        });
        afterAll(function () {
          _fetchMock["default"].restore();

          this.store = null;
        });
        it('then updates the key of the association', function () {
          (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1, 3]);
        });
      });
    });
    describe('and the association\'s UPDATE action occurs', function () {
      describe('and the previous values have been specified', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"].put('http://test.com/posts/1', new Promise(_nop["default"]));

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.updateItem(1, {
              title: 'Post 1',
              userIds: [2]
            }, {
              previousValues: {
                title: 'Post 1',
                userIds: [1]
              }
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT remove the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1]);
            (0, _resourceAssertions.expectToClearResourcesItemValues)(this, RESOURCE_NAME, '2', 'postIds');
          });
        });
        describe('and the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"].put('http://test.com/posts/1', {
              body: {
                title: 'Post 1',
                userIds: [2]
              }
            });

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.updateItem(1, {
              title: 'Post 1',
              userIds: [2]
            }, {
              previousValues: {
                title: 'Post 1',
                userIds: [1]
              }
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT display a warning', function () {
            expect(console.warn).not.toHaveBeenCalled();
          });
          it('then removes the associated item from old associated items', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', []);
          });
          it('then adds the associated item to new associated items', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '2', 'postIds', [1]);
          });
        });
      });
      describe('and the previous values have NOT been specified', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"].put('http://test.com/posts/1', new Promise(_nop["default"]));

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.updateItem(1, {
              title: 'Post 1',
              userIds: [2]
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT remove the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1]);
          });
        });
        describe('and the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"].put('http://test.com/posts/1', {
              body: {
                title: 'Post 1',
                userIds: [2]
              }
            });

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.updateItem(1, {
              title: 'Post 1',
              userIds: [2]
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then displays a warning', function () {
            expect(console.warn).toHaveBeenCalledWith('Redux and the REST: UPDATE_POST did not specify any previous values. This makes updating \'users.postIds\' much less efficient. Use the previousValues option for updateItem() to specify these values.');
          });
          it('then removes the associated item from old associated items', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', []);
          });
          it('then adds the associated item to new associated items', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '2', 'postIds', [1]);
          });
        });
      });
    });
    describe('and the association\'s DESTROY action occurs', function () {
      describe('and the previous values have been specified', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/posts/1', new Promise(_nop["default"]));

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.destroyItem(1, {
              previousValues: {
                title: 'Post 1',
                userIds: [1]
              }
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT remove the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1]);
          });
        });
        describe('and the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/posts/1', {
              body: {}
            });

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.destroyItem(1, {
              previousValues: {
                title: 'Post 1',
                userIds: [1]
              }
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT display a warning', function () {
            expect(console.warn).not.toHaveBeenCalled();
          });
          it('then removes the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', []);
          });
        });
      });
      describe('and the previous values have NOT been specified', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/posts/1', new Promise(_nop["default"]));

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.destroyItem(1));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then does NOT remove the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', [1]);
          });
        });
        describe('and the request has completed', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
              users: this.reducers,
              posts: this.posts.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/posts/1', {
              body: {}
            });

            spyOn(console, 'warn');
            this.store.dispatch(this.posts.actionCreators.destroyItem(1));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then displays a warning', function () {
            expect(console.warn).toHaveBeenCalledWith('Redux and the REST: DESTROY_POST did not specify any previous values. This makes updating \'users.postIds\' much less efficient. Use the previousValues option for destroyItem() to specify these values.');
          });
          it('then removes the associated item', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'postIds', []);
          });
        });
      });
    });
  });
});