"use strict";

var _index = require("../../index");

var _buildStore = _interopRequireDefault(require("../helpers/buildStore"));

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('localOnly:', function () {
  beforeAll(function () {
    this.initialState = {
      users: {
        items: {
          1: {
            values: {
              id: 1,
              username: 'Bob'
            },
            status: {
              type: _index.SUCCESS
            }
          }
        },
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        }),
        selectionMap: {
          1: true
        },
        newItemKey: 'temp'
      }
    };
  });
  describe('Given the localOnly option is NOT set', function () {
    beforeAll(function () {
      var _resources = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id'
      }, ['fetchList', 'fetchItem', 'createItem', 'updateItem', 'destroyItem']),
          usersReducers = _resources.reducers,
          _resources$actionCrea = _resources.actionCreators,
          fetchUser = _resources$actionCrea.fetchItem,
          fetchUsers = _resources$actionCrea.fetchList;

      this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
        users: usersReducers
      });
      this.fetchUser = fetchUser;
      this.fetchUsers = fetchUsers;
    });
    it('then exports the fetchItem action creators', function () {
      expect(_typeof(this.fetchUser)).toEqual('function');
      expect(_typeof(this.fetchUsers)).toEqual('function');
    });
  });
  describe('Given the localOnly option is set to true', function () {
    beforeAll(function () {
      spyOn(console, 'warn');

      var _resources2 = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id',
        localOnly: true
      }, ['fetchList', 'fetchItem', 'newItem', 'createItem', 'editItem', 'updateItem', 'destroyItem']),
          usersReducers = _resources2.reducers,
          _resources2$actionCre = _resources2.actionCreators,
          fetchUser = _resources2$actionCre.fetchItem,
          fetchUsers = _resources2$actionCre.fetchList,
          newUser = _resources2$actionCre.newItem,
          createUser = _resources2$actionCre.createItem,
          editUser = _resources2$actionCre.editItem,
          updateUser = _resources2$actionCre.updateItem,
          destroyUser = _resources2$actionCre.destroyItem;

      this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
        users: usersReducers
      });
      this.fetchUser = fetchUser;
      this.fetchUsers = fetchUsers;
      this.createUser = createUser;
      this.newUser = newUser;
      this.editUser = editUser;
      this.updateUser = updateUser;
      this.destroyUser = destroyUser;
    });
    describe('and the resource\'s CREATE action occurs WITHOUT a preceding NEW action', function () {
      beforeEach(function () {
        this.newValues = {
          username: 'Joseph'
        };
        this.store.dispatch(this.createUser('temp', this.newValues));
        this.users = this.store.getState().users;
      });
      it('then immediately adds the new resource', function () {
        expect(this.users.items.temp.values).toEqual(this.newValues);
        expect(this.users.items.temp.status.type).toEqual(_index.SUCCESS);
      });
    });
    describe('and the resource\'s CREATE action occurs after a NEW action', function () {
      beforeEach(function () {
        this.newValues = {
          username: 'Joseph'
        };
        this.store.dispatch(this.newUser('temp', this.newValues));
        this.store.dispatch(this.createUser('temp', this.newValues));
        this.users = this.store.getState().users;
      });
      it('then immediately adds the new resource', function () {
        expect(this.users.items.temp.values).toEqual(this.newValues);
        expect(this.users.items.temp.status.type).toEqual(_index.SUCCESS);
      });
    });
    describe('and the resource\'s UPDATE action occurs WITHOUT a preceding EDIT action', function () {
      beforeEach(function () {
        this.newValues = {
          username: 'Joseph'
        };
        this.store.dispatch(this.updateUser('1', this.newValues));
        this.users = this.store.getState().users;
      });
      it('then immediately updates the resource', function () {
        expect(this.users.items['1'].values).toEqual(_objectSpread(_objectSpread({}, this.initialState.users.items['1'].values), this.newValues));
        expect(this.users.items['1'].status.type).toEqual(_index.SUCCESS);
      });
    });
    describe('and the resource\'s UPDATE action occurs after an EDIT action', function () {
      beforeEach(function () {
        this.newValues = {
          username: 'Joseph'
        };
        this.store.dispatch(this.editUser('1', this.newValues));
        this.store.dispatch(this.updateUser('1', this.newValues));
        this.users = this.store.getState().users;
      });
      it('then immediately updates the resource', function () {
        expect(this.users.items['1'].values).toEqual(_objectSpread(_objectSpread({}, this.initialState.users.items['1'].values), this.newValues));
        expect(this.users.items['1'].status.type).toEqual(_index.SUCCESS);
      });
    });
    describe('and the resource\'s DESTROY action occurs', function () {
      beforeEach(function () {
        this.store.dispatch(this.destroyUser('1'));
        this.users = this.store.getState().users;
      });
      it('then immediately updates the resource', function () {
        expect(this.users.items['1']).not.toBeDefined();
      });
    });
  });
});