"use strict";

var _index = require("../../index");

var _buildStore = _interopRequireDefault(require("../helpers/buildStore"));

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index2 = require("../../../index");

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('metadata:', function () {
  describe('when configuring the INDEX action', function () {
    describe('and the metadata value is NOT set when defining the resource', function () {
      beforeAll(function () {
        var _resources = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users',
          keyBy: 'id'
        }, {
          fetchList: true
        }),
            reducers = _resources.reducers,
            fetchUsers = _resources.actionCreators.fetchList;

        this.reducers = reducers;
        this.fetchUsers = fetchUsers;
      });
      describe('and the metadata type is set when calling the action creator', function () {
        describe('and the request succeeds', function () {
          describe('before the request has completed', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', new Promise(_nop["default"]));

              this.store.dispatch(this.fetchUsers({}, {
                metadata: {
                  type: _index2.PREVIEW
                }
              }));
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then uses the value passed to the action creator for the item\'s metadata type', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual(_index2.PREVIEW);
            });
          });
          describe('when the request completes', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', {
                body: [{
                  id: 1,
                  username: 'Robert'
                }]
              });

              this.store.dispatch(this.fetchUsers({}, {
                metadata: {
                  type: _index2.PREVIEW
                },
                itemsMetadata: {
                  type: _index2.PREVIEW
                }
              }));
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then uses the value passed to the action creator for the item\'s metadata type', function () {
              expect(this.store.getState().users.items['1'].metadata.type).toEqual(_index2.PREVIEW);
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual(_index2.PREVIEW);
            });
          });
        });
        describe('and the request errors', function () {
          describe('before the request has completed', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', new Promise(_nop["default"]));

              this.store.dispatch(this.fetchUsers({}, {
                metadata: {
                  type: _index2.PREVIEW
                }
              }));
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then the list\'s metadata type is the value passed to the action creator', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual(_index2.PREVIEW);
            });
          });
          describe('when the request completes', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', {
                body: {
                  error: 'Not Found'
                },
                status: 404
              });

              this.store.dispatch(this.fetchUsers({}, {
                metadata: {
                  type: _index2.PREVIEW
                }
              }));
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then the list\'s metadata type is the value passed to the action creator', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual(_index2.PREVIEW);
            });
          });
        });
      });
    });
    describe('and the metadata type is set when defining the resource', function () {
      beforeAll(function () {
        var _resources2 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users',
          keyBy: 'id',
          metadata: {
            type: 'RESOURCE_METADATA'
          }
        }, {
          fetchList: true
        }),
            reducers = _resources2.reducers,
            fetchUsers = _resources2.actionCreators.fetchList;

        this.fetchUsers = fetchUsers;
        this.reducers = reducers;
      });
      describe('and the metadata type is NOT set when calling the action creator', function () {
        describe('and the request succeeds', function () {
          describe('before the request has completed', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', new Promise(_nop["default"]));

              this.store.dispatch(this.fetchUsers());
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then the item and list\'s metadata type is the value specified when defining the resource', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual('RESOURCE_METADATA');
            });
          });
          describe('when the request completes', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', {
                body: [{
                  id: 1,
                  username: 'Robert'
                }]
              });

              this.store.dispatch(this.fetchUsers());
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then the item and list\'s metadata type is the value specified when defining the resource', function () {
              expect(this.store.getState().users.items['1'].metadata.type).toEqual('RESOURCE_METADATA');
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual('RESOURCE_METADATA');
            });
          });
        });
        describe('and the request errors', function () {
          describe('before the request has completed', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', new Promise(_nop["default"]));

              this.store.dispatch(this.fetchUsers());
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then the list\'s metadata type is the value specified when defining the resource', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual('RESOURCE_METADATA');
            });
          });
          describe('when the request completes', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', {
                body: {
                  error: 'Not Found'
                },
                status: 404
              });

              this.store.dispatch(this.fetchUsers());
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then the list\'s metadata type is the value specified when defining the resource', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual('RESOURCE_METADATA');
            });
          });
        });
      });
      describe('and the metadata type is set when calling the action creator', function () {
        describe('and the request succeeds', function () {
          describe('before the request has completed', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', new Promise(_nop["default"]));

              this.store.dispatch(this.fetchUsers({}, {
                metadata: {
                  type: 'ACTION_CREATOR_METADATA'
                }
              }));
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then uses the value passed to the action creator for the list\'s metadata type', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual('ACTION_CREATOR_METADATA');
            });
          });
          describe('when the request completes', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', {
                body: [{
                  id: 1,
                  username: 'Robert'
                }]
              });

              this.store.dispatch(this.fetchUsers({}, {
                metadata: {
                  type: 'ACTION_CREATOR_METADATA'
                },
                itemsMetadata: {
                  type: 'ACTION_CREATOR_ITEM_METADATA'
                }
              }));
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then uses the value passed to the action creator for the item and list\'s metadata type', function () {
              expect(this.store.getState().users.items['1'].metadata.type).toEqual('ACTION_CREATOR_ITEM_METADATA');
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual('ACTION_CREATOR_METADATA');
            });
          });
        });
        describe('and the request errors', function () {
          describe('before the request has completed', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', new Promise(_nop["default"]));

              this.store.dispatch(this.fetchUsers({}, {
                metadata: {
                  type: 'ACTION_CREATOR_METADATA'
                }
              }));
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then uses the value passed to the action creator for the list\'s metadata type', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual('ACTION_CREATOR_METADATA');
            });
          });
          describe('when the request completes', function () {
            beforeAll(function () {
              this.store = (0, _buildStore["default"])({
                users: _objectSpread({}, _index.RESOURCES)
              }, {
                users: this.reducers
              });

              _fetchMock["default"].get('http://test.com/users', {
                body: {
                  error: 'Not Found'
                },
                status: 404
              });

              this.store.dispatch(this.fetchUsers({}, {
                metadata: {
                  type: 'ACTION_CREATOR_METADATA'
                }
              }));
            });
            afterAll(function () {
              _fetchMock["default"].restore();

              this.store = null;
            });
            it('then uses the value passed to the action creator for the list\'s metadata type', function () {
              expect(this.store.getState().users.lists[_EmptyKey["default"]].metadata.type).toEqual('ACTION_CREATOR_METADATA');
            });
          });
        });
      });
    });
  });
  describe('when configuring the SHOW action', function () {
    describe('and the metadata type is NOT set when defining the resource', function () {
      beforeAll(function () {
        var _resources3 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users/:id',
          keyBy: 'id'
        }, {
          fetchItem: true
        }),
            reducers = _resources3.reducers,
            fetchUser = _resources3.actionCreators.fetchItem;

        this.reducers = reducers;
        this.fetchUser = fetchUser;
      });
      describe('and the metadata type is set when calling the action creator', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users/1', new Promise(_nop["default"]));

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUser(1, {
              metadata: {
                type: _index2.PREVIEW
              }
            }));
            this.user = this.store.getState().users.items['1'];
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then uses the value passed to the action creator for the item\'s metadata type', function () {
            expect(this.user.metadata.type).toEqual(_index2.PREVIEW);
          });
        });
        describe('when the request completes', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users/1', {
              body: {
                id: 1,
                username: 'Robert'
              }
            });

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUser(1, {
              metadata: {
                type: _index2.PREVIEW
              }
            }));
            this.user = this.store.getState().users.items['1'];
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then uses the value passed to the action creator for the item\'s metadata type', function () {
            expect(this.user.metadata.type).toEqual(_index2.PREVIEW);
          });
        });
      });
    });
    describe('and the metadata type is set when defining the resource', function () {
      beforeAll(function () {
        var _resources4 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users/:id',
          keyBy: 'id',
          metadata: {
            type: 'RESOURCE_METADATA'
          }
        }, {
          fetchItem: true
        }),
            reducers = _resources4.reducers,
            fetchUser = _resources4.actionCreators.fetchItem;

        this.reducers = reducers;
        this.fetchUser = fetchUser;
      });
      describe('and the metadata type is NOT set when calling the action creator', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users/1', new Promise(_nop["default"]));

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUser(1));
            this.user = this.store.getState().users.items['1'];
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the item\'s metadata type is the value specified when defining the resource', function () {
            expect(this.user.metadata.type).toEqual('RESOURCE_METADATA');
          });
        });
        describe('when the request completes', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users/1', {
              body: {
                id: 1,
                username: 'Robert'
              }
            });

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUser(1));
            this.user = this.store.getState().users.items['1'];
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the item\'s metadata type is the value specified when defining the resource', function () {
            expect(this.user.metadata.type).toEqual('RESOURCE_METADATA');
          });
        });
      });
      describe('and the metadata type is set when calling the action creator', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users/1', new Promise(_nop["default"]));

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUser(1, {
              metadata: {
                type: 'ACTION_CREATOR_METADATA'
              }
            }));
            this.user = this.store.getState().users.items['1'];
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then uses the value passed to the action creator for the item\'s metadata type', function () {
            expect(this.user.metadata.type).toEqual('ACTION_CREATOR_METADATA');
          });
        });
        describe('when the request completes', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users/1', {
              body: {
                id: 1,
                username: 'Robert'
              }
            });

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUser(1, {
              metadata: {
                type: 'ACTION_CREATOR_METADATA'
              }
            }));
            this.user = this.store.getState().users.items['1'];
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then uses the value passed to the action creator for the item\'s metadata type', function () {
            expect(this.user.metadata.type).toEqual('ACTION_CREATOR_METADATA');
          });
        });
      });
    });
  });
});