"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _XHRMock = _interopRequireDefault(require("../helpers/XHRMock"));

var _index = require("../../index");

var _buildStore = _interopRequireDefault(require("../helpers/buildStore"));

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

var _resourceAssertions = require("../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('progress option', function () {
  describe('for the createItem action:', function () {
    describe('when it is NOT set to a truthy value', function () {
      beforeAll(function () {
        var _resources = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users',
          keyBy: 'id'
        }, {
          createItem: true
        }),
            reducers = _resources.reducers,
            createUser = _resources.actionCreators.createItem;

        this.reducers = reducers;
        this.createUser = createUser;
      });
      describe('and the API request succeeds', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            _fetchMock["default"].post('http://test.com/users', new Promise(_nop["default"]));

            this.store = (0, _buildStore["default"])({
              users: _index.RESOURCES
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.createUser('temp', {
              username: 'Bob'
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the item\'s status is CREATING', function () {
            expect(this.store.getState().users.items.temp.status.type).toEqual(_index.CREATING);
          });
          it('then there are no progress attributes on the item\'s status object', function () {
            var userStatus = this.store.getState().users.items.temp.status;
            expect(userStatus.progressUp).toEqual(undefined);
            expect(userStatus.progressDown).toEqual(undefined);
          });
        });
        describe('when the request completes', function () {
          beforeAll(function () {
            _fetchMock["default"].post('http://test.com/users', {
              body: {
                id: 1,
                username: 'Bob'
              }
            });

            this.store = (0, _buildStore["default"])({
              users: _index.RESOURCES
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.createUser('temp', {
              username: 'Bob'
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the item\'s status is SUCCESS', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, '1', 'type', _index.SUCCESS);
          });
          it('then does not set the item\'s progress status attributes', function () {
            var userStatus = this.store.getState().users.items['1'].status;
            expect(userStatus.progressUp).toEqual(undefined);
            expect(userStatus.progressDown).toEqual(undefined);
          });
        });
      });
    });
    describe('Given it is set to a truthy value', function () {
      beforeAll(function () {
        var _resources2 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users',
          keyBy: 'id'
        }, {
          createItem: {
            progress: true
          }
        }),
            reducers = _resources2.reducers,
            createUser = _resources2.actionCreators.createItem;

        this.reducers = reducers;
        this.createUser = createUser;

        _XHRMock["default"].setup();
      });
      afterAll(function () {
        _XHRMock["default"].tearDown();
      });
      describe('and the API request succeeds,', function () {
        beforeAll(function () {
          this.xhrMock = _XHRMock["default"].post('http://test.com/users', {
            body: {
              id: 1,
              username: 'Bob'
            }
          });
          this.store = (0, _buildStore["default"])({
            users: _index.RESOURCES
          }, {
            users: this.reducers
          });
          this.store.dispatch(this.createUser('temp', {
            username: 'Bob'
          }));
          this.userStatus = this.store.getState().users.items.temp.status;
        });
        afterAll(function () {
          _XHRMock["default"].reset();

          this.store = null;
        });
        describe('before the request has completed', function () {
          it('then the item\'s status is CREATING', function () {
            expect(this.userStatus.type).toEqual(_index.CREATING);
          });
          it('then there are no progress attributes on the item\'s status object', function () {
            expect(this.userStatus.progressUp).toEqual(undefined);
            expect(this.userStatus.progressDown).toEqual(undefined);
          });
        });
        describe('and the request progresses', function () {
          beforeAll(function () {
            this.xhrMock.setUploadProgress({
              lengthComputable: true,
              loaded: 12,
              total: 24
            });
            this.userStatus = this.store.getState().users.items.temp.status;
          });
          it('then the item\'s status is still CREATING', function () {
            expect(this.userStatus.type).toEqual(_index.CREATING);
          });
          it('then the item\'s progressUp status is updated with the current values', function () {
            expect(this.userStatus.progressUp).toEqual({
              percent: 50,
              loaded: 12,
              total: 24,
              lengthComputable: true
            });
            expect(this.userStatus.progressDown).toEqual({
              percent: 0,
              loaded: 0,
              total: undefined,
              lengthComputable: undefined
            });
          });
        });
        describe('and the upload has finished', function () {
          beforeAll(function () {
            this.xhrMock.completeUpload({
              status: 200
            });
            this.userStatus = this.store.getState().users.items.temp.status;
          });
          it('then the item\'s status is still CREATING', function () {
            expect(this.userStatus.type).toEqual(_index.CREATING);
          });
          it('then the item\'s progressUp is updated with the current values', function () {
            expect(this.userStatus.progressUp).toEqual({
              percent: 100,
              loaded: 24,
              total: 24,
              lengthComputable: true
            });
            expect(this.userStatus.progressDown).toEqual({
              percent: 0,
              loaded: 0,
              total: undefined,
              lengthComputable: undefined
            });
          });
          describe('and the response is still downloading', function () {
            beforeAll(function () {
              this.xhrMock.setDownloadProgress({
                lengthComputable: true,
                loaded: 13,
                total: 25
              });
              this.userStatus = this.store.getState().users.items.temp.status;
            });
            it('then the item\'s status is still CREATING', function () {
              expect(this.userStatus.type).toEqual(_index.CREATING);
            });
            it('then the item\'s progressUp is updated with the current values', function () {
              expect(this.userStatus.progressUp).toEqual({
                percent: 100,
                loaded: 24,
                total: 24,
                lengthComputable: true
              });
              expect(this.userStatus.progressDown).toEqual({
                percent: 52,
                loaded: 13,
                total: 25,
                lengthComputable: true
              });
            });
          });
          describe('and the response has finished downloading', function () {
            beforeAll(function () {
              var _this = this;

              return this.xhrMock.completeDownload().then(function () {
                _this.userStatus = _this.store.getState().users.items['1'].status;
              });
            });
            it('then the item\'s status type is set to SUCCESS', function () {
              (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, '1', 'type', _index.SUCCESS);
            });
            it('then the item\'s progressUp status is updated with the current values', function () {
              expect(this.store.getState().users.items['1'].status.progressUp).toEqual({
                percent: 100,
                loaded: 24,
                total: 24,
                lengthComputable: true
              });
              expect(this.store.getState().users.items['1'].status.progressDown).toEqual({
                percent: 100,
                loaded: 25,
                total: 25,
                lengthComputable: true
              });
            });
          });
        });
      });
    });
  });
  describe('for the update action:', function () {
    describe('when it is NOT set to a truthy value', function () {
      beforeAll(function () {
        var _resources3 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users/:id',
          keyBy: 'id'
        }, {
          updateItem: true
        }),
            reducers = _resources3.reducers,
            updateUser = _resources3.actionCreators.updateItem;

        this.reducers = reducers;
        this.updateUser = updateUser;
      });
      describe('and the API request succeeds', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            _fetchMock["default"].put('http://test.com/users/1', new Promise(_nop["default"]));

            this.store = (0, _buildStore["default"])({
              users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
                items: {
                  1: {
                    values: {
                      id: 1,
                      username: 'Bob'
                    },
                    status: {
                      type: _index.SUCCESS
                    }
                  }
                }
              })
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.updateUser(1, {
              id: 1,
              username: 'Bob'
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the item\'s status is UPDATING', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, '1', 'type', _index.UPDATING);
          });
          it('then there are no progress attributes on the item\'s status object', function () {
            var userStatus = this.store.getState().users.items['1'].status;
            expect(userStatus.progressUp).toEqual(undefined);
            expect(userStatus.progressDown).toEqual(undefined);
          });
        });
        describe('when the request completes', function () {
          beforeAll(function () {
            _fetchMock["default"].put('http://test.com/users/1', {
              body: {
                id: 1,
                username: 'Robert'
              }
            });

            this.store = (0, _buildStore["default"])({
              users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
                items: {
                  1: {
                    values: {
                      id: 1,
                      username: 'Bob'
                    },
                    status: {
                      type: _index.SUCCESS
                    }
                  }
                }
              })
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.updateUser(1, {
              id: 1,
              username: 'Bob'
            }));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the item\'s status is SUCCESS', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, '1', 'type', _index.SUCCESS);
          });
          it('then does not set the item\'s progress status attributes', function () {
            var userStatus = this.store.getState().users.items['1'].status;
            expect(userStatus.progressUp).toEqual(undefined);
            expect(userStatus.progressDown).toEqual(undefined);
          });
        });
      });
    });
    describe('when it is set to a truthy value', function () {
      beforeAll(function () {
        var _resources4 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users/:id',
          keyBy: 'id'
        }, {
          updateItem: {
            progress: true
          }
        }),
            reducers = _resources4.reducers,
            updateUser = _resources4.actionCreators.updateItem;

        this.reducers = reducers;
        this.updateUser = updateUser;

        _XHRMock["default"].setup();
      });
      afterAll(function () {
        _XHRMock["default"].tearDown();
      });
      describe('and the API request succeeds', function () {
        beforeAll(function () {
          this.xhrMock = _XHRMock["default"].put('http://test.com/users/1', {
            body: {
              id: 1,
              username: 'Bob'
            }
          });
          this.store = (0, _buildStore["default"])({
            users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
              items: {
                1: {
                  values: {
                    id: 1,
                    username: 'Bob'
                  },
                  status: {
                    type: _index.SUCCESS
                  }
                }
              }
            })
          }, {
            users: this.reducers
          });
          this.store.dispatch(this.updateUser(1, {
            id: 1,
            username: 'Robert'
          }));
          this.userStatus = this.store.getState().users.items['1'].status;
        });
        afterAll(function () {
          _XHRMock["default"].reset();

          this.store = null;
        });
        describe('before the request has completed', function () {
          it('then the item\'s status is UPDATING', function () {
            expect(this.userStatus.type).toEqual(_index.UPDATING);
          });
          it('then there are no progress attributes on the item\'s status object', function () {
            expect(this.userStatus.progressUp).toEqual(undefined);
            expect(this.userStatus.progressDown).toEqual(undefined);
          });
        });
        describe('and the request progresses', function () {
          beforeAll(function () {
            this.xhrMock.setUploadProgress({
              lengthComputable: true,
              loaded: 12,
              total: 24
            });
            this.userStatus = this.store.getState().users.items['1'].status;
          });
          it('then the item\'s status is still UPDATING', function () {
            expect(this.userStatus.type).toEqual(_index.UPDATING);
          });
          it('then the item\'s progressUp status is updated with the current values', function () {
            expect(this.userStatus.progressUp).toEqual({
              percent: 50,
              loaded: 12,
              total: 24,
              lengthComputable: true
            });
            expect(this.userStatus.progressDown).toEqual({
              percent: 0,
              loaded: 0,
              total: undefined,
              lengthComputable: undefined
            });
          });
        });
        describe('and the upload has finished', function () {
          beforeAll(function () {
            this.xhrMock.completeUpload({
              status: 200
            });
            this.userStatus = this.store.getState().users.items['1'].status;
          });
          it('then the item\'s status is still UPDATING', function () {
            expect(this.userStatus.type).toEqual(_index.UPDATING);
          });
          it('then the item\'s progressUp is updated with the current values', function () {
            expect(this.userStatus.progressUp).toEqual({
              percent: 100,
              loaded: 36,
              total: 36,
              lengthComputable: true
            });
            expect(this.userStatus.progressDown).toEqual({
              percent: 0,
              loaded: 0,
              total: undefined,
              lengthComputable: undefined
            });
          });
          describe('and the response is downloading', function () {
            beforeAll(function () {
              this.xhrMock.setDownloadProgress({
                lengthComputable: true,
                loaded: 13,
                total: 25
              });
              this.userStatus = this.store.getState().users.items['1'].status;
            });
            it('then the item\'s status is still UPDATING', function () {
              expect(this.userStatus.type).toEqual(_index.UPDATING);
            });
            it('then the item\'s progressUp is updated with the current values', function () {
              expect(this.userStatus.progressUp).toEqual({
                percent: 100,
                loaded: 36,
                total: 36,
                lengthComputable: true
              });
              expect(this.userStatus.progressDown).toEqual({
                percent: 52,
                loaded: 13,
                total: 25,
                lengthComputable: true
              });
            });
            describe('and downloading the response is complete', function () {
              beforeAll(function () {
                var _this2 = this;

                return this.xhrMock.completeDownload().then(function () {
                  _this2.userStatus = _this2.store.getState().users.items['1'].status;
                });
              });
              it('then the item\'s status type is set to SUCCESS', function () {
                expect(this.userStatus.type).toEqual(_index.SUCCESS);
              });
              it('then the item\'s progressUp status is updated with the current values', function () {
                expect(this.userStatus.progressUp).toEqual({
                  percent: 100,
                  loaded: 36,
                  total: 36,
                  lengthComputable: true
                });
                expect(this.userStatus.progressDown).toEqual({
                  percent: 100,
                  loaded: 25,
                  total: 25,
                  lengthComputable: true
                });
              });
            });
          });
        });
      });
    });
  });
  describe('for the fetchItem action:', function () {
    describe('when it is NOT set to a truthy value', function () {
      beforeAll(function () {
        var _resources5 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users/:id',
          keyBy: 'id'
        }, {
          fetchItem: true
        }),
            reducers = _resources5.reducers,
            fetchUser = _resources5.actionCreators.fetchItem;

        this.reducers = reducers;
        this.fetchUser = fetchUser;
      });
      describe('and the API request succeeds', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users/1', new Promise(_nop["default"]));

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUser(1));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the item\'s status is FETCHING', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, '1', 'type', _index.FETCHING);
          });
          it('then there are no progress attributes on the item\'s status object', function () {
            var userStatus = this.store.getState().users.items['1'].status;
            expect(userStatus.progressUp).toEqual(undefined);
            expect(userStatus.progressDown).toEqual(undefined);
          });
        });
        describe('when the request completes', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users/1', {
              body: {
                id: 1,
                username: 'Robert'
              }
            });

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUser(1));
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the item\'s status is SUCCESS', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, '1', 'type', _index.SUCCESS);
          });
          it('then does not set the item\'s progress status attributes', function () {
            var userStatus = this.store.getState().users.items['1'].status;
            expect(userStatus.progressUp).toEqual(undefined);
            expect(userStatus.progressDown).toEqual(undefined);
          });
        });
      });
    });
    describe('when it is set to a truthy value', function () {
      beforeAll(function () {
        var _resources6 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users/:id',
          keyBy: 'id'
        }, {
          fetchItem: {
            progress: true
          }
        }),
            reducers = _resources6.reducers,
            fetchUser = _resources6.actionCreators.fetchItem;

        this.reducers = reducers;
        this.fetchUser = fetchUser;

        _XHRMock["default"].setup();
      });
      afterAll(function () {
        _XHRMock["default"].tearDown();
      });
      describe('and the API request succeeds', function () {
        beforeAll(function () {
          this.xhrMock = _XHRMock["default"].get('http://test.com/users/1', {
            body: {
              id: 1,
              username: 'Bob'
            }
          });
          this.store = (0, _buildStore["default"])({
            users: _objectSpread({}, _index.RESOURCES)
          }, {
            users: this.reducers
          });
          this.store.dispatch(this.fetchUser(1));
          this.userStatus = this.store.getState().users.items['1'].status;
        });
        afterAll(function () {
          _XHRMock["default"].reset();
        });
        describe('before the request has completed', function () {
          it('then the item\'s status is FETCHING', function () {
            expect(this.userStatus.type).toEqual(_index.FETCHING);
          });
          it('then there are no progress attributes on the item\'s status object', function () {
            expect(this.userStatus.progressUp).toEqual(undefined);
            expect(this.userStatus.progressDown).toEqual(undefined);
          });
        });
        describe('and the request progresses', function () {
          beforeAll(function () {
            this.xhrMock.setUploadProgress({
              lengthComputable: true,
              loaded: 12,
              total: 24
            });
            this.userStatus = this.store.getState().users.items['1'].status;
          });
          it('then the item\'s status is still FETCHING', function () {
            expect(this.userStatus.type).toEqual(_index.FETCHING);
          });
          it('then the item\'s progressUp status is updated with the current values', function () {
            expect(this.userStatus.progressUp).toEqual({
              percent: 50,
              loaded: 12,
              total: 24,
              lengthComputable: true
            });
            expect(this.userStatus.progressDown).toEqual({
              percent: 0,
              loaded: 0,
              total: undefined,
              lengthComputable: undefined
            });
          });
        });
        describe('and the upload has finished', function () {
          beforeAll(function () {
            this.xhrMock.completeUpload({
              status: 200
            });
            this.userStatus = this.store.getState().users.items['1'].status;
          });
          it('then the item\'s status is still FETCHING', function () {
            expect(this.userStatus.type).toEqual(_index.FETCHING);
          });
          it('then the item\'s progressUp is updated with the current values', function () {
            expect(this.userStatus.progressUp).toEqual({
              percent: 100,
              loaded: 0,
              total: 0,
              lengthComputable: true
            });
            expect(this.userStatus.progressDown).toEqual({
              percent: 0,
              loaded: 0,
              total: undefined,
              lengthComputable: undefined
            });
          });
          describe('and the response is downloading', function () {
            beforeAll(function () {
              this.xhrMock.setDownloadProgress({
                lengthComputable: true,
                loaded: 13,
                total: 25
              });
              this.userStatus = this.store.getState().users.items['1'].status;
            });
            it('then the item\'s status is still FETCHING', function () {
              expect(this.userStatus.type).toEqual(_index.FETCHING);
            });
            it('then the item\'s progressUp is updated with the current values', function () {
              expect(this.userStatus.progressUp).toEqual({
                percent: 100,
                loaded: 0,
                total: 0,
                lengthComputable: true
              });
              expect(this.userStatus.progressDown).toEqual({
                percent: 52,
                loaded: 13,
                total: 25,
                lengthComputable: true
              });
            });
            describe('and downloading the response is complete', function () {
              beforeAll(function () {
                var _this3 = this;

                return this.xhrMock.completeDownload().then(function () {
                  _this3.userStatus = _this3.store.getState().users.items['1'].status;
                });
              });
              it('then the item\'s status type is set to SUCCESS', function () {
                expect(this.userStatus.type).toEqual(_index.SUCCESS);
              });
              it('then the item\'s progressUp status is updated with the current values', function () {
                expect(this.userStatus.progressUp).toEqual({
                  percent: 100,
                  loaded: 0,
                  total: 0,
                  lengthComputable: true
                });
                expect(this.userStatus.progressDown).toEqual({
                  percent: 100,
                  loaded: 25,
                  total: 25,
                  lengthComputable: true
                });
              });
            });
          });
        });
      });
    });
  });
  describe('for the fetchList action:', function () {
    describe('when it is NOT set to a truthy value', function () {
      beforeAll(function () {
        var _resources7 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users/:id?',
          keyBy: 'id'
        }, {
          fetchList: true
        }),
            reducers = _resources7.reducers,
            fetchUsers = _resources7.actionCreators.fetchList;

        this.reducers = reducers;
        this.fetchUsers = fetchUsers;
      });
      describe('and the API request succeeds', function () {
        describe('before the request has completed', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users', new Promise(_nop["default"]));

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUsers());
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the list\'s status is FETCHING', function () {
            expect(this.store.getState().users.lists[_EmptyKey["default"]].status.type).toEqual(_index.FETCHING);
          });
          it('then there are no progress attributes on the list\'s status object', function () {
            var userStatus = this.store.getState().users.lists[_EmptyKey["default"]].status;

            expect(userStatus.progressUp).toEqual(undefined);
            expect(userStatus.progressDown).toEqual(undefined);
          });
        });
        describe('when the request completes', function () {
          beforeAll(function () {
            _fetchMock["default"].get('http://test.com/users', {
              body: [{
                id: 1,
                username: 'Robert'
              }]
            });

            this.store = (0, _buildStore["default"])({
              users: _objectSpread({}, _index.RESOURCES)
            }, {
              users: this.reducers
            });
            this.store.dispatch(this.fetchUsers());
          });
          afterAll(function () {
            _fetchMock["default"].restore();

            this.store = null;
          });
          it('then the list\'s status is SUCCESS', function () {
            expect(this.store.getState().users.lists[_EmptyKey["default"]].status.type).toEqual(_index.SUCCESS);
          });
          it('then the list\'s items\' status is SUCCESS', function () {
            (0, _resourceAssertions.expectToChangeResourcesItemStatusTo)(this, RESOURCE_NAME, '1', 'type', _index.SUCCESS);
          });
          it('then does not set the list\'s progress status attributes', function () {
            var listStatus = this.store.getState().users.lists[_EmptyKey["default"]].status;

            expect(listStatus.progressUp).toEqual(undefined);
            expect(listStatus.progressDown).toEqual(undefined);
          });
          it('then does not set then the list\'s items\' progress status attributes', function () {
            var userStatus = this.store.getState().users.items['1'].status;
            expect(userStatus.progressUp).toEqual(undefined);
            expect(userStatus.progressDown).toEqual(undefined);
          });
        });
      });
    });
    describe('when it is set to a truthy value', function () {
      beforeAll(function () {
        var _resources8 = (0, _index.resources)({
          name: 'users',
          url: 'http://test.com/users/:id?',
          keyBy: 'id'
        }, {
          fetchList: {
            progress: true
          }
        }),
            reducers = _resources8.reducers,
            fetchUsers = _resources8.actionCreators.fetchList;

        this.reducers = reducers;
        this.fetchUsers = fetchUsers;

        _XHRMock["default"].setup();
      });
      afterAll(function () {
        _XHRMock["default"].tearDown();
      });
      describe('and the API request succeeds', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])({
            users: _objectSpread({}, _index.RESOURCES)
          }, {
            users: this.reducers
          });
          this.xhrMock = _XHRMock["default"].get('http://test.com/users', {
            body: [{
              id: 1,
              username: 'Bob'
            }]
          });
          this.store.dispatch(this.fetchUsers());
          this.userListStatus = this.store.getState().users.lists[_EmptyKey["default"]].status;
        });
        afterAll(function () {
          _XHRMock["default"].reset();
        });
        describe('before the request has completed', function () {
          it('then the list\'s status is FETCHING', function () {
            expect(this.userListStatus.type).toEqual(_index.FETCHING);
          });
          it('then there are no progress attributes on the list\'s status object', function () {
            expect(this.userListStatus.progressUp).toEqual(undefined);
            expect(this.userListStatus.progressDown).toEqual(undefined);
          });
        });
        describe('and the request progresses', function () {
          beforeAll(function () {
            this.xhrMock.setUploadProgress({
              lengthComputable: true,
              loaded: 12,
              total: 24
            });
            this.userListStatus = this.store.getState().users.lists[_EmptyKey["default"]].status;
          });
          it('then the list\'s status is still FETCHING', function () {
            expect(this.userListStatus.type).toEqual(_index.FETCHING);
          });
          it('then the list\'s progressUp status is updated with the current values', function () {
            expect(this.userListStatus.progressUp).toEqual({
              percent: 50,
              loaded: 12,
              total: 24,
              lengthComputable: true
            });
            expect(this.userListStatus.progressDown).toEqual({
              percent: 0,
              loaded: 0,
              total: undefined,
              lengthComputable: undefined
            });
          });
        });
        describe('and the upload has finished', function () {
          beforeAll(function () {
            this.xhrMock.completeUpload({
              status: 200
            });
            this.userListStatus = this.store.getState().users.lists[_EmptyKey["default"]].status;
          });
          it('then the list\'s status is still FETCHING', function () {
            expect(this.userListStatus.type).toEqual(_index.FETCHING);
          });
          it('then the list\'s progressUp is updated with the current values', function () {
            expect(this.userListStatus.progressUp).toEqual({
              percent: 100,
              loaded: 0,
              total: 0,
              lengthComputable: true
            });
            expect(this.userListStatus.progressDown).toEqual({
              percent: 0,
              loaded: 0,
              total: undefined,
              lengthComputable: undefined
            });
          });
          describe('and the response is downloading', function () {
            beforeAll(function () {
              this.xhrMock.setDownloadProgress({
                lengthComputable: true,
                loaded: 13,
                total: 25
              });
              this.userListStatus = this.store.getState().users.lists[_EmptyKey["default"]].status;
            });
            it('then the list\'s status is still FETCHING', function () {
              expect(this.userListStatus.type).toEqual(_index.FETCHING);
            });
            it('then the list\'s progressUp is updated with the current values', function () {
              expect(this.userListStatus.progressUp).toEqual({
                percent: 100,
                loaded: 0,
                total: 0,
                lengthComputable: true
              });
              expect(this.userListStatus.progressDown).toEqual({
                percent: 52,
                loaded: 13,
                total: 25,
                lengthComputable: true
              });
            });
            describe('and downloading the response is complete', function () {
              beforeAll(function () {
                var _this4 = this;

                return this.xhrMock.completeDownload().then(function () {
                  _this4.userListStatus = _this4.store.getState().users.lists[_EmptyKey["default"]].status;
                  _this4.userStatus = _this4.store.getState().users.items['1'].status;
                });
              });
              it('then the list\'s status type is set to SUCCESS', function () {
                expect(this.userListStatus.type).toEqual(_index.SUCCESS);
              });
              it('then the list\'s items\' status is SUCCESS', function () {
                expect(this.userStatus.type).toEqual(_index.SUCCESS);
              });
              it('then the list\'s progressUp status is updated with the current values', function () {
                expect(this.userListStatus.progressUp).toEqual({
                  percent: 100,
                  loaded: 0,
                  total: 0,
                  lengthComputable: true
                });
                expect(this.userListStatus.progressDown).toEqual({
                  percent: 100,
                  loaded: 25,
                  total: 25,
                  lengthComputable: true
                });
              });
              it('then does not set then the list\'s items\' progress status attributes', function () {
                expect(this.userStatus.progressUp).toEqual(undefined);
                expect(this.userStatus.progressDown).toEqual(undefined);
              });
            });
          });
        });
      });
    });
  });
});