"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../index");

var _buildStore = _interopRequireDefault(require("../helpers/buildStore"));

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

var _resourceAssertions = require("../helpers/resourceAssertions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var RESOURCE_NAME = 'users';
describe('reducesOn:', function () {
  beforeAll(function () {
    this.initialState = {
      users: {
        items: {
          1: {
            values: {
              id: 1,
              username: 'Bob',
              signedIn: true
            },
            status: {
              type: _index.SUCCESS
            }
          }
        },
        lists: _defineProperty({}, _EmptyKey["default"], {
          positions: [1],
          status: {
            type: _index.SUCCESS
          }
        })
      },
      session: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
        items: {
          1: {
            values: {
              active: true
            },
            status: {
              type: _index.SUCCESS
            }
          }
        }
      })
    };

    var _resources = (0, _index.resources)({
      name: 'session',
      url: 'http://test.com/session/:id'
    }, {
      destroyItem: true
    }),
        sessionReducers = _resources.reducers,
        destroySession = _resources.actionCreators.destroyItem,
        actions = _resources.actions;

    this.sessionReducers = sessionReducers;
    this.destroySession = destroySession;
    this.sessionActions = actions;

    _fetchMock["default"]["delete"]('http://test.com/session/1', {
      body: {
        status: 200,
        response: {}
      },
      status: 200
    });

    var _resources2 = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id',
      reducesOn: _defineProperty({}, this.sessionActions.destroyItem, function (users, _ref, _ref2) {
        var key = _ref.key;
        var mergeItemValues = _ref2.mergeItemValues;
        return mergeItemValues(users, key, {
          signedIn: false
        });
      })
    }, {
      fetchList: true,
      newItem: true
    }),
        usersReducers = _resources2.reducers;

    this.store = (0, _buildStore["default"])(_objectSpread({}, this.initialState), {
      users: usersReducers,
      session: this.sessionReducers
    });
  });
  afterAll(function () {
    _fetchMock["default"].restore();
  });
  it('calls the specified reducer when the specified action occurs', function () {
    var _this = this;

    (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(this, RESOURCE_NAME, '1', 'signedIn', true);
    return this.store.dispatch(this.destroySession(1)).then(function () {
      (0, _resourceAssertions.expectToChangeResourcesItemValuesTo)(_this, RESOURCE_NAME, '1', 'signedIn', false);
    });
  });
});