"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../index");

var _buildStore = _interopRequireDefault(require("../helpers/buildStore"));

var _nop = _interopRequireDefault(require("../../utils/function/nop"));

var _ResourceRegistry = _interopRequireDefault(require("../../utils/ResourceRegistry"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('Request options:', function () {
  afterEach(function () {
    (0, _index.configure)({});
  });
  beforeAll(function () {
    this.originalHeaders = {
      'Content-Type': 'application/json+custom',
      'Accept': 'application/json+custom'
    };
  });
  afterEach(function () {
    _fetchMock["default"].restore();

    this.store = null;

    _ResourceRegistry["default"].getRegistry().clear();
  });
  describe('Given the request options have been globally configured BEFORE a resource has been defined', function () {
    beforeEach(function () {
      (0, _index.configure)({
        request: {
          headers: this.originalHeaders
        }
      });
      defineResources(this);
    });
    describe('when an action creator is called', function () {
      it('then uses the request options', function () {
        checkRequestIsMadeWithHeaders(this.originalHeaders, this);
      });
    });
  });
  describe('Given the request options have been globally configured AFTER a resource has been defined', function () {
    beforeEach(function () {
      defineResources(this);
      (0, _index.configure)({
        request: {
          headers: this.originalHeaders
        }
      });
    });
    describe('when an action creator is called', function () {
      it('then uses the request options', function () {
        checkRequestIsMadeWithHeaders(this.originalHeaders, this);
      });
    });
  });
  describe('Given the request options have been globally configured at runtime', function () {
    beforeEach(function () {
      (0, _index.configure)({
        request: {
          headers: this.originalHeaders
        }
      });
      defineResources(this);
      var configuration = (0, _index.getConfiguration)();
      this.runtimeHeaders = _objectSpread(_objectSpread({}, configuration.request.headers), {}, {
        'Authorization': 'Token token="MY-TOKEN"'
      });
      (0, _index.configure)({
        request: {
          headers: this.runtimeHeaders
        }
      });
    });
    describe('when an action creator is called', function () {
      it('then uses the request options', function () {
        checkRequestIsMadeWithHeaders(this.runtimeHeaders, this);
      });
    });
  });

  function defineResources(context) {
    var _resources = (0, _index.resources)({
      name: 'users',
      url: 'http://test.com/users/:id?',
      keyBy: 'id'
    }, ['fetchList']),
        reducers = _resources.reducers,
        fetchUsers = _resources.actionCreators.fetchList;

    context.fetchUsers = fetchUsers;
    context.reducers = reducers;
  }

  function checkRequestIsMadeWithHeaders(headers, context) {
    _fetchMock["default"].get('http://test.com/users', new Promise(_nop["default"]), {
      headers: headers
    });

    context.store = (0, _buildStore["default"])({
      users: context.resourceBefore
    }, {
      users: context.reducers
    });
    context.store.dispatch(context.fetchUsers());
  }
});