"use strict";

var _fetchMock = _interopRequireDefault(require("fetch-mock"));

var _index = require("../../index");

var _buildStore = _interopRequireDefault(require("../helpers/buildStore"));

var _EmptyKey = _interopRequireDefault(require("../../constants/EmptyKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('Specifying a response adaptor:', function () {
  describe('Specifying a response adaptor for all actions', function () {
    beforeAll(function () {
      var _resources = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id',
        responseAdaptor: function responseAdaptor(body) {
          return {
            values: body.response,
            error: body.error
          };
        }
      }, {
        fetchList: true,
        fetchItem: true,
        updateItem: true,
        destroyItem: true,
        createItem: true
      }),
          reducers = _resources.reducers,
          _resources$actionCrea = _resources.actionCreators,
          createUser = _resources$actionCrea.createItem,
          fetchUsers = _resources$actionCrea.fetchList,
          fetchUser = _resources$actionCrea.fetchItem,
          updateUser = _resources$actionCrea.updateItem,
          destroyUser = _resources$actionCrea.destroyItem;

      this.reducers = reducers;
      this.createUser = createUser;
      this.fetchUsers = fetchUsers;
      this.fetchUser = fetchUser;
      this.updateUser = updateUser;
      this.destroyUser = destroyUser;
    });
    describe('when the fetchList request', function () {
      describe('succeeds', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])({
            users: _index.RESOURCES
          }, {
            users: this.reducers
          });

          _fetchMock["default"].get('http://test.com/users', {
            body: {
              status: 200,
              response: [{
                id: 1,
                username: 'Bob'
              }]
            },
            status: 200
          });
        });
        afterAll(function () {
          _fetchMock["default"].restore();
        });
        it('then uses the response adaptor', function () {
          this.store.dispatch(this.fetchUsers());
        });
      });
      describe('errors', function () {
        describe('with a JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _index.RESOURCES
            }, {
              users: this.reducers
            });

            _fetchMock["default"].get('http://test.com/users', {
              body: {
                status: 404,
                error: 'NOT_FOUND'
              },
              status: 404
            });
          });
          afterAll(function () {
            _fetchMock["default"].restore();
          });
          it('then uses the response adaptor', function () {
            var _this = this;

            return this.store.dispatch(this.fetchUsers()).then(function () {
              var list = _this.store.getState().users.lists[_EmptyKey["default"]];

              expect(list.status.type).toEqual(_index.ERROR);
              expect(list.status.error.message).toEqual('NOT_FOUND');
              expect(list.status.httpCode).toEqual(404);
            });
          });
        });
        describe('with a non-JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _index.RESOURCES
            }, {
              users: this.reducers
            });

            _fetchMock["default"].get('http://test.com/users', {
              body: 'Long error stack trace',
              headers: {
                'Content-Type': 'text/html'
              },
              status: 500
            });
          });
          afterAll(function () {
            _fetchMock["default"].restore();
          });
          it('then does not use the response adaptor', function () {
            var _this2 = this;

            return this.store.dispatch(this.fetchUsers()).then(function () {
              var list = _this2.store.getState().users.lists[_EmptyKey["default"]];

              expect(list.status.type).toEqual(_index.ERROR);
              expect(list.status.error.message).toEqual('Long error stack trace');
              expect(list.status.httpCode).toEqual(500);
            });
          });
        });
      });
    });
    describe('when the fetchItem request', function () {
      describe('succeeds', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])({
            users: _index.RESOURCES
          }, {
            users: this.reducers
          });

          _fetchMock["default"].get('http://test.com/users/1', {
            body: {
              status: 200,
              response: {
                id: 1,
                username: 'Bob'
              }
            },
            status: 200
          });
        });
        afterAll(function () {
          _fetchMock["default"].restore();
        });
        it('then uses the response adaptor', function () {
          var _this3 = this;

          return this.store.dispatch(this.fetchUser(1)).then(function () {
            var item = _this3.store.getState().users.items['1'];

            expect(item.status.type).toEqual(_index.SUCCESS);
            expect(item.values).toEqual({
              id: 1,
              username: 'Bob'
            });
          });
        });
      });
      describe('errors', function () {
        describe('with a JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _index.RESOURCES
            }, {
              users: this.reducers
            });

            _fetchMock["default"].get('http://test.com/users/1', {
              body: {
                status: 404,
                error: 'NOT_FOUND'
              },
              status: 404
            });
          });
          afterAll(function () {
            return _fetchMock["default"].restore();
          });
          it('then uses the response adaptor', function () {
            var _this4 = this;

            return this.store.dispatch(this.fetchUser(1)).then(function () {
              var item = _this4.store.getState().users.items['1'];

              expect(item.status.type).toEqual(_index.ERROR);
              expect(item.status.error.message).toEqual('NOT_FOUND');
              expect(item.values).toEqual({});
            });
          });
        });
        describe('with a non-JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _index.RESOURCES
            }, {
              users: this.reducers
            });

            _fetchMock["default"].get('http://test.com/users/1', {
              body: 'Long error stack trace',
              headers: {
                'Content-Type': 'text/html'
              },
              status: 500
            });
          });
          afterAll(function () {
            _fetchMock["default"].restore();
          });
          it('then does not use the response adaptor', function () {
            var _this5 = this;

            return this.store.dispatch(this.fetchUser(1)).then(function () {
              var list = _this5.store.getState().users.items['1'];

              expect(list.status.type).toEqual(_index.ERROR);
              expect(list.status.error.message).toEqual('Long error stack trace');
              expect(list.status.httpCode).toEqual(500);
            });
          });
        });
      });
    });
    describe('when a create request', function () {
      describe('succeeds', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])({
            users: _index.RESOURCES
          }, {
            users: this.reducers
          });

          _fetchMock["default"].post('http://test.com/users', {
            body: {
              status: 200,
              response: {
                id: 1,
                username: 'Bob'
              }
            },
            status: 200
          });
        });
        afterAll(function () {
          _fetchMock["default"].restore();
        });
        it('then uses the response adaptor', function () {
          var _this6 = this;

          return this.store.dispatch(this.createUser('temp', {
            username: 'Bob'
          })).then(function () {
            var item = _this6.store.getState().users.items['1'];

            expect(item.status.type).toEqual(_index.SUCCESS);
            expect(item.values).toEqual({
              id: 1,
              username: 'Bob'
            });
          });
        });
      });
      describe('errors', function () {
        describe('with a JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _index.RESOURCES
            }, {
              users: this.reducers
            });

            _fetchMock["default"].post('http://test.com/users', {
              body: {
                status: 404,
                error: 'NOT_FOUND'
              },
              status: 404
            });
          });
          afterAll(function () {
            return _fetchMock["default"].restore();
          });
          it('then uses the response adaptor', function () {
            var _this7 = this;

            return this.store.dispatch(this.createUser('temp', {
              username: 'Bob'
            })).then(function () {
              var item = _this7.store.getState().users.items.temp;

              expect(item.status.type).toEqual(_index.ERROR);
              expect(item.status.error.message).toEqual('NOT_FOUND');
              expect(item.values).toEqual({
                username: 'Bob'
              });
            });
          });
        });
        describe('with a non-JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _index.RESOURCES
            }, {
              users: this.reducers
            });

            _fetchMock["default"].post('http://test.com/users', {
              body: 'Long error stack trace',
              headers: {
                'Content-Type': 'text/html'
              },
              status: 500
            });
          });
          afterAll(function () {
            _fetchMock["default"].restore();
          });
          it('then does not use the response adaptor', function () {
            var _this8 = this;

            return this.store.dispatch(this.createUser('temp', {
              username: 'Bob'
            })).then(function () {
              var item = _this8.store.getState().users.items.temp;

              expect(item.status.type).toEqual(_index.ERROR);
              expect(item.status.error.message).toEqual('Long error stack trace');
              expect(item.status.httpCode).toEqual(500);
            });
          });
        });
      });
    });
    describe('when an update request', function () {
      describe('succeeds', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])({
            users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
              items: {
                1: {
                  values: {
                    id: 1,
                    username: 'Bob'
                  },
                  status: {
                    type: _index.SUCCESS
                  }
                }
              }
            })
          }, {
            users: this.reducers
          });

          _fetchMock["default"].put('http://test.com/users/1', {
            body: {
              status: 200,
              response: {
                id: 1,
                username: 'Robert'
              }
            },
            status: 200
          });
        });
        afterAll(function () {
          _fetchMock["default"].restore();
        });
        it('then uses the response adaptor', function () {
          var _this9 = this;

          return this.store.dispatch(this.updateUser(1, {
            username: 'Robert'
          })).then(function () {
            var item = _this9.store.getState().users.items['1'];

            expect(item.status.type).toEqual(_index.SUCCESS);
            expect(item.values).toEqual({
              id: 1,
              username: 'Robert'
            });
          });
        });
      });
      describe('errors', function () {
        describe('with a JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
                items: {
                  1: {
                    values: {
                      id: 1,
                      username: 'Bob'
                    },
                    status: {
                      type: _index.SUCCESS
                    }
                  }
                }
              })
            }, {
              users: this.reducers
            });

            _fetchMock["default"].put('http://test.com/users/1', {
              body: {
                status: 404,
                error: 'NOT_FOUND'
              },
              status: 404
            });
          });
          afterAll(function () {
            return _fetchMock["default"].restore();
          });
          it('then uses the response adaptor', function () {
            var _this10 = this;

            return this.store.dispatch(this.updateUser(1, {
              username: 'Robert'
            })).then(function () {
              var item = _this10.store.getState().users.items['1'];

              expect(item.status.type).toEqual(_index.ERROR);
              expect(item.status.error.message).toEqual('NOT_FOUND');
              expect(item.values).toEqual({
                id: 1,
                username: 'Robert'
              });
            });
          });
        });
        describe('with a non-JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
                items: {
                  1: {
                    values: {
                      id: 1,
                      username: 'Bob'
                    },
                    status: {
                      type: _index.SUCCESS
                    }
                  }
                }
              })
            }, {
              users: this.reducers
            });

            _fetchMock["default"].put('http://test.com/users/1', {
              body: 'Long error stack trace',
              headers: {
                'Content-Type': 'text/html'
              },
              status: 500
            });
          });
          afterAll(function () {
            _fetchMock["default"].restore();
          });
          it('then does not use the response adaptor', function () {
            var _this11 = this;

            return this.store.dispatch(this.updateUser(1, {
              username: 'Robert'
            })).then(function () {
              var item = _this11.store.getState().users.items['1'];

              expect(item.status.type).toEqual(_index.ERROR);
              expect(item.status.error.message).toEqual('Long error stack trace');
              expect(item.status.httpCode).toEqual(500);
            });
          });
        });
      });
    });
    describe('when a destroyItem action', function () {
      describe('succeeds', function () {
        beforeAll(function () {
          this.store = (0, _buildStore["default"])({
            users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
              items: {
                1: {
                  values: {
                    id: 1,
                    username: 'Bob'
                  },
                  status: {
                    type: _index.SUCCESS
                  }
                }
              }
            })
          }, {
            users: this.reducers
          });

          _fetchMock["default"]["delete"]('http://test.com/users/1', {
            body: {
              status: 200,
              response: {}
            },
            status: 200
          });
        });
        afterAll(function () {
          _fetchMock["default"].restore();
        });
        it('then uses the response adaptor', function () {
          var _this12 = this;

          return this.store.dispatch(this.destroyUser(1)).then(function () {
            var item = _this12.store.getState().users.items['1'];

            expect(item).toEqual(undefined);
          });
        });
      });
      describe('errors', function () {
        describe('with a JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
                items: {
                  1: {
                    values: {
                      id: 1,
                      username: 'Bob'
                    },
                    status: {
                      type: _index.SUCCESS
                    }
                  }
                }
              })
            }, {
              users: this.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/users/1', {
              body: {
                status: 404,
                error: 'NOT_FOUND'
              },
              status: 404
            });
          });
          afterAll(function () {
            return _fetchMock["default"].restore();
          });
          it('then uses the response adaptor', function () {
            var _this13 = this;

            return this.store.dispatch(this.destroyUser(1)).then(function () {
              var item = _this13.store.getState().users.items['1'];

              expect(item.status.type).toEqual(_index.DESTROY_ERROR);
              expect(item.status.error.message).toEqual('NOT_FOUND');
              expect(item.values).toEqual({
                id: 1,
                username: 'Bob'
              });
            });
          });
        });
        describe('with a non-JSON response', function () {
          beforeAll(function () {
            this.store = (0, _buildStore["default"])({
              users: _objectSpread(_objectSpread({}, _index.RESOURCES), {}, {
                items: {
                  1: {
                    values: {
                      id: 1,
                      username: 'Bob'
                    },
                    status: {
                      type: _index.SUCCESS
                    }
                  }
                }
              })
            }, {
              users: this.reducers
            });

            _fetchMock["default"]["delete"]('http://test.com/users/1', {
              body: 'Long error stack trace',
              headers: {
                'Content-Type': 'text/html'
              },
              status: 500
            });
          });
          afterAll(function () {
            _fetchMock["default"].restore();
          });
          it('then does not use the response adaptor', function () {
            var _this14 = this;

            return this.store.dispatch(this.destroyUser(1)).then(function () {
              var item = _this14.store.getState().users.items['1'];

              expect(item.status.type).toEqual(_index.DESTROY_ERROR);
              expect(item.status.error.message).toEqual('Long error stack trace');
              expect(item.status.httpCode).toEqual(500);
            });
          });
        });
      });
    });
  });
  describe('when the default is overridden for a particular action', function () {
    beforeAll(function () {
      var _resources2 = (0, _index.resources)({
        name: 'users',
        url: 'http://test.com/users/:id?',
        keyBy: 'id',
        responseAdaptor: function responseAdaptor(body) {
          return {
            values: body.response,
            error: body.error
          };
        }
      }, {
        fetchList: {
          responseAdaptor: function responseAdaptor(body) {
            return {
              values: body.items,
              error: body.error
            };
          }
        },
        fetchItem: true
      }),
          reducers = _resources2.reducers,
          _resources2$actionCre = _resources2.actionCreators,
          fetchUsers = _resources2$actionCre.fetchList,
          fetchUser = _resources2$actionCre.fetchItem;

      this.fetchUsers = fetchUsers;
      this.fetchUser = fetchUser;
      this.store = (0, _buildStore["default"])({
        users: _index.RESOURCES
      }, {
        users: reducers
      });
    });
    it('then uses the response adaptor for that action', function () {
      _fetchMock["default"].get('http://test.com/users', {
        body: {
          status: 200,
          items: [{
            id: 1,
            username: 'Bob'
          }]
        },
        status: 200
      });

      this.store.dispatch(this.fetchUsers());

      _fetchMock["default"].restore();
    });
    it('then uses the default response adaptor for other actions', function () {
      _fetchMock["default"].get('http://test.com/users/1', {
        body: {
          id: 1,
          username: 'Bob'
        },
        status: 200
      });

      this.store.dispatch(this.fetchUser(1));

      _fetchMock["default"].restore();
    });
  });
});