"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _assertInDevMode = _interopRequireDefault(require("./assertInDevMode"));

var _warn = _interopRequireDefault(require("./dev/warn"));

var _standardiseAssociationOptions = _interopRequireDefault(require("./standardiseAssociationOptions"));

var _addAsssociationReducer = _interopRequireDefault(require("../reducers/helpers/addAsssociationReducer"));

var _toPlural = _interopRequireDefault(require("./string/toPlural"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var registry = null;

var ResourceRegistry = function () {
  _createClass(ResourceRegistry, null, [{
    key: "getRegistry",
    value: function getRegistry() {
      if (!registry) {
        registry = new ResourceRegistry();
      }

      return registry;
    }
  }]);

  function ResourceRegistry() {
    _classCallCheck(this, ResourceRegistry);

    this.clear();
  }

  _createClass(ResourceRegistry, [{
    key: "addResource",
    value: function addResource(options, definition, reducersDictionary) {
      var _this = this;

      var name = options.name;
      (0, _assertInDevMode["default"])(function () {
        if (_this.getResource(name)) {
          (0, _warn["default"])("Overriding resource with name '".concat(name, "', which has already been defined. This can result in unpredictable behaviour."));
        }
      });
      this.setResource(name, {
        options: options,
        definition: definition,
        reducersDictionary: reducersDictionary
      });
      ['belongsTo', 'hasAndBelongsToMany'].forEach(function (associationType) {
        var _associationOptions = options[associationType];

        if (_associationOptions) {
          var associationOptions = (0, _standardiseAssociationOptions["default"])(_associationOptions);
          Object.keys(associationOptions).forEach(function (associatedResourceName) {
            var resource = _this.getResource(associatedResourceName);

            if (resource) {
              var associatedResourcesActions = resource.definition.actions;
              (0, _addAsssociationReducer["default"])(reducersDictionary, name, associationType, associatedResourceName, associatedResourcesActions, associationOptions[associatedResourceName]);
            } else {
              if (!_this.getAssociation(associatedResourceName)) {
                _this.setAssociation(associatedResourceName, []);
              }

              _this.getAssociation(associatedResourceName).push({
                resourceName: name,
                associationType: associationType
              });
            }
          });
        }
      });
      var pendingAssociations = this.getAssociation(name);

      if (pendingAssociations) {
        pendingAssociations.forEach(function (_ref) {
          var resourceName = _ref.resourceName,
              associationType = _ref.associationType;

          var _this$getResource = _this.getResource(resourceName),
              _this$getResource$opt = _this$getResource.options,
              associatedResourceName = _this$getResource$opt.name,
              _this$getResource$opt2 = _this$getResource$opt[associationType][name],
              associationOptions = _this$getResource$opt2 === void 0 ? {} : _this$getResource$opt2,
              associatedResourceReducersDict = _this$getResource.reducersDictionary;

          (0, _addAsssociationReducer["default"])(associatedResourceReducersDict, associatedResourceName, associationType, options.name, definition.actions, associationOptions);
        });
        Reflect.deleteProperty(this.associations, name);
      }
    }
  }, {
    key: "getResource",
    value: function getResource(key) {
      return this.getKey(this.resources, key);
    }
  }, {
    key: "getAssociation",
    value: function getAssociation(key) {
      return this.getKey(this.associations, key);
    }
  }, {
    key: "setResource",
    value: function setResource(key, value) {
      this.setKey(this.resources, key, value);
    }
  }, {
    key: "setAssociation",
    value: function setAssociation(key, value) {
      this.setKey(this.associations, key, value);
    }
  }, {
    key: "getKey",
    value: function getKey(target, key) {
      return target[(0, _toPlural["default"])(key)];
    }
  }, {
    key: "setKey",
    value: function setKey(target, key, value) {
      target[(0, _toPlural["default"])(key)] = value;
    }
  }, {
    key: "clear",
    value: function clear() {
      this.resources = {};
      this.associations = {};
    }
  }]);

  return ResourceRegistry;
}();

var _default = ResourceRegistry;
exports["default"] = _default;