"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _configuration = require("../configuration");

var _assertInDevMode = _interopRequireDefault(require("./assertInDevMode"));

var _warn = _interopRequireDefault(require("./dev/warn"));

var _without = _interopRequireDefault(require("./list/without"));

var _isFunction = _interopRequireDefault(require("./object/isFunction"));

var _ActionQueue = require("./ActionQueue");

var _hasDefinedStatus = _interopRequireDefault(require("../public-helpers/hasDefinedStatus"));

var _Statuses = require("../constants/Statuses");

var _isUndefined = _interopRequireDefault(require("./isUndefined"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function getOrFetch(options, resourcesState) {
  var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var actionCreatorOptions = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var keyFunction = options.keyFunction,
      getFunction = options.getFunction,
      fetchFunction = options.fetchFunction,
      action = options.action,
      localOnly = options.localOnly;

  var _getConfiguration = (0, _configuration.getConfiguration)(),
      store = _getConfiguration.store;

  (0, _assertInDevMode["default"])(function () {
    if (!store) {
      (0, _warn["default"])('Cannot use getOrFetchItem() without setting the store instance using the configure() function. Falling back to returning an empty item.');
    }
  });
  var key = keyFunction(params);
  var itemOrList = getFunction(resourcesState, key);
  var forceAction = evaluateForceCondition(actionCreatorOptions.forceFetch, itemOrList);

  if (!localOnly && (!(0, _hasDefinedStatus["default"])(itemOrList) || forceAction)) {
    if (!(0, _ActionQueue.isActionPending)(action, key) || forceAction) {
      (0, _ActionQueue.enqueuePendingAction)(action, key);
      setTimeout(function () {
        var fetcher = fetchFunction(params, (0, _without["default"])(actionCreatorOptions, ['forceFetch']));

        if ((0, _isUndefined["default"])(fetcher)) {
          (0, _ActionQueue.registerActionEnd)(action, key);
        } else {
          fetcher.then(function () {
            return (0, _ActionQueue.registerActionEnd)(action, key);
          });
        }
      }, 0);
    }

    return _objectSpread(_objectSpread({}, itemOrList), {}, {
      status: {
        type: _Statuses.FETCHING
      }
    });
  }

  return itemOrList;
}

function evaluateForceCondition(forceFetch, itemOrList) {
  return (0, _isFunction["default"])(forceFetch) ? forceFetch(itemOrList) : Boolean(forceFetch);
}

var _default = getOrFetch;
exports["default"] = _default;